<?php

/**
 * @file
 * abstract class to import resources into archibald
 */

/**
 * Abstrat class form importing resources into dasb_client/archibald 2
 *
 * @author prdatur <c.ackermann@educa.ch>
 */
abstract class ArchibaldImport {

  /**
   * inserts lom object into archibald client
   *
   * @param Lom $lom_object
   *   the lom object
   *
   * @return boolean
   *   TRUE on success, else FALSE
   */
  public function import($lom_object) {
    return ArchibaldLomSaveHandler::insert($lom_object);
  }

  /**
   * read the xml struct if it is not present
   * this function will be called at the beginning
   * within getRowAt() function
   */
  abstract public function prepare();

  /**
   * generate the table contents for drupal table renderer
   * return array must be "identifier key" => "displayed value like title"
   *
   * @return array
   *   the elements
   */
  abstract public function getTableElements();

  /**
   * returns a parsed lom object at given row_id
   *
   * @param integer $row_index
   *   the row id
   * @param boolean $refaktor
   *   refactore after parsing?
   *
   * @return array
   *   the parsed array
   */
  abstract public function getRowAt($i, $refaktor = TRUE);

  /**
   * get mapped local lom_id by remote id
   * from import log
   *
   * @param string $plugin
   * @param string $remote_id
   *
   * @return string lom_id
   */
  protected function searchLomId($plugin, $remote_id) {
    $res = db_select('archibald_import_log', 'archibaldil')
      ->fields('archibaldil')
      ->condition('remote_id', $remote_id, '=')
      ->condition('plugin', $plugin, '=')
      ->execute()
      ->fetch();

    if (empty($res)) {
      return '';
    }
    return $res->lom_id;
  }

  /**
   * downloads a file and stores it into lokal drupal client
   *
   * @param string $url
   *   the source url
   * @param string $type
   *   type, if it not 'identifier' it will create a preview image,
   *   else its the main file
   *
   * @return string
   *   the internal lokal file uri
   */
  public function downloadFile($url, $type = 'identifier') {

    $filename = basename($url);

    $temp_file = '/tmp/temp_file_' . uniqid();

    // for preview images
    if ($type != 'identifier') {
      $destination = 'public://files/preview_image/';
    }
    // for identifier:
    else {
      $destination = 'public://files/archibald/';
    }
    // Begin building file object.
    $file = new stdClass();
    $file->filename = trim($filename, '.');

    $destination_scheme = file_uri_scheme($destination);
    $file->uri = file_destination(
      $destination . $file->filename, FILE_EXISTS_REPLACE
    );

    $file_tmp = db_query(
      'SELECT fid, filename FROM {file_managed} WHERE uri = :uri',
      array(':uri' => $file->uri)
    )->fetchObject();

    if (!empty($file_tmp->fid) && !empty($file_tmp->filename)) {
      if (file_exists($file->uri)) {
        return archibald_file_get_url($file_tmp);
      }
      else {
        $file->fid = $file_tmp->fid;
      }
    }

    file_put_contents($temp_file, file_get_contents($url));

    // admin
    $file->uid      = 1;
    $file->status   = FILE_STATUS_PERMANENT;
    $file->filemime = file_get_mimetype($temp_file);
    $file->filesize = filesize($temp_file);

    copy($temp_file, $file->uri);

    drupal_chmod($file->uri);

    $file = file_save($file);

    // for preview images
    if ($type != 'dentifier') {
      file_usage_add($file, 'archibald', 'preview_image', 1);
    }
    // for identifier:
    else {
      file_usage_add($file, 'archibald', 'identifier_file', 1);
    }
    return archibald_file_get_url($file);
  }

  /**
   * get temp file content or donwload it
   *
   * @param string $url
   *   URL to ressource
   * @param integer $life_time
   *   to to refresh in second
   * @param string $ident
   *   ident sting for drupal variables
   * @global object $user
   *
   * @return string
   */
  protected function getTempFile($url, $life_time, $ident) {
    global $user;

    $destination = 'temporary://';

    $fid = variable_get($ident);

    if (!empty($fid)) {

      $file = file_load($fid);
      if (is_object($file) &&
        // file exists
        is_file($file->uri) &&
        // file is not empy
        filesize($file->uri) > 0 &&
        filemtime($file->uri) + $life_time > time()
      ) {
        // file is not outdated
        return file_get_contents($file->uri);
      }
    }

    $file = new stdClass();
    $file->uid = $user->uid;
    // temporary
    $file->status      = 0;
    $file->filename    = $ident . ' .data';
    $file->destination = file_destination(
      $destination . $file->filename, FILE_EXISTS_REPLACE
    );
    $file->uri = $file->destination;
    $file->filemime = 'application/octet-stream';

    $orig_source = '';
    if (!empty($url)) {
      $orig_source = file_get_contents($url);
      file_put_contents($file->uri, $orig_source);
    }
    else {
      file_put_contents($file->uri, '');
    }
    $file->filesize = filesize($file->uri);

    $file->fid = @db_query(
      'SELECT fid FROM {file_managed} WHERE uri=:uri',
      array('uri' => $file->uri)
    )->fetchColumn(0);

    drupal_chmod($file->uri);
    $file = file_save($file);

    variable_set($ident, $file->fid);

    return $orig_source;
  }

  /**
   * write content to perm file
   *
   * @param string $ident
   *   ident sting for drupal variables
   * @param string $content
   *   the content to be written
   * @global object $user
   *
   * @return string
   */
  protected function writePermFile($ident, $content) {
    $fid = variable_get($ident);
    if (!empty($fid)) {
      $file = file_load($fid);
    }
    if (!empty($file)) {
      if (is_object($file) && is_file($file->uri)) {
        file_put_contents($file->uri, $content);
        drupal_chmod($file->uri);
        file_save($file);

        return TRUE;
      }
    }
    variable_set($ident, '');
    $this->getPermFile($ident);
    $this->writePermFile($ident, $content);
    return FALSE;
  }

  /**
   * get perm file content or create an empty file
   *
   * @param string $ident
   *   ident sting for drupal variables
   * @global object $user
   *
   * @return string
   */
  protected function getPermFile($ident) {
    global $user;

    $fid = variable_get($ident);

    if (!empty($fid)) {
      $file = file_load($fid);
      if (!empty($file)) {
        return file_get_contents($file->uri);
      }
    }

    $destination = 'public://files/';

    // Begin building file object.
    $file = new stdClass();
    // admin
    $file->uid          = 1;
    $file->status       = FILE_STATUS_PERMANENT;
    $file->filename     = $ident;
    $file->filesize     = 0;
    $destination_scheme = file_uri_scheme($destination);
    @unlink($file->uri);
    $file->uri = file_destination($destination . $ident, FILE_EXISTS_RENAME);
    file_put_contents($file->uri, '');

    drupal_chmod($file->uri);

    $file = file_save($file);
    file_usage_add($file, 'archibald', $ident, 1);
    variable_set($ident, $file->fid);
    return "";
  }
}

if (!function_exists('archibald_xml2array')) {

  /**
   * parse a xml string into an array
   *
   * @param string $contents
   *   the xml source code
   * @param boolean $get_attributes
   *   if set to TRUE, attributes will be returned as well
   * @param strubg $priority
   *   can be tag or some other string, if
   *
   * @return array
   */
  function archibald_xml2array($contents, $get_attributes = 1, $priority = 'tag') {
    if (!$contents) {
      return array();
    }

    if (!function_exists('xml_parser_create')) {
      //print "'xml_parser_create()' function not found!";
      return array();
    }

    //Get the XML parser of PHP
    //PHP must have this module for the parser to work
    $parser = xml_parser_create('');
    # http://minutillo.com/steve/weblog/2004/6/17/php-xml-and-character-encodings-a-tale-of-sadness-rage-and-data-loss
    xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, 'UTF-8');
    xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
    xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
    xml_parse_into_struct($parser, trim($contents), $xml_values);
    xml_parser_free($parser);

    // Hmm...
    if (!$xml_values)
    return;



    //Initializations
    $xml_array   = array();
    $parents     = array();
    $opened_tags = array();
    $arr         = array();

    // Refference
    $current = &$xml_array;
    //Go through the tags.
    //Multiple tags with same name will be turned into an array
    $repeated_tag_index = array();
    foreach ($xml_values as $data) {
      //Remove existing values, or there will be trouble
      unset($attributes, $value);

      //This command will extract these variables into the foreach scope
      // tag(string), type(string), level(int), attributes(array).
      // We could use the array by itself, but this cooler.
      extract($data);

      $result = array();
      $attributes_data = array();

      if (isset($value)) {
        if ($priority == 'tag') {
          //Put the value in a assoc array if we are in the 'Attribute' mode
          $result = $value;
        }
        else {
          $result['value'] = $value;
        }
      }

      //Set the attributes too.
      if (isset($attributes) and $get_attributes) {
        foreach ($attributes as $attr => $val) {
          if ($priority == 'tag') {
            //Set all the attributes in a array called 'attr'
            $attributes_data[$attr] = $val;
          }
          else {
            $result['@attributes'][$attr] = $val;
          }
        }
      }

      //See tag status and do the needed.
      // The starting of the tag '<tag>'
      if ($type == "open") {
        $parent[$level - 1] = &$current;
        if (!is_array($current) or (!in_array($tag, array_keys($current)))) {
          //Insert New tag
          $current[$tag] = $result;
          if ($attributes_data) {
            $current[$tag . '_attr'] = $attributes_data;
          }
          $repeated_tag_index[$tag . '_' . $level] = 1;

          $current = &$current[$tag];
        }
        // There was another element with the same tag name
        else {
          if (isset($current[$tag][0])) {
            //If there is a 0th element it is already an array
            $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;
            $repeated_tag_index[$tag . '_' . $level]++;
          }
          else {
            //This section will make the value an array
            //if multiple tags with the same name appear together
            //This will combine the existing item
            //and the new item together to make an array
            $current[$tag] = array($current[$tag], $result);
            $repeated_tag_index[$tag . '_' . $level] = 2;

            if (isset($current[$tag . '_attr'])) {
              //The attribute of the last(0th)
              //tag must be moved as well
              $current[$tag]['0_attr'] = $current[$tag . '_attr'];
              unset($current[$tag . '_attr']);
            }
          }
          $last_item_index = $repeated_tag_index[$tag . '_' . $level] - 1;
          $current = &$current[$tag][$last_item_index];
        }
      }
      // Tags that ends in 1 line '<tag />'
      elseif ($type == "complete") {
        //See if the key is already taken.
        // New Key
        if (!isset($current[$tag])) {
          $current[$tag] = $result;
          $repeated_tag_index[$tag . '_' . $level] = 1;
          if ($priority == 'tag' && $attributes_data) {
            $current[$tag . '_attr'] = $attributes_data;
          }
        }
        // If taken, put all things inside a list(array)
        else {
          if (isset($current[$tag][0]) && is_array($current[$tag])) {
            //If it is already an array...
            // ...push the new element into that array.
            $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;

            if ($priority == 'tag' && $get_attributes && $attributes_data) {
              $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
            }
            $repeated_tag_index[$tag . '_' . $level]++;
          }
          // If it is not an array...
          else {
            $current[$tag] = array($current[$tag], $result);
            //...Make it an array using using the existing value and the new value
            $repeated_tag_index[$tag . '_' . $level] = 1;
            if ($priority == 'tag' and $get_attributes) {
              if (isset($current[$tag . '_attr'])) {
                //The attribute of the last(0th) tag must be moved as well
                $current[$tag]['0_attr'] = $current[$tag . '_attr'];
                unset($current[$tag . '_attr']);
              }

              if ($attributes_data) {
                $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
              }
            }
            // 0 and 1 index is already taken
            $repeated_tag_index[$tag . '_' . $level]++;
          }
        }
      }
      // End of tag '</tag>'
      elseif ($type == 'close') {
        $current = &$parent[$level - 1];
      }
    }

    return ($xml_array);
  }
}

