<?php

/**
 * @file
 * class to import Mark21 resources in fritic dialect into archibald
 * Mark21 XML 2 Lom - Import / Export
 */
require_once ARCHIBALD_IMPORT_PATH . 'plugins/mark21.class.inc';

/**
 * class to import Mark21 resources in fritic dialect into archibald
 */
class ArchibaldMark21Fritic extends ArchibaldMark21 {

  /**
   * vocabular mapping cache
   *
   * @var array
   */
  public $mapping = array();
  private $relationMapping = array(
    'lom2rel' => array(),
    'rel2lom' => array(),
  );
  private $mark21RelationsIdent = 'ArchibaldMark21_fritic_import_relations2';
  private $relations = array();
  private $rowindex = array();

  /**
   * read the xml struct if it is not present
   * this function will be called at the beginning within getRowAt() function
   * @staticvar array $orig_source
   *   cache variable, because we do not want to read out the hole xml struct
   *   with every batch import call
   */
  public function prepare() {
    static $orig_source;
    static $relations;

    $relation_content = trim($this->getPermFile($this->mark21RelationsIdent));
    $rel_mapping = (array) json_decode($relation_content);

    if (!empty($rel_mapping) && is_array($rel_mapping)) {
      if (!empty($rel_mapping['lom2rel'])) {
        foreach ($rel_mapping['lom2rel'] as $k => $rel) {
          $this->relationMapping['lom2rel'][$k] = $rel;
        }
      }

      if (!empty($rel_mapping['rel2lom'])) {
        foreach ($rel_mapping['rel2lom'] as $k => $rel) {
          $this->relationMapping['rel2lom'][$k] = $rel;
        }
      }
    }

    if (empty($orig_source)) {
      $orig_source = $this->getMarc21Source();
      $orig_source = archibald_xml2array($orig_source, 1, '');
    }
    $this->origSource = $orig_source;
    if (empty($this->relations)) {
      $this->getRelations();
    }
  }

  /**
   * # NOT NEEDED IN PRODUCTION
   * function to generate the xml mapping file
   */
  public function parseVocabular() {
    foreach ($this->mapping as $name => $voc) {
      echo "<br /><br />[" . $name . "]<br />";
      foreach ($voc as $term => $irgendwas) {
        $term = trim($term);
        if (empty($term)) {
          continue;
        }
        echo $term . ";NOTSET;NOTSET<br />";
      }
    }
  }

  /**
   * add the given voc to mapping vocabular with given name
   *
   * @param String $name
   *   the mapping vocabular name
   * @param mixed $voc
   *   string or array with vocabular values
   */
  private function addMappingVocabular($name, $voc) {

    if (!isset($this->mapping[$name])) {
      $this->mapping[$name] = array();
    }

    if (!is_array($voc)) {
      $voc = array($voc);
    }
    foreach ($voc as $term) {
      $term = trim($term);
      if (empty($term)) {
        continue;
      }
      $this->mapping[$name][$term] = TRUE;
    }
  }

  /**
   * Print out the the xml mapping file content
   */
  public function getVocab() {
    $this->prepare();
    $c = count($this->origSource['collection']['record']);

    echo $c . "<br />\n";
    for ($i = 0; $i < $c; $i++) {
      $this->import($i);
    }
    $this->parseVocabular();
  }

  /**
   * get the relations
   * @staticvar array $relations
   *  the the relations cache
   *
   * @return boolean
   *  return TRUE on success, else FALSE
   */
  public function getRelations() {
    static $relations;
    //cache file so we do not need to generate the hole thing multiple times
    $ident = 'archibald_fritic_marc21_relation_cache';
    $relation = trim($this->getTempFile('', (60 * 60 * 24), $ident));

    if (!empty($relation)) {
      $relation_arr = (array) json_decode($relation);
    }
    // File is cached use it
    if (!empty($relation_arr)) {
      $this->relations = array();
      foreach ($relation_arr as $k => $v) {
        //Setup relations array
        $this->relations[$k] = array(
          'parent' => $v->parent,
          'title' => $v->title,
        );
      }
    }
    // Not cached
    else {
      $c = count($this->origSource['collection']['record']);

      $relation_sub_titles = array();

      //Resource which will be skipped
      $skip_resources = array(
        347 => TRUE,
        349 => TRUE,
        350 => TRUE,
        351 => TRUE,
        352 => TRUE,
        353 => TRUE,
        354 => TRUE,
        355 => TRUE,
        356 => TRUE,
      );

      //Loop through all resources
      for ($i = 0; $i < $c; $i++) {
        //source is not valid, continue
        if (!($source = $this->getRowAt($i))) {
          continue;
        }
        $this->source = $source;

        //Get resource_id
        list($a, $b, $ressource_id) = explode(':', $this->getValueAt('024.8. .a'), 3);

        if (isset($skip_resources[$ressource_id])) {
          //Skip resource if it is set as skip resource
          continue;
        }

        //Get first subtitle
        $sub = explode(",", $this->getValueAt("245.0. .0.b"));
        if (empty($sub) || !is_array($sub) || count($sub) == 0) {
          $sub = '';
        }
        else {
          $sub = $sub[0];
        }

        //if no subtitle found use the title
        if (empty($sub)) {
          $sub = $this->getValueAt("245.0. .0.a");
        }

        $relation_sub_titles[$ressource_id] = $sub;

        //Get relations
        $relations = $this->getValueAt('787. . ');
        if (!empty($relations)) {
          if (!is_array($relations)) {
            $relations = array($relations);
          }
          $this->relations[$ressource_id] = array();
          foreach ($relations as $tmp_arr) {
            if (empty($tmp_arr['w'])) {
              continue;
            }
            //Map the resource_id with the childs
            $this->relations[$ressource_id][] = $tmp_arr['w'];
          }
        }
      }

      //@bugfix fritic resource 329 has a child which points to it self,
      //        should be 328
      foreach ($this->relations[329] as $k => $relation_arr) {
        if ($relation_arr == 329) {
          $this->relations[329][$k] = 328;
        }
      }

      $relations_good = array();

      foreach ($this->relations as $res_id => $relation_arr) {
        ////loop through all resources
        foreach ($relation_arr as $relation) {
          ////loop through relations
          if (empty($this->relations[$relation]) || count($this->relations[$relation]) != count($relation_arr)) {
            //this relation is bad so skip this relation
            continue;
          }

          $relation_ok = TRUE;

          //Set resource relations
          $tmp_relation_arr = $relation_arr;
          //Add parent to relations
          $tmp_relation_arr[] = $res_id;

          foreach ($tmp_relation_arr as $k => $v) {
            //loop through the relations
            //and transform it to res_id => res_id instead of count => res_id
            $tmp_relation_arr[$v] = $v;
            unset($tmp_relation_arr[$k]);
          }
          //remove current relation id from relation array
          unset($tmp_relation_arr[$relation]);

          foreach ($this->relations[$relation] as $rel_relation) {
            //loop through parent relations
            //if parent relation is not set in current relation,
            //skip it and set relation_ok to FALSE
            if (!isset($tmp_relation_arr[$rel_relation])) {
              $relation_ok = FALSE;
              continue;
            }
            //relation is in current relation relations. unset it
            unset($tmp_relation_arr[$rel_relation]);
          }

          //if we have a relation left on current relation
          //the hole relation has errors so set it to FALSE
          if (count($tmp_relation_arr) > 0) {
            $relation_ok = FALSE;
          }

          //Get again the hole relation with all childs
          $hole_releation = $this->relations[$res_id];
          $hole_releation[] = $res_id;
          sort($hole_releation);
          reset($hole_releation);


          if ($relation_ok == TRUE) {
            //if the relation is good add it to the array
            //the identifier is the first relation resource id
            $relations_good[current($hole_releation)] = array(
              'title' => trim($relation_sub_titles[current($hole_releation)]),
              'childs' => $hole_releation,
            );
          }
        }
      }

      $this->relations = array();
      foreach ($relations_good as $k => $arr) {
        //loop through all good relations
        //Generating the parent identifier for this relation
        $parent_relation = implode('', $arr['childs']);

        foreach ($arr['childs'] as $relation_child) {
          //loop through relation childs
          //Setup the needed information to generate the parent resource
          //for every child
          //All childs have the same information about the parent resource
          $this->relations[$relation_child] = array(
            'parent' => $parent_relation,
            'title' => $arr['title'],
          );
        }
      }
      //Write cache file
      $this->writePermFile($ident, json_encode($this->relations));
    }
    return TRUE;
  }

  /**
   * Get the vocabular for current resource
   */
  private function getVocabulary() {

    $this->addMappingVocabular('context', $this->getValueAt("084. . .0.a"));
    $this->addMappingVocabular('learningressourcetype', $this->getValueAt("245.0. .0.k"));
    $this->addMappingVocabular('difficulty', $this->getValueAt("594. . .0.2"));
    $this->addMappingVocabular('typical learning type', $this->getValueAt("595. . .0.2"));
    $this->addMappingVocabular('cost + copyright', $this->getValueAt("540. . .9"));

    $target_group = $this->getValueAt("521. . .0.9");
    if (!empty($target_group)) {
      list($a, $b, $enduserrole, $typical_age_range, $b) = explode(":", $target_group, 5);

      $this->addMappingVocabular("intended end user role", $enduserrole);
      $this->addMappingVocabular("typical age range", $typical_age_range);
    }
  }

  /**
   * Test function for developing and test all resources against
   * test rules. we can test things on each resource within the
   * or ($row_index = 0; $row_index < $c; $row_index++) { loop
   *
   * @param string $identifier
   *
   * @return boolean
   */
  public function getAll($identifier) {
    $differents = array();
    $all        = array();
    $c          = count($this->origSource['collection']['record']);
    for ($row_index = 0; $row_index < $c; $row_index++) {
      $row = $this->getRowAt($row_index);
      if (!$row) {
        continue;
      }
      $this->source = $row;
      //Resource id

      if (empty($row['598'][' '][' '])) {
        continue;
      }

      list($a, $b, $ressource_id) = explode(":", $this->getValueAt("024.8. .a"), 3);


      foreach ($this->getValueAt("650. .6", array()) as $kw) {
        $kw['a'] = trim($kw['a']);
        if (empty($kw['a'])) {
          continue;
        }

        if (!isset($coverages_already['fr'])) {
          $coverages_already['fr'] = array();
        }

        // Prevent duplicates
        if (isset($coverages_already['fr'][$kw['a']])) {
          continue;
        }
        $coverages_already['fr'][$kw['a']] = TRUE;
        $result[md5($kw['a'])] = $kw['a'];
      }

      /*
        // Get title
        #echo "Description: ".$this->getValueAt("245.0. .0.a")." (".$ressource_id.")\n";
        $entry = $row['598'][' '][' '][0];
        $check = md5($entry['g']."|".$entry['t']."|".$entry['u']."|".$entry['w']);
        if (!isset($differents[$entry['w']])) {
        $differents[$entry['w']] = array();
        $all[$entry['w']] = 0;
        }
        $all[$entry['w']]++;
        $differents[$entry['w']][$check] = TRUE;
        #echo "\n########################################\n"; */
    }
    var_export($result);
    die();

    /* foreach ($differents as $key => $arr) {
      $c = count($arr);
      if ($c == 1) {
      continue;
      }
      echo $key." = ".$c." von ".$all[$key]."\n";
      } */

    return FALSE;
  }

  /**
   * returns the row index (array index key) for the given resource id
   *
   * @param string $res_id
   *
   * @return string
   *  lom_id
   */
  public function getRowindexByResourceId($res_id) {
    //cache file so we do not need to generate the hole thing multiple times
    $ident = 'archibald_fritic_marc21_rowindex_cache';
    $rowindex = trim($this->getTempFile('', (60 * 60 * 24), $ident));

    if (!empty($rowindex)) {
      $rowindex_arr = (array) json_decode($rowindex);
    }
    // File is cached use it
    if (!empty($rowindex_arr)) {
      $this->rowindex = $rowindex_arr;
    }
    // Not cached
    else {
      $c = count($this->origSource['collection']['record']);
      for ($row_index = 0; $row_index < $c; $row_index++) {
        $row = $this->getRowAt($row_index);
        if (!$row) {
          continue;
        }

        //Resource id
        list($a, $b, $ressource_id) = explode(":", $this->getValueAt("024.8. .a", '', $row), 3);

        $this->rowindex[$ressource_id] = $row_index;
      }

      $this->writePermFile($ident, json_encode($this->rowindex));
    }
    $rows = array();
    foreach ($this->rowindex as $i => $v) {
      $rows['' . $i] = $v;
    }
    $this->rowindex = $rows;
    if (empty($this->rowindex[$res_id])) {
      return FALSE;
    }

    return $this->rowindex[$res_id];
  }

  /**
   * get a resource by id
   *
   * @param string $res_id
   *
   * @return array
   *   a parsed array
   */
  public function findResByResourceId($res_id) {
    $rowindex = $this->getRowindexByResourceId($res_id);
    if ($rowindex == FALSE) {
      return FALSE;
    }
    return $this->getRowAt($rowindex);
  }

  /**
   * return lom object from given marcxml data
   * If you need to know which codes are for what look at wiki
   * http://wiki.ubuntu/digitale_schulbibliothek/import/fritic
   *
   * @param mixed $row_index
   *   the row number within the imported array
   * @param array $context
   *   batch context infos (optional, default empty)
   *
   * @return boolean
   *   TRUE on success, else FALSE
   */
  public function import($row_index, &$context = array(), $no_relations = FALSE) {
    if (!($source = $this->getRowAt($row_index))) {
      //Resource not found, return FALSE
      return FALSE;
    }
    $this->source = $source;

    //Get the vocabulary for this resource
    $this->getVocabulary();

    //Setup educa curriculum
    $curr_name = 'curriculum_educa';
    module_load_include('php', 'archibald', 'includes/curriculum/' . $curr_name . '.class');
    $curr_class_name = 'ArchibaldCurriculumEduca';
    $educa_handler = new $curr_class_name();



    $lom = new ArchibaldLom();

    $general = new ArchibaldLomDataGeneral();

    $title = '';
    // Get title
    $title .= $this->getValueAt("245.0. .0.a");
    // Get subtitle
    $subtitle = $this->getValueAt("245.0. .0.b");
    if (!empty($subtitle)) {
      // Add a new line if subtitle is not empty to get title \n subtitle
      $subtitle = ": " . $subtitle;
    }
    else {
      $subtitle = '';
    }

    //Resource id
    list($a, $b, $ressource_id) = explode(
      ":", $this->getValueAt("024.8. .a"), 3
    );

    //Main Language
    list($lang, $a) = explode(":", $this->getValueAt("040. . .0.b"));
    $lang = drupal_substr($lang, 0, 2);

    $general->setTitle(new ArchibaldLomDataLangString($title . $subtitle, $lang));
    $general->setDescription(new ArchibaldLomDataLangString($this->getValueAt("520. . .0.a"), $lang));
    $general->addLanguage($lang);

    //Aggregation level
    list($a, $b, $aggregation_string, $c) = explode(":", $this->getValueAt("591. . .0.2"), 4);

    $aggregation_array = explode("-", $aggregation_string);
    $aggregation_level = $aggregation_array[count($aggregation_array) - 1];
    $general->setAggregationLevel($aggregation_level);

    //Free Keywords
    //Coverages
    $coverages_already = array();
    foreach ($this->getValueAt("659. . ", array()) as $kw) {
      if (empty($kw['x']) || empty($kw['a'])) {
        continue;
      }
      if (drupal_strlen($kw['x']) > 3) {
        $kw['x'] = $lang;
      }
      $general->addKeyword(new ArchibaldLomDataLangString($kw['a'], $this->parseLangKey($kw['x'])));
    }

    //Controlled Keywords language choosen manual
    foreach ($this->getValueAt("650. .0", array()) as $kw) {
      $kw['a'] = trim($kw['a']);
      if (empty($kw['a'])) {
        continue;
      }
      if (!isset($coverages_already['ck_' . $lang])) {
        $coverages_already['ck_' . $lang] = array();
      }

      if (isset($coverages_already['ck_' . $lang][$kw['a']])) {
        //Prevent duplicates
        continue;
      }
      $coverages_already['ck_' . $lang][$kw['a']] = TRUE;
      $general->addCoverage(new ArchibaldLomDataLangString($kw['a'], $lang));
    }

    //Controlled keywords lang FR
    foreach ($this->getValueAt("650. .6", array()) as $kw) {
      $kw['a'] = trim($kw['a']);
      if (empty($kw['a'])) {
        continue;
      }

      if (!isset($coverages_already['fr'])) {
        $coverages_already['fr'] = array();
      }

      // Prevent duplicates
      if (isset($coverages_already['fr'][$kw['a']])) {
        continue;
      }
      $coverages_already['fr'][$kw['a']] = TRUE;
      //lang fr couse of thsauri ramount (just fr)
      $general->addCoverage(new ArchibaldLomDataLangString($kw['a'], 'fr'));
    }

    //Controlled keywords coming from some tsauri's, for now just SWD
    foreach ($this->getValueAt("650. .7", array()) as $kw) {
      $kw['a'] = trim($kw['a']);
      if (empty($kw['a'])) {
        continue;
      }
      $lang_key = '';
      switch ($kw['2']) {
        case 'SWD':
          //swd tsauri just in german
        default:
          $lang_key = 'de';
      }

      if (!isset($coverages_already['ckv_' . $lang_key])) {
        $coverages_already['ckv_' . $lang_key] = array();
      }

      if (isset($coverages_already['ckv_' . $lang_key][$kw['a']])) {
        //Prevent duplicates
        continue;
      }
      $coverages_already['ckv_' . $lang_key][$kw['a']] = TRUE;

      $general->addCoverage(new ArchibaldLomDataLangString($kw['a'], $lang_key));
    }


    $meta_metadata = new ArchibaldLomDataMetaMetadata();
    $lifecycle = new ArchibaldLomDataLifeCycle();


    foreach ($this->getValueAt("700. . ", array()) as $contribute) {
      $contribute_obj = new ArchibaldLomDataContribute();
      $contribute['a'] = trim($contribute['a']);
      if (empty($contribute['a'])) {
        continue;
      }
      if (!preg_match("/,/", $contribute['a'])) {
        $contribute['a'] .= ", ";
      }
      list($last, $first) = explode(", ", $contribute['a'], 2);
      $lom_contributer = new ArchibaldLomContributer();
      $lom_contributer->renderDataObject(array(
          'firstname' => $first,
          'lastname' => $last,
        ));

      $lom_contributer->save();
      // add vcard
      $contribute_obj->addEntity($lom_contributer->getVcard());

      // set role
      $vocabulary = new ArchibaldLomDataVocabulary();
      $vocabulary->setSource('FriPortal');
      $vocabulary->setValue($contribute['e']);
      $contribute_obj->setRole($vocabulary);

      if ($contribute['e'] == "validator") {
        $meta_metadata->addContribute($contribute_obj);
        continue;
      }

      $lifecycle->addContribute($contribute_obj);
    }

    $vocabulary = new ArchibaldLomDataVocabulary();
    $vocabulary->setSource('LOMv1.0');
    $vocabulary->setValue('draft');
    $lifecycle->setStatus($vocabulary);

    $version = array();
    $a = $this->getValueAt("260. . .0.a");
    if (is_array($a)) {
      $a = implode(", ", $a);
    }
    $c = $this->getValueAt("260. . .0.c");
    if (!empty($a)) {
      $version[] = $a;
    }
    if (!empty($c)) {
      $version[] = $c;
    }
    $lifecycle->setVersion(new ArchibaldLomDataLangString(implode(", ", $version), $lang));


    $meta_metadata->setLanguage($lang);
    $lom->setMetaMetadata($meta_metadata);

    $technical = new ArchibaldLomDataTechnical();

    $tmp_arr = $this->getValueAt("856.4.0");
    if (!is_array($tmp_arr)) {
      $tmp_arr = array();
    }
    foreach ($tmp_arr as $entry) {
      if ($entry['x'] != 'main') {
        continue;
      }
      $technical->setFormat($entry['q']);
      if (isset($entry['s'])) {
        $technical->setSize($entry['s']);
      }
      if (preg_match("/educanet2\.ch\/cds/is", $entry['u'])) {
        continue;
      }
      $general->addIdentifier(new ArchibaldLomDataIdentifier("URL", $entry['u']));
      $technical->setLocation($entry['u']);
    }

    $title_arr = array();
    $multi_title = $this->getValueAt("856.4.8");
    if (is_array($multi_title)) {
      foreach ($multi_title as $row) {
        if (preg_match("/educanet2\.ch\/cds/is", $row['u'])) {
          continue;
        }
        $identi = new ArchibaldLomDataIdentifier("URL", $row['u'], new ArchibaldLomDataLangString(trim($row['y']), $lang));
        $general->addIdentifier($identi);
      }
    }

    $tmp_arr = $this->getValueAt("856.4. ");
    if (!is_array($tmp_arr)) {
      $tmp_arr = array();
    }
    $publisher_logo = '';
    foreach ($tmp_arr as $entry) {
      if ($entry['x'] == 'publisher') {
        $publisher_logo = $this->downloadFile($entry['u'], 'publisher');
        continue;
      }
      if ($entry['x'] != 'preview') {
        continue;
      }

      $technical->setPreviewImage($this->downloadFile($entry['u'], 'preview'));
    }

    foreach ($this->getValueAt("260. . ", array()) as $contribute) {
      if (empty($contribute['a']) || empty($contribute['b'])) {
        continue;
      }

      if (!is_array($contribute['a'])) {
        $contribute['a'] = array($contribute['a']);
      }

      if (!is_array($contribute['b'])) {
        $contribute['b'] = array($contribute['b']);
      }

      //Fix differnt size of city / company if
      //a city or company is less than the other,
      //fill it with the first element
      $count_a = count($contribute['a']);
      $count_b = count($contribute['b']);
      if ($count_a < $count_b) {
        for ($i = 0; $i < ($count_b - $count_a); $i++) {
          $contribute['a'][] = $contribute['b'][0];
        }
      }

      $count_a = count($contribute['a']);
      $count_b = count($contribute['b']);
      if ($count_b < $count_a) {
        for ($i = 0; $i < ($count_a - $count_b); $i++) {
          $contribute['b'][] = $contribute['a'][0];
        }
      }

      foreach ($contribute['a'] as $i => $city) {

        $city = trim($city);
        $company = trim($contribute['b'][$i]);

        $contribute_obj = new ArchibaldLomDataContribute();

        $render_data = array(
          'organisation' => $company,
          'city' => $city,
        );

        if (!empty($publisher_logo)) {
          $l                   = new stdClass();
          $l->uri              = $publisher_logo;
          $render_data['logo'] = $l;
        }
        $lom_contributer = new ArchibaldLomContributer();
        $lom_contributer->renderDataObject($render_data);
        $lom_contributer->save();
        // add vcard
        $contribute_obj->addEntity($lom_contributer->getVcard());

        // set role
        $vocabulary = new ArchibaldLomDataVocabulary();
        $vocabulary->setSource('LOMv1.0');
        $vocabulary->setValue('editor');
        $contribute_obj->setRole($vocabulary);

        $lifecycle->addContribute($contribute_obj);
      }
    }

    $technical->setOtherPlattformRequirements(new ArchibaldLomDataLangString($this->getValueAt("538. . .0.a"), $lang));

    // needed here couse general identifier
    $lom->setGeneral($general);
    $lom->setTechnical($technical);


    $education = new ArchibaldLomDataEducation();
    //@TODO learning resource type, intended end user role, difficulty, context
    $learning_time = $this->getValueAt("595. . .0.2");
    switch ($learning_time) {
      case 'minus-1':
        $learning_time = "PT50M";
        break;

      case '1-5':
        $learning_time = 'PT150M';
        break;

      case 'plus-5':
        $learning_time = 'PT251M';
        break;

      default:
        $learning_time = 0;
        break;
    }

    $education->setTypicalLearningTime(new ArchibaldLomDataDuration($learning_time));
    $education->setDescription(new ArchibaldLomDataLangString($this->getValueAt("919. . .0.a"), $lang));
    $target_group = $this->getValueAt("521. . .0.9");

    $typical_age_range = $this->getTypicalAgeRange($this->getValueAt("521. . .0.a"));

    if (!empty($typical_age_range)) {
      $education->addTypicalAgeRange(new ArchibaldLomDataLangString($typical_age_range, $lang));
    }
    if (!empty($target_group)) {
      list($a, $b, $enduserrole, $typical_age_range, $b) = explode(":", $target_group, 5);

      if (empty($typical_age_range)) {
        $education->addTypicalAgeRange(new ArchibaldLomDataLangString($typical_age_range, $lang));
      }

      $roles = $this->getMapping($enduserrole, 'intended end user role');
      foreach ($roles as $role) {
        $education->addIntendedEndUserRole(new ArchibaldLomDataVocabulary($role['key'], $role['source']));
      }
    }

    $difficult = $this->getValueAt("594. . .0.2");
    if (!empty($difficult)) {
      foreach ($this->getMapping($difficult, 'difficulty') as $diffi) {
        $education->setDifficult(new ArchibaldLomDataVocabulary($diffi['key'], $diffi['source']));
      }
    }

    $tmp_arr = $this->getValueAt("084. . .0.a");
    if (!is_array($tmp_arr)) {
      $tmp_arr = array();
    }
    $tmp_check = array();
    static $already_context = array();
    foreach ($tmp_arr as $context) {
      if (empty($context) || isset($tmp_check[$context])) {
        continue;
      }
      $tmp_check[$context] = TRUE;
      $contextes = $this->getMapping($context, "context", $classifications);
      foreach ($contextes as $con) {
        if (empty($con['key'])) {
          continue;
        }
        if (isset($already_context[$con['key'] . "|" . $con['source']])) {
          continue;
        }
        $already_context[$con['key'] . "|" . $con['source']] = TRUE;
        $education->addContext(new ArchibaldLomDataVocabulary($con['key'], $con['source']));
      }
    }
    if (!empty($classifications['disziplin']) && !empty($classifications['educational_level'])) {
      foreach ($classifications['educational_level'] as $education_level) {
        foreach ($classifications['disziplin'] as $disziplin) {
          $educational_level_list = $educa_handler->getTaxonomyPathByKey('educa_school_levels', $education_level, 'key');

          $educa_school_subjects_list = $educa_handler->getTaxonomyPathByKey('educa_school_subjects', $disziplin, 'key');

          if (empty($educational_level_list) || empty($educa_school_subjects_list)) {
            continue;
          }
          $educa_handler->addTaxonPathEduca($educational_level_list, $educa_school_subjects_list);
        }
      }
    }


    $tmp_arr = $this->getValueAt("245.0. .0.k");
    if (!is_array($tmp_arr)) {
      $tmp_arr = array();
    }
    $already_added = array();
    foreach ($tmp_arr as $type) {
      if (empty($type)) {
        continue;
      }
      $learningressourcetypes = $this->getMapping($type, 'learningressourcetype');

      foreach ($learningressourcetypes as $type_val) {
        if (isset($already_added[$type_val['key']])) {
          continue;
        }
        $already_added[$type_val['key']] = TRUE;
        $education->addLearningResourceType(new ArchibaldLomDataVocabulary($type_val['key'], $type_val['source']));
      }
    }



    $lom->setEducation($education);
    $rights = new ArchibaldLomDataRights();
    $r_desc = trim($this->getValueAt("540. . .0.a") . "\n" . $this->getValueAt("540. . .0.u"));
    $rights->setDescription(new ArchibaldLomDataLangString($r_desc, $lang));
    $cr = "yes";
    if (empty($r_desc)) {
      $cr = 'no';
    }

    $rights->setCopyrightAndOtherRestrictions(new ArchibaldLomDataVocabulary($cr));
    if (preg_match("/price-(.+):.+/iUs", $this->getValueAt("540. . .9"), $matches)) {
      $rights->setCost($matches[1]);
    }

    $lom->setRights($rights);

    $per2default = variable_get('archibald_per2default', array());

    $curr_name = 'curriculum_per';
    module_load_include('php', 'archibald', 'includes/curriculum/curriculum_per.class');
    $curr_handler = new ArchibaldCurriculumPer();

    /**
     * For Debuging following code.
     * drush archibald_import_test mark21_fritic 1
     * drush archibald_import_test mark21_fritic 11
     */
    $val = $this->getValueAt("598. . ");
    if (!empty($val)) {
      if (!is_array($val)) {
        $val = array($val);
      }
      foreach ($val as $per) {
        $subtitle_hits = array();

        // Example for w:  Plan d'études romand (PER)/FG 12
        if (preg_match("/\/([^\/]+)$/is", $per['w'], $matches)) {

          $per_id = $matches[1];
          $subtitles = $curr_handler->getSubtitles($per_id);

          // Example for g:  Application de règles de sécurité dans un contexte donné. Recherche de solutions privilégiant l'intégrité physique et l'estime de soi dans diverses situations observées ou vécues. Reconnaissance des conduites à risque
          foreach (explode(';', $per['g']) as $details_part) {
            $subtitle_hit = NULL;
            foreach ($subtitles['subtitle'] as $subtitle1) {
              foreach ($subtitle1->details as $detail) {
                $simular_chars = similar_text($details_part, $detail->text, $percent_simularety);
                if ($percent_simularety > 80 && (empty($subtitle_hit) || $subtitle_hit['percent'] < $percent_simularety)) {
                  // find the best match with highes simularety
                  $subtitle_hit = array(
                    'percent' => $percent_simularety,
                    'detail_key' => $detail->url_part,
                    'subitle_uuid' => $subtitle1->uuid,
                  );
                }
              }

              foreach ($subtitle1->childs as $subtitle2) {
                foreach ($subtitle2->details as $detail) {
                  $simular_chars = similar_text($details_part, $detail->text, $percent_simularety);
                  if ($percent_simularety > 80 && (empty($subtitle_hit) || $subtitle_hit['percent'] < $percent_simularety)) {
                    $subtitle_hit = array(
                      'percent' => $percent_simularety,
                      'detail_key' => $detail->url_part,
                      'subitle_uuid' => $subtitle2->uuid,
                    );
                  }
                }
              }
            }

            if (!empty($subtitle_hit)) {
              $subtitle_hits[$subtitle_hit['subitle_uuid']][] = $subtitle_hit['detail_key'];
            }
          }

          if (!empty($subtitle_hits)) {
            foreach ($subtitle_hits AS $subtitle_key => $details) {
              $curr_handler->addByObjectCode($per_id, $subtitle_key, $details);
            }
          }
          else{
            $curr_handler->addByObjectCode($per_id);
          }

          if (isset($per2default[$per_id]) && !empty($per2default[$per_id])) {

            foreach ($per2default[$per_id] as $raw_mapping_obj) {
              $key = $raw_mapping_obj->educational_level . '|' . $raw_mapping_obj->discipline;

              $educational_level_list = $educa_handler->getTaxonomyPathByKey('educa_school_levels', $raw_mapping_obj->educational_level, 'key');
              $educa_school_subjects_list = $educa_handler->getTaxonomyPathByKey('educa_school_subjects', $raw_mapping_obj->discipline, 'key');

              if (empty($educational_level_list) || empty($educa_school_subjects_list)) {
                continue;
              }
              $proposals[$key] = $educa_handler->themeVal($educational_level_list, $educa_school_subjects_list);
              $educa_handler->addTaxonPathEduca($educational_level_list, $educa_school_subjects_list);
            }
          }
        }
      }
    }
    $lom_curriculum = array();
    if (is_object($curr_handler)) {
      $lom_curriculum = $curr_handler->getLomCurriculum();
      foreach ($lom_curriculum as $curri) {
        $lom->addCurriculum($curri);
      }
    }
    $lom_classification = array();
    if (is_object($educa_handler)) {
      $educa_handler->cleanUp();
      $lom_classification = $educa_handler->getLomClassification();
      foreach ($lom_classification as $classi) {
        $lom->addClassification($classi);
      }
    }

    $lom->setLifeCycle($lifecycle);
    $unique_id      = $this->getValueAt("024.8. .a");
    $unique_id      = explode(":", $unique_id);
    $unique_id      = $unique_id[2];
    $lom_id         = $this->searchLomId(get_class(), $unique_id);
    $add_import_log = TRUE;
    if (!empty($lom_id)) {
      $add_import_log = FALSE;
      $lom->setLomId($lom_id);
    }

    $lom_id = parent::import($lom);
    if ($add_import_log == TRUE && !empty($lom_id)) {
      $fields['lom_id']    = $lom_id;
      $fields['remote_id'] = $unique_id;
      $fields['plugin']    = get_class();
      db_insert('archibald_import_log')
        ->fields($fields)
        ->execute();
    }

    if ($no_relations == FALSE) {
      $must_insert_child = TRUE;
      $mother = $this->getValueAt("773. . .0.w");
      if (!empty($mother)) {


        if (isset($this->relationMapping['rel2lom']["fr" . $mother])) {
          //Mother relation already exists
          $relation_lom_id = $this->relationMapping['rel2lom']["fr" . $mother];
          $relation_parent = ArchibaldLomSaveHandler::load($relation_lom_id);

          foreach ($relation_parent->getRelation() as $relation) {
            $identifiers = $relation->getResource()->getIdentifier();

            foreach ($identifiers as $identifier) {
              if ($identifier->getEntry() == $lom_id) {
                //Child already inserted, do not add
                $must_insert_child = FALSE;
              }
            }
          }
        }
        // No mother exists for this relation, create it
        else {
          $tmp_arr          = array();
          $relation_lom_id = $this->import($this->getRowindexByResourceId($mother), $tmp_arr, TRUE);
          $relation_parent = ArchibaldLomSaveHandler::load($relation_lom_id);
        }

        // We must insert this child
        if ($must_insert_child == TRUE) {
          $resource = new ArchibaldLomDataResource();
          $resource->addIdentifier(new ArchibaldLomDataIdentifier("educa", $lom_id));

          $relation = new ArchibaldLomDataRelation();
          $relation->setResource($resource);
          $relation->setKind(new ArchibaldLomDataVocabulary(ArchibaldLomDataRelation::KIND_HAS_PART));

          $relation_parent->addRelation($relation);

          $relation_lom_id = parent::import($relation_parent);

          $this->relationMapping['lom2rel'][$relation_lom_id] = "fr" . $mother;
          $this->relationMapping['rel2lom']["fr" . $mother] = $relation_lom_id;
          $this->writePermFile($this->mark21RelationsIdent, json_encode($this->relationMapping));
        }
      }
      elseif (isset($this->relations[$ressource_id])) {
        //If resource is within a relation
        $relation_key = $this->relations[$ressource_id]['parent'];
        if (isset($this->relationMapping['rel2lom'][$relation_key])) {
          //Mother relation already exists
          $relation_lom_id = $this->relationMapping['rel2lom'][$relation_key];
          $relation_parent = ArchibaldLomSaveHandler::load($relation_lom_id);

          foreach ($relation_parent->getRelation() as $relation) {
            $identifiers = $relation->getResource()->getIdentifier();

            foreach ($identifiers as $identifier) {
              if ($identifier->getEntry() == $lom_id) {
                //Child already inserted, do not add
                $must_insert_child = FALSE;
              }
            }
          }
        }
        // No mother exists for this relation, create it
        else {
          $relation_parent = new ArchibaldLom();
          $relation_parent_general = new ArchibaldLomDataGeneral();
          $langstring = new ArchibaldLomDataLangString($this->relations[$ressource_id]['title'], $lang);

          $relation_parent_general->setTitle($langstring);
          $relation_parent->setGeneral($relation_parent_general);
        }

        // We must insert this child
        if ($must_insert_child == TRUE) {
          $resource = new ArchibaldLomDataResource();
          $resource->addIdentifier(new ArchibaldLomDataIdentifier("educa", $lom_id));

          $relation = new ArchibaldLomDataRelation();
          $relation->setResource($resource);
          $relation->setKind(new ArchibaldLomDataVocabulary(ArchibaldLomDataRelation::KIND_HAS_PART));
          $relation_parent->addRelation($relation);


          $meta_metadata = new ArchibaldLomDataMetaMetadata();
          $meta_metadata->setLanguage($lang);
          $relation_parent->setMetaMetadata($meta_metadata);

          $life_cycle = new ArchibaldLomDataLifeCycle();
          $vocabulary = new ArchibaldLomDataVocabulary();
          $vocabulary->setSource('LOMv1.0');
          $vocabulary->setValue('draft');
          $life_cycle->setStatus($vocabulary);

          $relation_parent->setLifeCycle($life_cycle);

          $relation_lom_id = parent::import($relation_parent);

          $this->relationMapping['lom2rel'][$relation_lom_id] = $relation_key;
          $this->relationMapping['rel2lom'][$relation_key] = $relation_lom_id;
          $this->writePermFile($this->mark21RelationsIdent, json_encode($this->relationMapping));
        }
      }
    }
    return $lom_id;
  }

  /**
   * parse textual string to new number format
   *
   * @param string $typical_age_range_string
   *   the old textual string
   *
   * @return string
   *   the new typical age range
   */
  private function getTypicalAgeRange($typical_age_range_string) {

    if (preg_match("/.*\((.+)\)$/", trim($typical_age_range_string), $matches)) {

      switch (trim($matches[1])) {
        case 'Children without literacy':
        case 'Kinder ohne Lesekenntnisse':
        case 'Enfants sans lecture':
          return "0-6";

        case 'Childrens with literacy':
        case 'Kinder mit Lesekenntnissen':
        case 'Enfants avec lecture':
          return "7-9";

        case 'Adolescents':
        case 'Jugendliche':
        case 'Adolescents':
          return "10-13";

        case 'Young adults':
        case 'Junge Erwachsene':
        case 'Jeunes adultes':
          return "14-18";

        case 'Adults':
        case 'Erwachsene':
        case 'Adultes':
          return "18-0";

        case 'All groups of ages':
        case 'Alle Altersgruppen':
        case 'Toute les tranches d\'âges':
          return "0-0";
      }
    }
  }

  /**
   * get the new mapping for given oldkey and section
   *
   * @param string $oldkey
   *   the current old key
   * @param string $section
   *   the section like lifecycle or general (optional, default = empty)
   *
   * @return string
   *   the new key
   */
  public function getMapping($oldkey, $section = '', &$classifications = array()) {
    $oldkey = drupal_strtolower($oldkey);
    #echo $oldkey;
    $mapping = $this->getCsv();
    $educational_level = array(
      'cycle_1' => 'compulsory education|LREv3.0',
      'compulsory education' => 'compulsory education|LREv3.0',
      'cycle_2' => 'compulsory education|LREv3.0',
      'post compulsory education' => 'post-compulsory education|ArchibaldLom-CHv1.0',
      'cycle_3' => 'compulsory education|LREv3.0',
      'vocational schools' => 'post-compulsory education|ArchibaldLom-CHv1.0',
      'pre-school' => 'pre-school|LREv3.0',
    );

    if (!isset($mapping[$section][$oldkey]) ||
      $mapping[$section][$oldkey] == "-"
    ) {
      //Not mapped or not found, insert original value with source oldFriportai
      $this->getTermByKey($oldkey);
      return array(array('key' => $oldkey, 'source' => 'oldFriportai'));
    }
    $return = $mapping[$section][$oldkey];
    if ($section == 'context') {
      foreach ($return as &$rat) {
        if (isset($educational_level[$rat['key']])) {
          list($mapped_newkey, $mapped_source) = explode("|", $educational_level[$rat['key']]);
          $classifications['educational_level'][] = $rat['key'];
          $rat['key'] = $mapped_newkey;
          $rat['source'] = $mapped_source;
        }
        else {
          $classifications['disziplin'][] = $rat['key'];
          return array(array('key' => '', 'source' => ''));
        }
      }
    }
    return $return;
  }

  /**
   * read the mapping csv file (vocabulars)
   *
   * @return array
   *   with [section][old_key] = newkey
   */
  public function getCsv() {
    $csv = file_get_contents(ARCHIBALD_IMPORT_PATH . "plugins/fritic_mapping.csv");

    $mapping         = array();
    $classifications = array();
    $section         = '';
    foreach (explode("\n", $csv) as $line) {
      $line = mb_convert_encoding(trim($line), 'UTF-8', array('WINDOWS-1252', 'UTF-8', 'ISO-8859-1'));

      if (empty($line) || preg_match("/^(#|\/\/)/", $line)) {
        continue;
      }
      if (preg_match("/\[([^\[]+)\]/", $line, $matches)) {
        $section = $matches[1];
        $mapping[$section] = array();
        continue;
      }

      list($oldkey, $newkey, $source) = explode(";", $line, 3);
      if ($section == "context") {
        $classifications[] = $newkey;
      }
      if ($newkey == '-' || $source == '-') {
        continue;
      }
      $oldkey = drupal_strtolower($oldkey);
      if (!isset($mapping[$section][trim($oldkey)])) {
        $mapping[$section][trim($oldkey)] = array();
      }
      $mapping[$section][drupal_strtolower(trim($oldkey))][] = array(
        'key' => drupal_strtolower(trim($newkey)),
        'source' => $source,
      );
    }
    return $mapping;
  }

  /**
   * refactor plan markxml array
   * change ontology keys with their old values and so on.
   * this function will be called after parsing the xml, if needed
   *
   * @param array $return_array
   */
  public function refactorMarkXML(&$return_array) {

    $tmp_array = array();
    foreach ($return_array["024"][8][" "] as $arr) {
      foreach ($arr as $k => $v) {

        if (isset($tmp_array[$k]) && !is_array($tmp_array[$k])) {
          $tmp_array[$k] = array($tmp_array[$k]);
        }

        if (isset($tmp_array[$k]) && is_array($tmp_array[$k])) {
          $tmp_array[$k][] = $v;
          continue;
        }

        $tmp_array[$k] = $v;
      }
    }
    $return_array["024"][8][" "] = $tmp_array;

    if (isset($return_array["245"]["0"][" "][0]['k']) && is_array($return_array["245"]["0"][" "][0]['k'])) {
      foreach ($return_array["245"]["0"][" "][0]['k'] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }

    if (isset($return_array["919"][" "][" "][0]['x']) && is_array($return_array["919"][" "][" "][0]['x'])) {
      foreach ($return_array["919"][" "][" "][0]['x'] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }
    if (isset($return_array["919"][" "][" "][0]['y']) && is_array($return_array["919"][" "][" "][0]['y'])) {
      foreach ($return_array["919"][" "][" "][0]['y'] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }
    if (isset($return_array["980"]["1"][" "][0]['b']) && is_array($return_array["980"]["1"][" "][0]['b'])) {
      foreach ($return_array["980"]["1"][" "][0]['b'] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }
    if (isset($return_array["980"]["1"][" "][0]['c']) && is_array($return_array["980"]["1"][" "][0]['c'])) {
      foreach ($return_array["980"]["1"][" "][0]['c'] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }

    //Removed old ontology replacement
    /* if (isset($return_array["591"][" "][" "][0][2])) {
      list($a, $b, $return_array["591"][" "][" "][0][2], $c) = explode(":", $return_array["591"][" "][" "][0][2], 4);
      $this->getTermByKey($return_array["591"][" "][" "][0][2]);
      } */


    if (isset($return_array["592"][" "][" "][0][2]) && is_array($return_array["592"][" "][" "][0][2])) {
      foreach ($return_array["592"][" "][" "][0][2] as &$val) {
        list($val, $tmp) = explode(":", $val, 2);
        $this->getTermByKey($val);
      }
    }

    if (isset($return_array["593"][" "][" "][0][2])) {
      list($a, $b, $return_array["593"][" "][" "][0][2], $c) = explode(":", $return_array["593"][" "][" "][0][2], 4);
      $this->getTermByKey($return_array["593"][" "][" "][0][2]);
    }

    if (isset($return_array["594"][" "][" "][0][2])) {
      list($a, $b, $return_array["594"][" "][" "][0][2], $c) = explode(":", $return_array["594"][" "][" "][0][2], 4);
      $this->getTermByKey($return_array["594"][" "][" "][0][2]);
    }

    if (isset($return_array["595"][" "][" "][0][2])) {
      list($a, $b, $return_array["595"][" "][" "][0][2], $c) = explode(":", $return_array["595"][" "][" "][0][2], 4);
    }
    static $bla2 = array();
    foreach ($return_array["084"][" "][" "] as &$arr) {
      foreach ($arr as $k => &$tv) {
        if ($k == "2") {
          continue;
        }
        if (is_array($tv)) {
          foreach ($tv as $k2 => &$v) {
            if ($k == "a" && $k2 == 0) {
              continue;
            }
            if (!isset($bla2[$v])) {
              $bla2[$v] = $this->soapClient->get_individual($v, "eng");
            }
            $v = $bla2[$v];
          }
        }
        else {
          if (!isset($bla2[$tv])) {
            $bla2[$tv] = $this->soapClient->get_individual($tv, "eng");
          }
          $tv = $bla2[$tv];
        }
      }
    }
  }

  /**
   * Return 2-char lang code from given 3-char langcode
   *
   * @param string $oldkey
   *   the old 3-char lang code
   *
   * @return string
   *   the parsed language, if nothing found return 'en'
   */
  private function parseLangKey($oldkey) {
    switch ($oldkey) {
      case 'fre':
        return 'fr';

      case 'ita':
        return 'it';

      case 'deu':
        return 'de';

      default:
        return 'en';
    }
  }

  /**
   * get marc 21 cache data
   *
   * @return string
   *   xml data
   */
  protected function getMarc21Source() {
    global $user;

    $url = 'http://biblio.friportail.ch/search?ln=de&p=&f=&action_search=Suchen&sf=&so=d&rg=50000&sc=0&of=xm';
    // 1 Day
    $life_time = (60 * 60 * 24);
    $ident = 'archibald_fritic_marc21_cache';

    return $this->getTempFile($url, $life_time, $ident);
  }
}

