<?php

/**
 * @file
 * lom resource add/edit form part: for education tab
 */

/**
 * part of lom resource add/edit form
 *  tab: education
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 * @param string $language
 */
function archibald_content_form_education(&$form, &$form_state, &$lom, $language) {
  $lom_education = $lom->getEducation();
  if (!($lom_education instanceof ArchibaldLomDataEducation)) {
    $lom_education = new ArchibaldLomDataEducation();
  }

  $lom_general = $lom->getGeneral();
  if (!($lom_general instanceof ArchibaldLomDataGeneral)) {
    $lom_general = new ArchibaldLomDataGeneral();
  }

  // learningResourceType
  $default_value = archibald_lom_vocab_default_val($lom_education->getLearningResourceType());

  $form['education'] = array(
    '#type' => 'fieldset',
    '#title' => archibald_get_field_label('education'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'vt_main',
    '#tree' => TRUE,
    '#theme' => 'archibald_education',
  );

  $options = archibald_get_learning_resource_type_options($default_value);
  $form['education']['learningResourceType_analog'] = array(
    '#type' => 'checkboxes',
    '#attributes' => array(
      'archibald_help_selector' => 'for|edit-education-learningresourcetype',
      'archibald_help_ident' => 'education.learning_resource_type',
    ),
    '#options' => $options['analog'],
    '#default_value' => $default_value,
  );
  $form['education']['learningResourceType_digital'] = array(
    '#type' => 'checkboxes',
    '#attributes' => array(
      'archibald_help_selector' => 'for|edit-education-learningresourcetype',
      'archibald_help_ident' => 'education.learning_resource_type',
    ),
    '#options' => $options['digital'],
    '#default_value' => $default_value,
  );


  // intendedEndUserRole
  $default_value = archibald_lom_vocab_default_val($lom_education->getIntendedEndUserRole());

  $form['education']['intendedEndUserRole'] = array(
    '#type' => 'checkboxes',
    '#title' => archibald_get_field_label('education.intendedEndUserRole') .
      archibald_get_require_type_value('education.intendedEndUserRole'),
    '#attributes' => array(
      'archibald_help_selector' => 'for|edit-education-intendedenduserrole',
      'archibald_help_ident' => 'education.intended_end_user_role',
    ),
    '#options' => archibald_get_intended_end_user_role_options($default_value),
    '#default_value' => $default_value,
  );

  // typicalAgeRange
  $default_value_from = 0;
  $default_value_to   = 0;
  $default_value      = '';
  $lang_string        = $lom_education->getTypicalAgeRange(0);
  if ($lang_string instanceof ArchibaldLomDataLangString) {
    $default_value = ArchibaldLom::getLangstringText($lang_string, 'en');
  }

  if (preg_match('/^([\-U0-9]{0,3})\-([\-U0-9]{0,3})$/i', (string) $default_value, $hits)) {
    $default_value_from = (INT) $hits[1];
    $default_value_to = (INT) $hits[2];
  }

  $form['education']['typicalAgeRange'] = array(
    '#type' => 'markup',
    '#title' => archibald_get_field_label('education.typicalAgeRange') .
      archibald_get_require_type_value('education.typicalAgeRange'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => TRUE,
    '#theme' => 'archibald_education_typical_age_range',
  );

  $options = range(0, 99);
  $options = array_combine($options, $options);
  $options[0] = t('Unknown');
  $form['education']['typicalAgeRange']['from'] = array(
    '#type' => 'select',
    '#title' => t('from'),
    '#default_value' => $default_value_from,
    '#options' => $options,
  );
  $form['education']['typicalAgeRange']['to'] = array(
    '#type' => 'select',
    '#title' => t('to'),
    '#default_value' => $default_value_to,
    '#options' => $options,
  );


  // typicalLearningTime
  $default_value = $lom_education->getTypicalLearningTime();
  if ($default_value instanceof ArchibaldLomDataDuration) {
    $default_value = $default_value->getDurationIn('minutes');
  }
  else {
    $default_value = 0;
  }

  $form['education']['typicalLearningTime'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('education.typicalLearningTime') .
      archibald_get_require_type_value('education.typicalLearningTime'),
    '#default_value' => archibald_get_education_typical_lerning_time($default_value, TRUE),
    '#options' => array('' => t('Unknown')) + archibald_get_education_typical_lerning_time_options(),
  );
  # $form['education']['typicalLearningTime'] =
  //  archibald_duration_form(
  //     $default_value,
  //     'Typical learning time',
  //     FALSE,
  //     'dt',
  //     array(
  //       'archibald_help_selector' => 'for|edit-duration',
  //       'archibald_help_ident' => 'education.typical_learning_time'
  //       )
  //     );
  //
  // Difficulty

  $default_value = archibald_lom_vocab_default_val(
    $lom_education->getDifficult()
  );

  $form['education']['difficulty'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('education.difficulty') .
      archibald_get_require_type_value('education.difficulty'),
    '#default_value' => $default_value,
    '#attributes' => array('archibald_help_ident' => 'education.difficulty'),
    '#options' => array_merge(array('' => t('Unknown')), archibald_get_education_difficulty_options()),
  );

  // Aggregation level
  $default_value = $lom_general->getAggregationLevel();
  if ($default_value instanceof ArchibaldLomDataVocabulary) {
    $default_value = $default_value->getValue();
  }
  $form['education']['aggregationLevel'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('education.aggregationLevel') .
      archibald_get_require_type_value('education.aggregationLevel'),
    '#default_value' => $default_value,
    '#attributes' => array(
      'archibald_help_ident' => 'general.aggregationLevel',
      'style' => 'width:250px;'
    ),
    '#options' => array(
      '' => t('Unknown'),
      1 => t('1: Raw media data or fragments'),
      2 => t('2: A lesson'),
      3 => t('3: A course'),
      4 => t('4: A collection of courses'),
    ),
  );


  // Description
  $default_value = '';
  $lang_string = $lom_education->getDescription();
  if ($lang_string instanceof ArchibaldLomDataLangString) {
    $default_value = $lang_string->getString($language);
  }
  $form['education']['description'] = array(
    '#type' => 'textarea',
    '#title' => archibald_get_field_label('education.description') .
      archibald_get_require_type_value('education.description'),
    '#default_value' => $default_value,
    '#rows' => 5,
    '#attributes' => array(
      'class' => array(
        'lang_string_autocomplete',
        'lang_string',
      ),
      'archibald_help_ident' => 'education.description',
    ),
  );


  if (empty($default_value) || $lom->getLomId() == '') {
    $type = 'value';
  }
  else {
    $type = 'checkbox';
  }

  /**
   * @cha, 26.06.2012
   * Disabled spellcheck
   *
   * $form['education']['description_spell_check'] = array(
   *   '#type' => $type,
   *   '#title' => t('Spell check'),
   *   '#default_value' => 1,
   *   '#attributes' => array(
   *     'class' => array('lang_string_type_error'),
   *   ),
   * );
   */
}

/**
 * generate options list for drupal form api select
 *
 * @param string $default_value
 *
 * @return array
 */
function archibald_get_learning_resource_type_options(&$default_value) {
  $options = array();
  $options['analog'] = archibald_get_taxonomy_options(
    'learning_resourcetype', 'analog'
  );

  $options['digital'] = archibald_get_taxonomy_options(
    'learning_resourcetype', 'digital'
  );

  // extract vocabulars
  $default_value_assosiativ = array();
  foreach ($default_value as $value) {
    $k = archibald_select_options_default_value_filter_splitter($value);
    $default_value_assosiativ[$k] = $value;
  }

  // extract vocabulars
  $options_assosiativ = array();
  foreach ($options as $type => $values) {
    foreach ($values as $key => $value) {
      $k = archibald_select_options_default_value_filter_splitter($key);
      $options_assosiativ[$k] = $key;
    }
  }

  // find default_values within current excisting options
  $default_value_new = array();
  foreach ($default_value_assosiativ as $value => $vocabular) {
    if (!empty($options_assosiativ[$value])) {
      $default_value_new[] = $options_assosiativ[$value];
      unset($default_value_assosiativ[$value]);
    }
  }

  // add not exisiting default_values to options lists
  if (!empty($default_value_assosiativ)) {
    foreach ($default_value_assosiativ as $value => $vocabular) {
      $options['analog'][$vocabular] = t($value);
      $default_value_new[] = $vocabular;
    }
  }
  $default_value = $default_value_new;

  ksort($options['analog']);
  ksort($options['digital']);

  return $options;
}

/**
 * get option array
 * for a checkboxes / select form item
 * with all possible user_role taxonomy terms
 *
 * @param string $default_value
 *
 * @return array
 */
function archibald_get_intended_end_user_role_options($default_value) {
  $options = archibald_get_taxonomy_options('intended_enduserrole');
  archibald_select_options_default_value_filter($options, $default_value);

  ksort($options);

  return $options;
}

/**
 * get option array
 * for a checkboxes / select form item
 * with all possible difficulty taxonomy terms
 *
 * @return array
 */
function archibald_get_education_difficulty_options() {
  return archibald_get_taxonomy_options('difficulty');
}

/**
 * get option array
 * for a checkboxes / select form item
 * with all possible learning_time taxonomy terms
 *
 * @return array
 */
function archibald_get_education_typical_lerning_time_options() {
  return archibald_get_education_typical_lerning_time();
}

/**
 * theme a archibald form element typical age range
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_education_typical_age_range($form) {
  $form = reset($form);

  $output  = '<label for="edit-education_typical_age_range" archibald_help_ident="education.typical_age_range" archibald_help_selector="for|edit-education_typical_age_range">';
  $output .= $form['#title'];
  $output .= '</label>';
  $output .= '<div class="archibald_education_typical_age_range">';
  $output .= theme($form['from']['#type'], $form['from']) . ' - ' . theme($form['to']['#type'], $form['to']);
  $output .= '</div>';

  return $output;
}

/**
 * theme a archibald form block education
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_education($form) {
  $form = reset($form);

  $html = array(
    'top' => array(),
    'left' => array(),
    'right' => array(),
    'bottom' => array(),
  );
  foreach (element_children($form, TRUE) as $key) {
    switch ($key) {
      case 'learningResourceType_analog':
      case 'learningResourceType_digital':
        $html['left'][] = '<div class="' . $key . '">' . drupal_render($form[$key]) . '</div>';
        break;

      case 'description':
      /**
       * @cha, 26.06.2012
       * Disabled spellcheck
       *
       * case 'description_spell_check':
       */
        $html['top'][] = drupal_render($form[$key]);
        break;

      default:
        $html['right'][] = drupal_render($form[$key]);
        break;
    }
  }

  $output = '<label for="edit-relations-title" style="display: inline;">' . $form['#title'] . '</label>';
  $output .= '<table id="archibald_education" border="0">';

  $top = implode("\n", $html['top']);
  if (!empty($top)) {
    $output .= '<tr>';
    $output .= '<td class="archibald_education_td archibald_education_td_top" colspan=2>';
    $output .= $top;
    $output .= '</td>';
    $output .= '</tr>';
  }

  $left = implode("\n", $html['left']);
  $right = implode("\n", $html['right']);
  if (!empty($left) || !empty($right)) {
    $output .= '<tr>';
    if (!empty($left)) {
      $output .= '<td class="archibald_education_td archibald_education_td_left">';

      $output .= ' <label for="edit-education-learningresourcetype-analog" style="display: block;">' .
        t('Learning resource type') .
        archibald_get_require_type_value('education.learningResourceType');

      $output .= '</label>';

      $output .= $left;
      $output .= '<div class="clearfix"></div>';
      $output .= '</td>';
    }
    if (!empty($right)) {
      $output .= '<td class="archibald_education_td archibald_education_td_right">';
      $output .= $right;
      $output .= '</td>';
      $output .= '</tr>';
    }
  }
  $output .= '</table>';

  return $output;
}

/**
 * save handler for lom education form part
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 */
function archibald_content_form_education_submit($form, &$form_state, &$lom) {
  $lom_education = $lom->getEducation();
  if (!($lom_education instanceof ArchibaldLomDataEducation)) {
    $lom_education = new ArchibaldLomDataEducation();
  }

  $lom_general = $lom->getGeneral();
  if (!($lom_general instanceof ArchibaldLomDataGeneral)) {
    $lom_general = new ArchibaldLomDataGeneral();
  }


  ##### handle learningResourceType
  if (archibald_check_field_permission('education learningResourceType')) {
    $learning_resource_types = array();
    $selected_options = array_merge(
      $form_state['values']['education']['learningResourceType_analog'],
      $form_state['values']['education']['learningResourceType_digital']
    );

    foreach ($selected_options as $i => $learning_resource_type) {
      if (empty($learning_resource_type)) {
        continue;
      }

      $key = drupal_strtolower(trim($learning_resource_type));
      if (strpos($key, '|') !== FALSE) {
        list($tmp_source, $key) = explode('|', $key, 2);
      }

      if (empty($key) || !preg_match('/(.+)\|(.+)/', $learning_resource_type, $parts)) {
        continue;
      }

      $vocabulary = new ArchibaldLomDataVocabulary();
      $vocabulary->setSource($parts[1]);
      $vocabulary->setValue($parts[2]);
      $learning_resource_types[] = $vocabulary;
    }

    $lom_education->setLearningResourceType($learning_resource_types);
  }

  ##### handle intendedEndUserRole
  if (archibald_check_field_permission('education intendedEndUserRole')) {
    $existing_intended_end_user_roles = array();
    foreach ($lom_education->getIntendedEndUserRole() as $intended_end_user_role) {
      if ($intended_end_user_role instanceof ArchibaldLomDataVocabulary) {
        $key = $intended_end_user_role->getSource() . '|' . $intended_end_user_role->getValue();
      }
      else {
        // this shoud realy never happen
        continue;
      }

      $existing_intended_end_user_roles[$key] = $intended_end_user_role;
    }

    foreach ($form_state['values']['education']['intendedEndUserRole'] as $i => $intended_end_user_role) {
      $key = drupal_strtolower(trim($intended_end_user_role));

      if (empty($key) || !preg_match('/(.+)\|(.+)/', $intended_end_user_role, $parts)) {
        continue;
      }

      if (isset($existing_intended_end_user_roles[$key])) {
        unset($existing_intended_end_user_roles[$key]);
      }
      else {
        $vocabulary = new ArchibaldLomDataVocabulary();
        $vocabulary->setSource($parts[1]);
        $vocabulary->setValue($parts[2]);
        $lom_education->addIntendedEndUserRole($vocabulary);
      }
    }

    foreach ($existing_intended_end_user_roles as $key => $intended_end_user_role) {
      $lom_education->removeIntendedEndUserRole($intended_end_user_role);
    }
  }
  ##### handle typicalAgeRange
  if (archibald_check_field_permission('education typicalAgeRange')) {
    $string = $form_state['values']['education']['typicalAgeRange']['from'] . '-' . $form_state['values']['education']['typicalAgeRange']['to'];
    $lang_string = new ArchibaldLomDataLangString();
    $lang_string->setString($string, 'en');
    $lom_education->setTypicalAgeRange($lang_string, 0);
  }
  ##### handle typicalLearningTime
  if (archibald_check_field_permission('education typicalLearningTime')) {
    $typical_learning_time = new ArchibaldLomDataDuration();
    $typical_learning_time->setDuration(array(
        'minutes' => $form_state['values']['education']['typicalLearningTime'],
      )
    );
    $lom_education->setTypicalLearningTime($typical_learning_time);
  }
  ##### handle description
  if (archibald_check_field_permission('education description')) {
    /**
     * @cha, 26.06.2012
     * Disabled spellcheck
     *
     * $lom_education->getDescription(), $form_state['values']['language'], $form_state['values']['education']['description'], $form_state['values']['education']['description_spell_check']     *
     */
    $lom_education->setDescription(
      archibald_langstring_submit($lom_education->getDescription(), $form_state['values']['language'], $form_state['values']['education']['description'], FALSE)
    );
  }
  ##### handle difficulty
  if (archibald_check_field_permission('education difficulty')) {
    if (isset($form_state['values']['education']['difficulty'])) {

      if (empty($form_state['values']['education']['difficulty'])) {
        $lom_education->setDifficult(new ArchibaldLomDataVocabulary());
      }
      elseif (preg_match('/(.+)\|(.+)/', $form_state['values']['education']['difficulty'], $parts)) {

        $vocabulary = new ArchibaldLomDataVocabulary();
        $vocabulary->setSource($parts[1]);
        $vocabulary->setValue($parts[2]);
        $lom_education->setDifficult($vocabulary);
      }
    }
  }
  ##### handle Aggregation level
  if (archibald_check_field_permission('education aggregationLevel')) {
    if (isset($form_state['values']['education']['aggregationLevel'])) {
      if (empty($form_state['values']['education']['aggregationLevel'])) {
        $lom_general->setAggregationLevel(new ArchibaldLomDataVocabulary());
      }
      else {
        $vocabulary = new ArchibaldLomDataVocabulary();
        $vocabulary->setValue($form_state['values']['education']['aggregationLevel']);
        $lom_general->setAggregationLevel($vocabulary);
      }
    }
  }
  $lom->setGeneral($lom_general);
  $lom->setEducation($lom_education);
}

