<?php

/**
 * @file
 * lom resource add/edit form part: for keywords tab
 */

/**
 * part of lom resource add/edit form
 *  tab: keywords
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 * @param string $language
 */
function archibald_content_form_keywords(&$form, &$form_state, &$lom, $language) {

  $lom_general = $lom->getGeneral();
  if (!($lom_general instanceof ArchibaldLomDataGeneral)) {
    $lom_general = new ArchibaldLomDataGeneral();
  }

  $keyword_access = archibald_check_field_permission('keywords keyword');
  $coverage_access = archibald_check_field_permission('keywords coverage');

  if ($keyword_access || $coverage_access) {
    $form['keywords'] = array(
      '#type' => 'fieldset',
      '#title' => t('Keywords'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#group' => 'vt_main',
      '#tree' => TRUE,
      '#theme' => 'archibald_keywords',
    );
  }

  if ($keyword_access) {

    /**
     * START Keyword handling
     */
    $form['keywords']['keyword'] = array(
      '#type' => 'fieldset',
      '#title' => '',
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
      '#attributes' => array('archibald_help_ident' => 'general.keyword'),
      '#prefix' => '<div class="clearfix" id="archibald_general_keyword-wrapper">',
      '#suffix' => '</div>',
      '#theme' => 'archibald_general_keyword',
    );

    /** Load keywords submitted by add more button * */
    $num_general_keyword = 0;
    if (!isset($form_state['num_general_keyword'])) {
      $form_state['num_general_keyword'] = 0;
    }
    $keywords = $lom_general->getKeyword();


    if (!empty($keywords)) {
      foreach ($keywords as $keyword) {
        $default_value = '';
        if ($keyword instanceof ArchibaldLomDataLangString) {
          $default_value = $keyword->getString($language);
          if (empty($default_value)) {
            $tmp = $keyword->getStrings();
            if (!empty($tmp)) {
              $tmp_keys      = array_keys($tmp);
              $tmp_key       = reset($tmp_keys);
              $default_value = $tmp[$tmp_key] . ' (' . $tmp_key . ') ' . t('Please translate');
            }
          }
        }

        archibald_content_form_general_add_keyword($form, $form_state, 'keyword_' . $keyword->getId(), $default_value);

        if (!empty($form_state['values']['keywords']['keyword']['add_more_general_keyword'])) {
          $form_state['num_general_keyword']++;
        }
        $num_general_keyword++;
      }
    }

    if (empty($form_state['num_general_keyword']) || $num_general_keyword == $form_state['num_general_keyword']) {
      $form_state['num_general_keyword'] += 3;
    }

    for ($i = $num_general_keyword; $i < $form_state['num_general_keyword']; $i++) {
      archibald_content_form_general_add_keyword($form, $form_state);
    }

    $form['keywords']['keyword']['add_more_general_keyword'] = array(
      '#type' => 'submit',
      '#value' => t('Add 3 tags'),
      '#weight' => 9999,
      '#limit_validation_errors' => array(),
      //submit is required to use #limit_validation_errors
      '#submit' => array(
        'archibald_content_form_add_more_general_keyword_submit',
      ),
      '#ajax' => array(
        'callback' => 'archibald_ajax_content_form_general_add_empty_keyword',
        'wrapper' => 'archibald_general_keyword-wrapper',
        'method' => 'replace',
        #'effect'   => 'fade',
      ),
    );
  }


  if ($coverage_access) {

    /**
     * START coverage handling
     */
    $form['keywords']['coverage'] = array(
      '#type' => 'fieldset',
      '#title' => '',
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#tree' => TRUE,
      '#attributes' => array('archibald_help_ident' => 'general.coverage'),
      '#prefix' => '<div class="clearfix" id="archibald_general_coverage-wrapper">',
      '#suffix' => '</div>',
      '#theme' => 'archibald_general_coverage',
    );

    /** Load coverage submitted by add more button * */
    $num_general_coverage = 0;
    if (!isset($form_state['num_general_coverage'])) {
      $form_state['num_general_coverage'] = 0;
    }

    $coverages = $lom_general->getCoverage();
    if (!empty($coverages)) {
      foreach ($coverages as $coverage) {
        $default_value = '';
        if ($coverage instanceof ArchibaldLomDataLangString) {
          $default_value = $coverage->getString($language);

          if (empty($default_value)) {
            $tmp = $coverage->getStrings();
            if (!empty($tmp)) {
              $tmp_keys      = array_keys($tmp);
              $tmp_key       = reset($tmp_keys);
              $default_value = $tmp[$tmp_key] . ' (' . $tmp_key . ') ' . t('Please translate');
            }
          }
        }

        archibald_content_form_general_add_coverage($form, $form_state, $language, 'coverage_' . $coverage->getId(), $default_value);

        if (!empty($form_state['values']['keywords']['coverage']['add_more_general_coverage'])) {
          $form_state['num_general_coverage']++;
        }
        $num_general_coverage++;
      }
    }

    if (empty($form_state['num_general_coverage']) || $num_general_coverage == $form_state['num_general_coverage']) {
      $form_state['num_general_coverage'] += 3;
    }

    for ($i = $num_general_coverage; $i <= $form_state['num_general_coverage']; $i++) {
      archibald_content_form_general_add_coverage($form, $form_state, $language);
    }

    $form['keywords']['coverage']['add_more_general_coverage'] = array(
      '#type' => 'submit',
      '#value' => t('Add 3 coverage'),
      '#weight' => 9999,
      '#limit_validation_errors' => array(),
      // #submit is required to use #limit_validation_errors
      '#submit' => array(
        'archibald_content_form_add_more_general_coverage_submit',
      ),
      '#ajax' => array(
        'callback' => 'archibald_ajax_content_form_general_add_empty_coverage',
        'wrapper' => 'archibald_general_coverage-wrapper',
        'method' => 'replace',
        #'effect'   => 'fade',
      ),
    );
  }
}

/**
 * add new general keyword, this could called by default form or ajax callback
 *
 * @param array $form
 * @param array $form_state
 * @param string $id
 *   the id of keyword
 * @param string $default_value
 *   default value
 *   is only used in cae of a untranslated langString
 */
function archibald_content_form_general_add_keyword(
  &$form, &$form_state, $id = '', $default_value = ''
) {

  if (empty($id)) {
    $i = 0;
    foreach (array_keys($form['keywords']['keyword']) as $k) {
      if (drupal_substr($k, 0, 8) == 'keyword_') {
        $i++;
      }
    }
    $id = 'keyword_' . $i;
  }

  $form['keywords']['keyword'][$id] = array(
    '#type' => 'textfield',
    '#title' => archibald_get_field_label('general.keyword'),
    '#autocomplete_path' => 'archibald/autocomplete/langstring',
    '#default_value' => $default_value,
    '#attributes' => array('class' => array('general_keyword')),
    '#size' => 40,
  );
}

/**
 * ajax menu callback
 * place new empty keywords fields on bottom of keywords list
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function archibald_ajax_content_form_general_add_empty_keyword($form, $form_state) {
  return $form['keywords']['keyword'];
}

$_archibald_keyword_id_count = 0;

/**
 * theme a archibald form element general keywords
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_general_keyword($form) {
  global $_archibald_keyword_id_count;
  $form = reset($form);

  $output = '<label for="edit-general-keyword-' . $_archibald_keyword_id_count++ . '">' . t('Tags') .
    archibald_get_require_type_value('general.keyword');

  $output .= '</label>';

  $output .= '<div id="archibald_general_keyword">';

  foreach ($form as $k => $element) {
    if (drupal_substr($k, 0, 8) == 'keyword_') {
      unset($element['#title']);
      $output .= drupal_render($element);
    }
  }

  $output .= '</div>';
  $output .= drupal_render($form['add_more_general_keyword']);

  return $output;
}

/**
 * add new general coverage, this could called by default form or ajax callback
 *
 * @param array $form
 * @param array $form_state
 * @param string $id
 *   the id of keyword
 * @param string $default_value
 *   default value
 *   is only used in cae of a untranslated langString
 */
function archibald_content_form_general_add_coverage(&$form, &$form_state, $language, $id = '', $default_value = '') {

  if (empty($id)) {
    $i = 0;
    foreach (array_keys($form['keywords']['coverage']) as $k) {
      if (drupal_substr($k, 0, 9) == 'coverage_') {
        $i++;
      }
    }
    $id = 'coverage_' . $i;
  }

  $form['keywords']['coverage'][$id] = array(
    '#type' => 'textfield',
    '#title' => archibald_get_field_label('general.coverage'),
    '#autocomplete_path' => 'archibald/autocomplete/coverage/' . $language,
    '#default_value' => $default_value,
    '#attributes' => array('class' => array('general_coverage')),
    '#size' => 40,
  );
}

/**
 * theme a archibald form element general coverage
 *
 * @param array $form
 *
 * @return string
 */
function archibald_ajax_content_form_general_add_empty_coverage($form, $form_state) {

  return $form['keywords']['coverage'];
}

$_archibald_coverage_id_count = 0;

/**
 * theme a archibald form element general coverages
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_general_coverage($form) {
  global $_archibald_coverage_id_count;
  $form = reset($form);

  $output = '<label for="edit-general-coverage-' . $_archibald_coverage_id_count++ . '">' . archibald_get_field_label('general.coverage') .
    archibald_get_require_type_value('general.coverage');

  $output .= '</label>';
  $output .= '<div id="archibald_general_coverage">';

  foreach ($form as $k => $element) {
    if (drupal_substr($k, 0, 9) == 'coverage_') {
      unset($element['#title']);
      $output .= drupal_render($element);
    }
  }

  $output .= '</div>';
  $output .= drupal_render($form['add_more_general_coverage']);

  return $output;
}

/**
 * Submit handler for the "Add more" button.
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_form_add_more_general_keyword_submit($form, &$form_state) {

  $form_state['num_general_keyword'] += 3;
  $form_state['rebuild'] = TRUE;
}

/**
 * Submit handler for the "Add more" button.
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_form_add_more_general_coverage_submit($form, &$form_state) {

  $form_state['num_general_coverage'] += 3;
  $form_state['rebuild'] = TRUE;
}

/**
 * theme a archibald form block genera
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_keywords($form) {
  $form = reset($form);

  $output = '<label for="edit-relations-title" style="display: inline;">' . $form['#title'] . '</label>';
  $output .= '<table id="archibald_general" border="0">';

  $html = array('left' => '', 'right' => '', 'bottom' => '');
  foreach (element_children($form, TRUE) as $key) {
    switch ($key) {
      case 'keyword':
        $html['left'][] = drupal_render($form[$key]);
        break;

      case 'coverage':
        $html['right'][] = drupal_render($form[$key]);
        break;

      default:
        $html['bottom'][] = drupal_render($form[$key]);
        break;
    }
  }

  if (empty($html['right']) || empty($html['left'])) {
    $output .= '<tr>';
    $output .= '<td class="archibald_general_td ' . 'archibald_general_td_middle" colspan=2>';
    $output .= (!empty($html['left'])) ? implode("\n", $html['left']) : '';
    $output .= (!empty($html['right'])) ? implode("\n", $html['right']) : '';
    $output .= '</td>';
    $output .= '</tr>';
  }
  else {
    $output .= '<tr>';
    $output .= '<td class="archibald_general_td ' . 'archibald_general_td_middle_left">';
    $output .= implode("\n", $html['left']);
    $output .= '</td>';
    $output .= '<td class="archibald_general_td ' . 'archibald_general_td_middle_right">';
    $output .= implode("\n", $html['right']);
    $output .= '</td>';
    $output .= '</tr>';
  }

  if (!empty($html['bottom'])) {
    $output .= '<tr>';
    $output .= '<td class="archibald_general_td ' . 'archibald_general_td_bottom" colspan=2>';
    $output .= implode("\n", $html['bottom']);
    $output .= '</td>';
    $output .= '</tr>';
  }

  $output .= '</table>';

  return $output;
}

/**
 * save handler for lom general form part
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 */
function archibald_content_form_keywords_submit($form, &$form_state, &$lom) {
  $lom_general = $lom->getGeneral();
  if (!($lom_general instanceof ArchibaldLomDataGeneral)) {
    $lom_general = new ArchibaldLomDataGeneral();
  }

  ##### handle keyword
  if (archibald_check_field_permission('keywords keyword')) {
    $existing_keywords = array();
    $already_added = array();
    foreach ($lom_general->getKeyword() as $kw_id => $keyword) {
      if ($keyword instanceof ArchibaldLomDataLangString) {
        $key = drupal_strtolower(
          trim(
            $keyword->getString($form_state['values']['language'])
          )
        );
      }
      else {
        // this shoud realy never happen
        continue;
      }

      $existing_keywords[$keyword->getId()] = $kw_id;
    }

    foreach ($form_state['values']['keywords']['keyword'] as $i => $keyword) {

      // Prevents double inserting.
      if (isset($already_added[$keyword])) {
        continue;
      }
      $already_added[$keyword] = TRUE;

      $keyword_language = $form_state['values']['language'];
      if (drupal_substr($i, 0, 8) == 'keyword_') {
        /// if is a keyword with is not existing in current language we like to save it in original language
        if (preg_match('/(.+) \(([a-z]+)\) ' . preg_quote(t('Please translate')) . '$/i', $keyword, $result)) {
          $keyword = $result[1];
          $keyword_language = $result[2];
        }

        $key = drupal_substr($i, 8);

        if (drupal_strlen($key) == 0 || empty($keyword)) {
          continue;
        }

        if (isset($existing_keywords[$key])) {
          // update keyword
          $kw_id = $existing_keywords[$key];
          // load langsting
          $keyword_langstring = $lom_general->getKeyword($kw_id);
          $old_keyword_langstring = ArchibaldLomSaveHandler::requestLangStringObj($key);;
          // update langstring
          $keyword_langstring->setString($keyword, $keyword_language);
          // save langstring back to ArchibaldLomDataGeneral
          $lom_general->setKeyword($keyword_langstring, $kw_id);

          archibald_handle_new_revision_is_required($key, $old_keyword_langstring, $keyword_langstring);

          unset($existing_keywords[$key]);
        }
        else {
          // add new keyword
          $lang_string = new ArchibaldLomDataLangString();
          $lang_string->setString($keyword, $keyword_language);
          $lom_general->addKeyword($lang_string);

          #archibald_handle_new_revision_is_required($kw_id, new ArchibaldLomDataLangString(), $lang_string);
        }
      }
    }

    foreach ($existing_keywords as $key => $kw_id) {
      # $current_langstring = ArchibaldLomSaveHandler::requestLangStringObj($kw_id);
      #archibald_handle_new_revision_is_required($key, new ArchibaldLomDataLangString(), $current_langstring);
      // remove deleted keyword
      $lom_general->removeKeyword($kw_id);
    }
  }

  ##### handle coverage
  if (archibald_check_field_permission('keywords coverage')) {

    $existing_coverages = array();
    $already_added = array();
    foreach ($lom_general->getCoverage() as $kw_id => $coverage) {
      if ($coverage instanceof ArchibaldLomDataLangString) {
        $key = drupal_strtolower(
          trim($coverage->getString($form_state['values']['language']))
        );
      }
      else {
        // this shoud realy never happen
        continue;
      }

      $existing_coverages[$coverage->getId()] = $kw_id;
    }
    foreach ($form_state['values']['keywords']['coverage'] as $i => $coverage) {

      // Prevents double inserting.
      if (isset($already_added[$coverage])) {
        continue;
      }
      $already_added[$coverage] = TRUE;

      $coverage_language = $form_state['values']['language'];
      // if is a keyword with is not existing
      // in current language we like to save it in original language
      if (preg_match('/(.+) \(([a-z]+)\) ' . preg_quote(t('Please translate')) . '$/i', $coverage, $result)) {
        $coverage = $result[1];
        $coverage_language = $result[2];
      }

      if (drupal_substr($i, 0, 9) == 'coverage_') {
        $key = drupal_substr($i, 9);

        if (drupal_strlen($key) == 0 || empty($coverage)) {
          continue;
        }

        if (isset($existing_coverages[$key])) {
          // update keyword
          $kw_id = $existing_coverages[$key];

          // load langsting
          $coverage_langstring = $lom_general->getCoverage($kw_id);
          $old_langstring = ArchibaldLomSaveHandler::requestLangStringObj($key);;
          // update langstring
          $coverage_langstring->setString($coverage, $coverage_language);
          // save langstring back to ArchibaldLomDataGeneral
          $lom_general->setCoverage($coverage_langstring, $kw_id);
          archibald_handle_new_revision_is_required($key, $old_langstring, $coverage_langstring);
          unset($existing_coverages[$key]);
        }
        else {
          $lang_string = new ArchibaldLomDataLangString();
          $lang_string->setString($coverage, $coverage_language);
          $lom_general->addCoverage($lang_string);
        }
      }
    }
    foreach ($existing_coverages as $key => $kw_id) {
      $lom_general->removeCoverage($kw_id);
    }
  }

  $lom->setGeneral($lom_general);
}

