<?php

/**
 * @file
 * lom resource add/edit form part: for relation tab
 */
define('ARCHIBALD_RELATIONS_PER_PAGE', 14);

/**
 * part of lom resource add/edit form
 *  tab: relation
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 * @param string $language
 */
function archibald_content_form_relation(&$form, &$form_state, ArchibaldLom &$lom, $language) {

  global $language;
  // Include the CTools tools that we need.
  ctools_include('ajax');
  ctools_include('modal');

  // Add CTools' javascript to the page.
  ctools_modal_add_js();

  $lom_relations = $lom->getRelation();

  $parents           = '';
  $parent_suffix     = '';


  if (!isset($form_state['relations_parents'])) {
    $form_state['relations_parents'] = array();
    if ($lom->getLomId() == NULL) {

    }
    else {
      $result = db_select('archibald_relations', 'archibaldr')
        ->fields('archibaldr', array('lom_id '))
        ->condition('relation_id', $lom->getLomId())
        ->execute();

      while ($row = $result->fetchAssoc()) {
        $form_state['relations_parents'][$row['lom_id']] = TRUE;
      }
    }
  }

  $has_parent_sql = NULL;
  if (count($form_state['relations_parents']) > 0) {
    $has_parent_sql = TRUE;

    foreach ($form_state['relations_parents'] as $parent_lom_id => $true) {
      $form_state['relations_parents'][$parent_lom_id] = ArchibaldLom::determinTitle($parent_lom_id);
    }
  }

  $form['relations'] = array(
    '#type' => 'fieldset',
    '#title' => archibald_get_field_label('relation'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'vt_main',
    '#tree' => TRUE,
  );

  $form['relations']['relation'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => TRUE,
    '#attributes' => array('archibald_help_ident' => 'relation.relation'),
    '#prefix' => '<div class="clearfix" id="archibald_relations_relation-wrapper">',
    '#suffix' => '</div>',
  );


  if (count($form_state['relations_parents']) <= 0) {
    $form['relations']['relation']['#theme'] = 'archibald_relations_relation';
  }
  else {
    $form['relations']['relation']['#prefix'] .= "<div class='clearfix message warning' id='archibald_relations_relation-wrapper-has_parent_div'>" . t('This description is a child of a collection, so no relations can be added to this description') . "</div>";
    $form['relations']['relation']['#prefix'] .= t("Release relations");
  }
  foreach ($form_state['relations_parents'] as $lom_id => $title) {
    $form['relations']['relation'][$lom_id . '-' . $lom->getLomId()] = array(
      '#type' => 'submit',
      '#value' => t(
        'Release description: @title', array(
          '@title' => $title,
        )
      ),
      '#weight' => 9999,
      '#limit_validation_errors' => array(),
      // #submit is required to use #limit_validation_errors
      '#submit' => array(
        'archibald_content_form_release_relations_relation_submit',
      ),
      '#ajax' => array(
        'callback' => 'archibald_ajax_content_form_relations_add_empty_relation',
        'wrapper' => 'archibald_relations_relation-wrapper',
        'method' => 'replace',
      ),
      '#suffix' => '<br /><br />',
    );
  }

  // Load identfiers from lom object if nothing was commited.
  $num_relations = 0;
  if (!isset($form_state['num_relations'])) {
    $form_state['num_relations'] = 0;
  }

  if (!empty($lom_relations)) {
    foreach ($lom_relations as $relation) {

      if (!($relation instanceof ArchibaldLomDataRelation)) {
        continue;
      }

      if ($relation->getKind()->getValue() != ArchibaldLomDataRelation::KIND_HAS_PART) {
        continue;
      }

      $identifiers = $relation->getResource()->getIdentifier();
      foreach ($identifiers as $identifier) {
        $form['relations']['relation']['relation_' . $num_relations] = array(
          '#type' => 'fieldset',
          '#collapsible' => FALSE,
          '#tree' => TRUE,
          '#weight' => $num_relations,
        );

        $default_value = $identifier->getEntry();
        $lsh           = new ArchibaldLomSQLSearch();
        $default_name  = $lsh->getLomTitle($default_value);

        $form['relations']['relation']['relation_' . $num_relations]['name'] = array(
          '#type' => 'textfield',
          '#title' => archibald_get_field_label('relation.resource'),
          '#default_value' => $default_name,
          '#size' => 80,
          '#attributes' => array('class' => array('relation_ressource')),
        );

        $form['relations']['relation']['relation_' . $num_relations]['ressource'] = array(
          '#type' => 'hidden',
          '#default_value' => $default_value,
        );
      }

      if (!empty($form_state['values']['relations']['relation']['add_more_relation'])) {
        $form_state['num_relations']++;
      }
      $num_relations++;
    }
  }



  if (empty($form_state['num_relations'])) {
    $form_state['num_relations'] = 1;
  }
  if (empty($has_parent_sql)) {
    for ($i = $num_relations; $i <= $form_state['num_relations']; $i++) {
      archibald_content_form_relations_add_empty_relation($form, $form_state);
    }

    $form['relations']['relation']['add_more_relation'] = array(
      '#type' => 'submit',
      '#value' => t('Add a relation'),
      '#weight' => 9999,
      '#limit_validation_errors' => array(),
      // #submit is required to use #limit_validation_errors
      '#submit' => array(
        'archibald_content_form_add_more_relations_relation_submit',
      ),
      '#ajax' => array(
        'callback' => 'archibald_ajax_content_form_relations_add_empty_relation',
        'wrapper' => 'archibald_relations_relation-wrapper',
        'method' => 'replace',
      ),
    );
  }
}

/**
 * save handler for each single relation
 * it clear the cached of the relate resources to show
 * there also the new relation
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_form_release_relations_relation_submit($form, &$form_state) {

  reset($form_state['values']['relations']['relation']);

  list($release_id, $relation_id) = explode('-', key($form_state['values']['relations']['relation']), 2);

  db_delete('archibald_relations')->condition('lom_id', $release_id)->condition('relation_id', $relation_id)->execute();

  $version_release = db_query('
    SELECT version
    FROM {archibald_lom}
    WHERE lom_id=:lom_id
    ORDER BY save_time DESC
    LIMIT 1',
    array(
      'lom_id' => $release_id,
    )
  )->fetchColumn(0);

  $version_relation = db_query('
    SELECT version
    FROM {archibald_lom}
    WHERE lom_id=:lom_id
    ORDER BY save_time DESC
    LIMIT 1',
    array(
      'lom_id' => $relation_id,
    )
  )->fetchColumn(0);

  cache_clear_all('lom:' . $relation_id . '__version:' . $version_relation, 'cache');
  cache_clear_all('lom:' . $release_id . '__version:' . $version_release, 'cache');

  unset($form_state['relations_parents'][$release_id]);

  $form_state['rebuild'] = TRUE;
}

/**
 * ajax handler to relaease a child resource from its mother
 *
 * @return string
 */
function archibald_ajax_content_form_relations_release_relation() {
  return '<div></div>';
}

/**
 * add new general keyword, this could called by default form or ajax callback
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_form_relations_add_empty_relation(&$form, &$form_state) {

  $i = 0;
  foreach (array_keys($form['relations']['relation']) as $k) {
    if (preg_match('/^relation_/', $k)) {
      $i++;
    }
  }

  $form['relations']['relation']['relation_' . $i] = array(
    '#type' => 'fieldset',
    '#collapsible' => FALSE,
    '#tree' => TRUE,
    '#weight' => $i,
  );

  $form['relations']['relation']['relation_' . $i]['name'] = array(
    '#type' => 'textfield',
    '#size' => 80,
    '#title' => t('Description'),
    '#attributes' => array('class' => array('relation_ressource')),
  );

  $form['relations']['relation']['relation_' . $i]['ressource'] = array(
    '#type' => 'hidden',
    '#default_value' => '',
  );
}

/**
 * ajax menu callback
 * place new empty relation fields on bottom of relation list
 *
 * @return string
 */
function archibald_ajax_content_form_relations_add_empty_relation($form, $form_state) {
  return $form['relations']['relation'];
}

/**
 * Submit handler for the "Add more" button.
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_form_add_more_relations_relation_submit($form, &$form_state) {
  $form_state['num_relations']++;
  $form_state['rebuild'] = TRUE;
}

/**
 * theme a archibald form element relation
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_relations_relation($form) {
  $form = reset($form);
  $output = '<label for="edit-relations-title">' . archibald_get_field_label('relation') .
    archibald_get_require_type_value('relation.relation');
  $output .= '</label>';

  $output .= '<div id="archibald_relations_relation">';

  foreach ($form as $k => $element) {
    if (!preg_match('/^relation_/', $k)) {
      continue;
    }
    unset($element['ressource']['#title']);

    // at some where here the MetaTMeta Date Object Key
    // to load it in modal frame
    $output .= '<table class="archibald_relations_relation_item">' . '<tr><td align="left">';
    $output .= drupal_render($element['name']);
    $output .= drupal_render($element['ressource']);
    $output .= '</td></tr>';
    $output .= '</table>';
  }


  $output .= '</div>';
  $output .= drupal_render($form['add_more_relation']);

  return $output;
}

/**
 * menu callback with handle the relation selection
 *
 * @param string $op
 * @param string $js  nojs or ajax
 * @param string $param
 * @param string $paramb
 *
 * @return string
 */
function archibald_choose_relation($op = '', $js = 'nojs', $param = '', $paramb = '') {

  // Include the CTools tools that we need.
  ctools_include('ajax');
  ctools_include('modal');

  // Add CTools' javascript to the page.
  ctools_modal_add_js();

  switch ($op) {
    case 'list':
    default:
      if ($js == 'page') {
        print archibald_choose_relation_search_page($param, $paramb);
        exit;
      }
      $output = archibald_choose_relation_search($param);

      if ($js == 'ajax') {
        $commands = array(
          ctools_modal_command_display(t('Choose relation'), $output),
        );
        print ajax_render($commands);
        exit;
      }
      break;
  }

  return $output;
}

/**
 * perform a relation search
 *
 * @param string $query
 * @param $page
 *   default=1
 * @param $per_page
 *   default=25
 * @param $pages default=1
 *   called by refecrence
 *   number of pages
 *
 * @return array result set
 */
function archibald_relation_search($query, $page = 1, $per_page = 25, &$pages = 1) {

  // to be equal as ArchibaldLomSQLSearch
  $sql = ' FROM archibald_lom l ' .
    'INNER JOIN archibald_general dg ON l.general_id = dg.general_id  ' .
    'INNER JOIN archibald_meta_metadata dm ON dm.meta_metadata_id=l.meta_metadata_id ' .
    'INNER JOIN archibald_langstring_terms dlt ON dlt.langstring_terms_id = dg.title  ' .
    'LEFT JOIN archibald_relations dr ON l.lom_id=dr.lom_id AND l.version=dr.lom_version ' .
    'WHERE l.save_time = (' .
      'SELECT la.save_time AS save_time ' .
      'FROM archibald_lom la ' .
      'WHERE (l.lom_id = la.lom_id) ' .
      'ORDER BY save_time DESC ' .
      'LIMIT 1 OFFSET 0) ' .
    'AND l.deleted IS NULL ' .
    'AND dr.relation_id IS NULL';

  $params = array();
  if (!empty($query)) {
    $i = 0;
    $words = preg_split('/[\s\.\?\!\-\_]+/', $query);
    foreach ($words as $word) {
      if (!empty($word)) {
        $sql .= " AND dlt.term LIKE :term$i";
        $params[':term' . $i] = '%' . $word . '%';
      }
    }
    $i++;
  }

  // dont show your self in list
  if (!empty($_POST['self_lom_id'])) {
    $sql .= " AND l.lom_id!=:self_lom_id";
    $params[':self_lom_id'] = filter_xss($_POST['self_lom_id']);
  }

  $sql .= ' GROUP BY l.lom_id';

  if (db_driver() == 'pgsql') {
    $sh = 'SELECT l.lom_id, ' . 'FIRST(dlt.term) AS term, ' . 'FIRST(dlt.langstring_terms_id) AS langstring_terms_id ' . $sql;
  }
  else {
    $sh = 'SELECT l.lom_id, dlt.term, dlt.langstring_terms_id ' . $sql;
  }

  $shc = 'SELECT COUNT(dlt.langstring_terms_id) AS count ' . $sql;

  $pages = ceil(db_query($shc, $params)->fetchColumn(1) / $per_page);
  if ($pages < 1) {
    $pages = 1;
  }

  if ($page > $pages) {
    $page = $pages;
  }

  if ($page < 1) {
    $page = 1;
  }

  $sh .= ' LIMIT ' . $per_page . ' OFFSET ' . (($page - 1) * $per_page);

  $result = array();
  $db_result = db_query($sh, $params);
  while ($row = $db_result->fetchAssoc()) {
    $result[] = $row;
  }

  return $result;
}

/**
 * search for relations
 *
 * @param string $relation_id to select
 *
 * @return string html
 */
function archibald_choose_relation_search($lom_id = '') {
  $search_result = archibald_relation_search(
    filter_xss($_POST['query']), 1, ARCHIBALD_RELATIONS_PER_PAGE, $pages
  );
  $return = theme(
    'archibald_choose_relation_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'query' => filter_xss($_POST['query']),
      'lom_id' => $lom_id,
      'relation_id' => filter_xss($_POST['ressource']),
    )
  );
  return $return;
}

/**
 * search for relation ajax request
 *
 * @param array $form
 * @param array $form_state
 *
 * @return string html
 */
function archibald_choose_relation_search_ajax($form, $form_state) {
  $search_result = archibald_relation_search(
    $form_state['values']['query'], 1, ARCHIBALD_RELATIONS_PER_PAGE, $pages
  );

  return theme(
    'archibald_choose_relation_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'query' => $form_state['values']['query'],
      'ajax' => TRUE,
      'relation_id' => $form_state['values']['ressource'],
    )
  );
}

/**
 * search for relation ajax request
 *
 * @param string $query
 * @param integer $page
 *
 * @return string html
 */
function archibald_choose_relation_search_page($query = '', $page = 1) {
  $search_result = archibald_relation_search(
    $query, $page, ARCHIBALD_RELATIONS_PER_PAGE, $pages
  );
  return theme(
    'archibald_choose_relation_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'page' => $page,
      'query' => $query,
      'ajax' => TRUE,
      'relation_id' => filter_xss($_GET['ressource']),
    )
  );
}

/**
 * theme archibald relation search
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_choose_relation_search($variables) {
  if (!isset($variables['lom_id'])) {
    $variables['lom_id'] = '';
  }
  if (!isset($variables['page'])) {
    $variables['page'] = 1;
  }

  $html = '';

  if (!empty($variables['relation_id'])) {
    $html .= '<div style="float:left; width:200px">' . '<input type="button" class="archibald_relation_reset form-submit" value="' . t('Reset') . '">' . '</div>';;
  }

  if (empty($variables['ajax'])) {
    $html .= drupal_render(
      drupal_get_form(
        'archibald_choose_relation_search_form', $variables['query']
      )
    );
  }

  $table = array(
    'header' => array(
      t('Description'),
      t('Options')
    ),
    'rows' => array(),
  );

  // generate table from result set
  foreach ($variables['search_result'] as $res_row) {
    $row = array(0 => '', 1 => array());
    $row[0] = array(
      'data' => trim($res_row['term']),
      'class' => 'archibald_relation_list_ressource',
    );

    $attributes = array(
      'class' => array(
        'archibald_relation_choose',
      ),
      'lom_id' => $res_row['lom_id'],
    );

    if ($variables['lom_id'] == $res_row['lom_id']) {
      $attributes['class'][] = ' archibald_relation_choose_this';
    }
    $row[1][] = l(
      t('Choose'),
      'admin/archibald/relation/nojs/' . $res_row['lom_id'],
      array(
        'attributes' => $attributes,
      )
    );

    $row[1] = implode(' | ', $row[1]);

    $table['rows'][] = $row;
  }

  $content = theme('table', $table);

  $items = array();
  for ($p = 1; $p <= $variables['pages']; $p++) {
    if ($p == $variables['page']) {
      $items[] = array(
        'class' => array('pager-current'),
        'data' => $p,
      );
    }
    else {
      $items[] = array(
        'class' => array('pager-item'),
        'data' => l(
          $p,
          'admin/archibald/relation/list/page/' .
          urlencode($variables['query']) . '/' . $p .
          '?ressource=' . @$_REQUEST['ressource'] .
          '&self_lom_id=' . @$_REQUEST['self_lom_id'],
          array(
            'attributes' => array(
              'class' => 'archibald_relation_page',
            ),
          )
        ),
      );
    }
  }
  $content .= '<h2 class="element-invisible">' . t('Pages') . '</h2>' . theme('item_list', array(
      'items' => $items,
      'attributes' => array(
        'class' => array('pager'),
      ),
    ));

  $html .= '<div id="archibald_choose_relation_search_result">' . $content . '</div>';

  return $html;
}

/**
 * search formular for archibald relation search
 *
 * @param array $form
 * @param array $form_state
 * @param $query query string
 *
 * @return array
 */
function archibald_choose_relation_search_form($form, &$form_state, $query) {
  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Term'),
    '#default_value' => $query,
  );

  $ressource = '';
  if (!empty($_POST['ressource'])) {
    $ressource = filter_xss($_POST['ressource']);
  }

  $self_lom_id = '';
  if (!empty($_POST['self_lom_id'])) {
    $self_lom_id = filter_xss($_POST['self_lom_id']);
  }

  $form['ressource'] = array(
    '#type' => 'hidden',
    '#default_value' => $ressource,
  );

  $form['self_lom_id'] = array(
    '#type' => 'hidden',
    '#default_value' => $self_lom_id,
  );

  $form['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#weight' => 9999,
    '#ajax' => array(
      'callback' => 'archibald_choose_relation_search_ajax',
      'wrapper' => 'archibald_choose_relation_search_result',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  $form['#theme'] = 'archibald_choose_relation_search_form';

  return $form;
}

/**
 * theme the search form
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_choose_relation_search_form($form) {
  $form = reset($form);
  #unset($form['query']['#title']);

  $html = '';
  $html .= '<div style="text-align:right;">';
  $html .= t('Term') . ': ' . theme('textfield', $form['query']) . drupal_render($form['search']);

  $form['query']['#printed'] = TRUE;
  $html .= drupal_render($form['ressource']);
  $html .= drupal_render($form['self_lom_id']);
  $html .= '</div>';
  $html .= drupal_render_children($form);

  return $html;
}

/**
 * save handler for lom relation form part
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 */
function archibald_content_form_relation_submit($form, &$form_state, &$lom) {
  if (empty($form_state['values']['relations']['relation'])) {
    return FALSE;
  }

  $relations = array();
  foreach ($form_state['values']['relations']['relation'] as $i => $relation) {
    if (empty($relation) || !is_array($relation) || empty($relation['ressource'])) {
      continue;
    }

    $new_resource = new ArchibaldLomDataResource();
    $new_resource->addIdentifier(new ArchibaldLomDataIdentifier('educa', $relation['ressource']));

    $new_relation = new ArchibaldLomDataRelation();
    $new_relation->setKind(new ArchibaldLomDataVocabulary(ArchibaldLomDataRelation::KIND_HAS_PART));

    $new_relation->setResource($new_resource);

    $relations[] = $new_relation;
  }

  $lom->setRelation($relations);
}

