<?php

/**
 * @file
 * lom resource add/edit form part: for rights tab
 */
function archibald_content_form_rights(&$form, &$form_state, &$lom, $language) {

  $lom_rights = $lom->getRights();
  if (!($lom_rights instanceof ArchibaldLomDataRights)) {
    $lom_rights = new ArchibaldLomDataRights();
  }

  $form['rights'] = array(
    '#type' => 'fieldset',
    '#title' => archibald_get_field_label('right'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'vt_main',
    '#tree' => TRUE,
    '#theme' => 'archibald_rights',
  );

  $form['rights']['cost'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('right.cost') .
      archibald_get_require_type_value('right.cost'),
    '#attributes' => array('archibald_help_ident' => 'rights.cost'),
    '#default_value' => archibald_lom_vocab_default_val($lom_rights->getCost()),
    '#options' => array(
      'LOMv1.0|yes' => t('Yes'),
      'LOMv1.0|no' => t('No'),
    ),
  );

  $form['rights']['copyrightAndOtherRestrictions'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('right.copyright') .
      archibald_get_require_type_value('right.copyright'),
    '#attributes' => array(
      'archibald_help_ident' => 'rights.copy_right_and_other_restrictions',
    ),
    '#default_value' => archibald_lom_vocab_default_val($lom_rights->getCopyrightAndOtherRestrictions()),
    '#options' => array(
      'LOMv1.0|yes' => t('Yes'),
      'LOMv1.0|no' => t('No'),
    ),
  );


  $default_value = '';
  $lang_string = $lom_rights->getDescription();
  if ($lang_string instanceof ArchibaldLomDataLangString) {
    $default_value = $lang_string->getString('en');

    if (empty($default_value)) {
      $default_value = reset($lang_string->getStrings());
    }
  }
  $form['rights']['description'] = array(
    '#type' => 'select',
    '#title' => archibald_get_field_label('right.description') .
      archibald_get_require_type_value('right.description'),
    '#attributes' => array('archibald_help_ident' => 'rights.license'),
    '#options' => archibald_get_licences_options($default_value),
    '#default_value' => $default_value,
  );
}

function archibald_get_licences_options(&$default_value) {
  $options = archibald_get_taxonomy_options('rights_licenses');
  archibald_select_options_default_value_filter($options, $default_value);

  if (!isset($options[''])) {
    $options[''] = t('Unknown');
  }

  ksort($options);

  return $options;
}

/**
 * save handler for lom education form part
 *
 * @param array $form
 * @param array $form_state
 * @param ArchibaldLom $lom
 */
function archibald_content_form_rights_submit($form, &$form_state, &$lom) {
  $lom_rights = $lom->getRights();
  if (!($lom_rights instanceof ArchibaldLomDataRights)) {
    $lom_rights = new ArchibaldLomDataRights();
  }

  ##### handle cost
  if (archibald_check_field_permission('rights cost')) {
    if (preg_match('/(.+)\|(.+)/', $form_state['values']['rights']['cost'], $parts)) {
      $vocabulary = new ArchibaldLomDataVocabulary();
      $vocabulary->setSource($parts[1]);
      $vocabulary->setValue($parts[2]);
      $lom_rights->setCost($vocabulary);
    }
    else {
      $lom_rights->setCost(NULL);
    }
  }

  ##### handle copyrightAndOtherRestrictions
  if (archibald_check_field_permission('rights copyrightAndOtherRestrictions')) {
    if (preg_match('/(.+)\|(.+)/', $form_state['values']['rights']['copyrightAndOtherRestrictions'], $parts)) {
      $vocabulary = new ArchibaldLomDataVocabulary();
      $vocabulary->setSource($parts[1]);
      $vocabulary->setValue($parts[2]);
      $lom_rights->setCopyrightAndOtherRestrictions($vocabulary);
    }
    else {
      $lom_rights->setCopyrightAndOtherRestrictions(NULL);
    }
  }

  ##### handle description / Licence
  if (archibald_check_field_permission('rights description')) {
    if (!empty($form_state['values']['rights']['description'])) {
      // if key was not able to pass through a form, get it from session store
      if (!empty($_SESSION['archibald_default_value_keys'][$form_state['values']['rights']['description']])) {
        $form_state['values']['rights']['description'] = $_SESSION['archibald_default_value_keys'][$form_state['values']['rights']['description']];
      }

      $lang_string = new ArchibaldLomDataLangString();
      $lang_string->setString($form_state['values']['rights']['description'], 'en');

      $lom_rights->setDescription($lang_string);
    }
    else {
      $lom_rights->setDescription(NULL);
    }
  }


  $lom->setRights($lom_rights);
}

/**
 * theme a archibald form block rights
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_rights($form) {
  $form = reset($form);
  $output = '<label for="edit-relations-title" style="display: inline;">' . $form['#title'] . '</label>';
  $output .= '<table id="archibald_rights" border="0">';

  $html = array(
    'top' => array(),
    'middle' => array(),
    'bottom' => array(),
  );
  foreach (element_children($form, TRUE) as $key) {
    switch ($key) {
      case 'cost':
        $html['top'][] = drupal_render($form[$key]);
        break;

      case 'copyrightAndOtherRestrictions':
        $html['middle'][] = drupal_render($form[$key]);
        break;

      case 'description':
        $html['bottom'][] = drupal_render($form[$key]);
        break;

      default:
        $html['top'][] = drupal_render($form[$key]);
        break;
    }
  }

  $top = implode("\n", $html['top']);
  if (!empty($top)) {
    $output .= '<tr>';
    $output .= '<td class="archibald_rights_td archibald_rights_td_top">';
    $output .= $top;
    $output .= '</td>';
    $output .= '</tr>';
  }
  $middle = implode("\n", $html['middle']);
  if (!empty($middle)) {
    $output .= '<tr>';
    $output .= '<td class="archibald_rights_td archibald_rights_td_middle">';
    $output .= $middle;
    $output .= '</td>';
    $output .= '</tr>';
  }

  $bottom = implode("\n", $html['bottom']);
  if (!empty($bottom)) {
    $output .= '<tr>';
    $output .= '<td class="archibald_rights_td archibald_rights_td_bottom">';
    $output .= $bottom;
    $output .= '</td>';
    $output .= '</tr>';
  }
  $output .= '</table>';

  return $output;
}

