<?php

/**
 * @file
 * Handles the saving for a lom object
 */
include_once dirname(__FILE__) . '/LomContributer.class.php';

/**
 * Description of ArchibaldLomSaveHandler
 *
 * @author prdatur
 */
class ArchibaldLomSaveHandler {

  /**
   * Holds the last created lom version.
   *
   * @var string
   */
  public static $lastLomVersion = "";

  /**
   *
   * @param ArchibaldLom $lom
   *
   * @return string
   *   json formated lom object
   */
  public static function jsonExport(ArchibaldLom &$lom) {
    return json_encode($lom);
  }

  /**
   *
   * @param ArchibaldLom $obj
   *
   * @return array
   *   the object as an array
   */
  private static function obj2arr($obj) {
    $arr = array();
    $_arr = is_object($obj) ? get_object_vars($obj) : $obj;
    foreach ($_arr as $key => $val) {
      if ((is_array($val) || is_object($val))) {
        $val = ArchibaldLomSaveHandler::obj2arr($val);
      }

      $arr[$key] = $val;
    }
    return $arr;
  }

  /**
   * converts json formated string into lom class object
   *
   * @param string $json_lom
   *   json formated string
   *
   * @return ArchibaldLom
   *   Object
   */
  public static function jsonImport($json_lom) {
    if (!is_object($json_lom) && !is_array($json_lom)) {
      $json_lom = json_decode($json_lom);
    }

    $json_lom = ArchibaldLomSaveHandler::obj2arr($json_lom);

    $tmp_lom = new ArchibaldLom();
    $tmp_lom->setFields($json_lom);
    $tmp_lom->relation = array();

    $rights = new ArchibaldLomDataRights();
    $rights->setCost(ArchibaldLomSaveHandler::parseVocabular($json_lom['rights']['cost']));
    $rights->setCopyrightAndOtherRestrictions(
      ArchibaldLomSaveHandler::parseVocabular($json_lom['rights']['copyrightAndOtherRestrictions'])
    );
    $rights->setDescription(ArchibaldLomSaveHandler::parseLangString($json_lom['rights']['description']));
    $tmp_lom->rights = $rights;

    $general = new ArchibaldLomDataGeneral();
    $general->setFields($json_lom['general']);
    $general->structure = ArchibaldLomSaveHandler::parseVocabular($json_lom['general']['structure']);
    $general->aggregationLevel = ArchibaldLomSaveHandler::parseVocabular($json_lom['general']['aggregationLevel']);
    $general->title = ArchibaldLomSaveHandler::parseLangString($json_lom['general']['title']);
    $general->description = ArchibaldLomSaveHandler::parseLangString($json_lom['general']['description']);
    $general->identifier = array();
    $general->keyword    = array();
    $general->coverage   = array();
    foreach ($json_lom['general']['identifier'] as $identifier) {
      if (!empty($identifier['title'])) {
        $identifier['title'] = ArchibaldLomSaveHandler::parseLangString($identifier['title']);
      }

      $identi = new ArchibaldLomDataIdentifier();
      $identi->setFields($identifier);
      $general->addIdentifier($identi);
    }


    foreach ($json_lom['general']['keyword'] as $keyword) {
      $general->addKeyword(ArchibaldLomSaveHandler::parseLangString($keyword));
    }
    foreach ($json_lom['general']['coverage'] as $coverage) {
      $general->addCoverage(ArchibaldLomSaveHandler::parseLangString($coverage));
    }

    $tmp_lom->general = $general;

    $lifecycle = new ArchibaldLomDataLifeCycle();
    $lifecycle->setFields($json_lom['lifeCycle']);
    $lifecycle->status = ArchibaldLomSaveHandler::parseVocabular($json_lom['lifeCycle']['status']);
    $lifecycle->version = ArchibaldLomSaveHandler::parseLangString($json_lom['lifeCycle']['version']);
    $lifecycle->contribute = array();

    foreach ($json_lom['lifeCycle']['contribute'] as $val) {
      $lifecycle->addContribute(ArchibaldLomSaveHandler::parseContribute($val));
    }
    $tmp_lom->lifeCycle = $lifecycle;

    $meta_metadata = new ArchibaldLomDataMetaMetadata();
    $meta_metadata->setFields($json_lom['metaMetadata']);
    $meta_metadata->identifier = array();
    $meta_metadata->contribute = array();
    foreach ($json_lom['metaMetadata']['identifier'] as $identifier) {
      $identi = new ArchibaldLomDataIdentifier();
      $identi->setFields($identifier);
      $meta_metadata->addIdentifier($identi);
    }

    foreach ($json_lom['metaMetadata']['contribute'] as $val) {
      $meta_metadata->addContribute(ArchibaldLomSaveHandler::parseContribute($val));
    }
    $tmp_lom->metaMetadata = $meta_metadata;

    $technical = new ArchibaldLomDataTechnical();
    $technical->setFields($json_lom['technical']);
    $technical->requirement = array();
    $technical->installationRemarks = new ArchibaldLomDataLangString();
    $technical->otherPlattformRequirements = ArchibaldLomSaveHandler::parseLangString(
      $json_lom['technical']['otherPlattformRequirements']
    );

    $duration = new ArchibaldLomDataDuration();
    $duration->duration = $json_lom['technical']['duration']['duration'];
    $duration->description = ArchibaldLomSaveHandler::parseLangString($json_lom['technical']['duration']['description']);
    $technical->duration = $duration;

    $tmp_lom->technical = $technical;

    $education = new ArchibaldLomDataEducation();
    $education->setFields($json_lom['education']);

    $education->interactiveType = ArchibaldLomSaveHandler::parseVocabular($json_lom['education']['interactiveType']);
    $education->interactivityLevel = ArchibaldLomSaveHandler::parseVocabular($json_lom['education']['interactivityLevel']);
    $education->semanticDensity = ArchibaldLomSaveHandler::parseVocabular($json_lom['education']['semanticDensity']);
    $education->difficult = ArchibaldLomSaveHandler::parseVocabular($json_lom['education']['difficult']);

    $duration = new ArchibaldLomDataDuration();
    $duration->duration = $json_lom['education']['typicalLearningTime']['duration'];
    $duration->description = ArchibaldLomSaveHandler::parseLangString($json_lom['education']['typicalLearningTime']['description']);
    $education->typicalLearningTime = $duration;

    $education->description = ArchibaldLomSaveHandler::parseLangString($json_lom['education']['description']);
    $education->typicalAgeRange = array();
    foreach ($json_lom['education']['typicalAgeRange'] as $typical_age_range) {
      $education->addTypicalAgeRange(ArchibaldLomSaveHandler::parseLangString($typical_age_range));
    }

    $education->language = $json_lom['education']['language'];
    $education->learningResourceType = array();
    $education->intendedEndUserRole = array();
    $education->context = array();
    foreach ($json_lom['education']['learningResourceType'] as $val) {
      $education->addLearningResourceType(ArchibaldLomSaveHandler::parseVocabular($val));
    }
    foreach ($json_lom['education']['intendedEndUserRole'] as $val) {
      $education->addIntendedEndUserRole(ArchibaldLomSaveHandler::parseVocabular($val));
    }
    foreach ($json_lom['education']['context'] as $val) {
      $education->addContext(ArchibaldLomSaveHandler::parseVocabular($val));
    }

    $tmp_lom->education = $education;
    #print_r($tmp_lom->education); die();

    $tmp_lom->relation = array();
    foreach ($json_lom['relation'] as $relation) {

      $rel = new ArchibaldLomDataRelation();
      $rel->setFields($relation);
      $rel->kind = ArchibaldLomSaveHandler::parseVocabular($relation['kind']);

      $res = new ArchibaldLomDataResource();
      $res->setFields($relation['resource']);
      $res->description = ArchibaldLomSaveHandler::parseLangString($relation['resource']['description']);
      $identifers = array();
      foreach ($relation['resource']['identifier'] as $identifier) {
        $ident = new ArchibaldLomDataIdentifier();
        $ident->setFields($identifier);
        $identifers[] = $ident;
      }
      $res->identifier    = $identifers;
      $rel->resource      = $res;
      $tmp_lom->relation[] = $rel;
    }


    $tmp_lom->annotation = array();
    foreach ($json_lom['annotation'] as $annotation) {
      $anno = new ArchibaldLomDataAnnotation();
      $anno->setFields($annotation);

      $date = new ArchibaldLomDataDateTime();
      $date->setFields($annotation['date']);
      $date->description = ArchibaldLomSaveHandler::parseLangString($annotation['date']['description']);
      $anno->date = $date;

      $anno->description = ArchibaldLomSaveHandler::parseLangString($annotation['description']);

      $tmp_lom->addAnnotation($anno);
    }
    $tmp_lom->classification = array();
    foreach ($json_lom['classification'] as $classification) {

      $classi = new ArchibaldLomDataClassification();
      $classi->setFields($classification);
      $classi->description = ArchibaldLomSaveHandler::parseLangString($classification['description']);
      $classi->purpose = ArchibaldLomSaveHandler::parseVocabular($classification['purpose']);
      $classi->taxonPath = array();
      $classi->keyword = array();
      foreach ($classification['taxonPath'] as $taxon_path) {

        $taxonp = new ArchibaldLomDataTaxonPath();
        $taxonp->setFields($taxon_path);
        $taxonp->source = ArchibaldLomSaveHandler::parseLangString($taxon_path['source']);
        $taxonp->taxon = array();
        foreach ($taxon_path['taxon'] as $taxon) {
          $tax = new ArchibaldLomDataTaxon();
          $tax->setFields($taxon);
          $tax->entry = ArchibaldLomSaveHandler::parseLangString($taxon['entry']);
          $taxonp->addTaxon($tax);
        }

        $classi->addTaxonPath($taxonp);
      }

      foreach ($classification['keyword'] as $keyword) {
        $classi->addKeyword(ArchibaldLomSaveHandler::parseLangString($keyword));
      }
      $tmp_lom->addClassification($classi);
    }
    $tmp_lom->curriculum = array();
    foreach ($json_lom['curriculum'] as $curriculum) {
      $curri = new ArchibaldLomDataCurriculum();
      $curri->setFields($curriculum);
      $tmp_lom->addCurriculum($curri);
    }
    return $tmp_lom;
  }

  /**
   * parse json array element to ArchibaldLomDataContribute object
   *
   * @param array $array
   *
   * @return ArchibaldLomDataContribute
   */
  private static function parseContribute($array) {

    $con = new ArchibaldLomDataContribute();
    if (is_null($array)) {
      return $con;
    }
    $con->setFields($array);

    $date = new ArchibaldLomDataDateTime();
    $date->setFields($array['date']);
    $date->description = ArchibaldLomSaveHandler::parseLangString($array['date']['description']);
    $con->date = $date;
    $con->role = ArchibaldLomSaveHandler::parseVocabular($array['role']);

    return $con;
  }

  /**
   * parse json array element to ArchibaldLomDataVocabulary object
   *
   * @param array $array
   *
   * @return ArchibaldLomDataVocabulary
   */
  private static function parseVocabular($array) {
    $voc = new ArchibaldLomDataVocabulary();
    if (is_null($array)) {
      return $voc;
    }
    $voc->setFields($array);
    return $voc;
  }

  /**
   * parse json array element to ArchibaldLomDataLangString object
   *
   * @param array $array
   *
   * @return ArchibaldLomDataLangString
   */
  private static function parseLangString($array) {
    $l = new ArchibaldLomDataLangString();
    if (is_null($array)) {
      return $l;
    }
    if (empty($array['strings'])) {
      $array['strings'] = array();
    }
    $l->setFields($array);
    return $l;
  }

  /**
   * delete the given lom from DB
   *
   * @param mixed $lom lomid
   *   as a string or
   *   the complete lom class object
   * @param string $version
   */
  public static function purge($lom, $version = 0) {
    if (($lom instanceof ArchibaldLom) == FALSE) {
      $lom = ArchibaldLomSaveHandler::load($lom, $version);
    }
    foreach ($lom->getClassification() as $classifications) {
      ArchibaldLomSaveHandler::deleteVocabular('lom:classification', $classifications->classification_id);

      foreach ($classifications->getTaxonPath() as $taxonpaths) {

        $db = db_delete("archibald_taxon")
          ->condition('taxon_path_id', $taxonpaths->taxon_path_id)
          ->execute();

        $db = db_delete("archibald_taxon_path")
          ->condition('taxon_path_id', $taxonpaths->taxon_path_id)
          ->execute();
      }

      $db = db_delete("archibald_classification_taxon")
        ->condition('classification_id', $classifications->classification_id)
        ->execute();
      $db = db_delete("archibald_classification_keywords")
        ->condition('classification_id', $classifications->classification_id)
        ->execute();
      $db = db_delete("archibald_classification")
        ->condition('classification_id', $classifications->classification_id)
        ->execute();
    }

    $db = db_delete("archibald_annotations")
      ->condition('lom_id', $lom->getLomId())
      ->execute();


    $mmid = $lom->getMetaMetadata()->meta_metadata_id;
    $db = db_delete("archibald_meta_metadata_identifier")
      ->condition('meta_metadata_id', $mmid)
      ->execute();

    foreach ($lom->getMetaMetadata()->getContribute() as $contribute) {
      ArchibaldLomSaveHandler::deleteContribute($contribute, 'meta_metadata');
    }

    $db = db_delete("archibald_meta_metadata_contributes")
      ->condition('meta_metadata_id', $mmid)
      ->execute();
    $db = db_delete("archibald_meta_metadata")
      ->condition('meta_metadata_id', $mmid)
      ->execute();

    ArchibaldLomSaveHandler::deleteVocabular('lom:rights_cost', $lom_id . '_' . $lom->getVersion());
    ArchibaldLomSaveHandler::deleteVocabular('lom:rights_copyrightAndOtherRestrictions', $lom_id . '_' . $lom->getVersion());


    $eduid = $lom->getEducation()->education_id;
    $db = db_delete('archibald_education_language')
      ->condition('education_id', $eduid)
      ->execute();
    $db = db_delete('archibald_education_typicalagerange')
      ->condition('education_id', $eduid)
      ->execute();

    $db = db_delete('archibald_education')
      ->condition('education_id', $eduid)
      ->execute();
    $db = db_delete('archibald_education_context')
      ->condition('education_id', $eduid)
      ->execute();
    $db = db_delete('archibald_education_intendedenduserrole')
      ->condition('education_id', $eduid)
      ->execute();
    $db = db_delete('archibald_education_learningresourcetype')
      ->condition('education_id', $eduid)
      ->execute();

    ArchibaldLomSaveHandler::deleteVocabular('education:interactiveType', $eduid);
    ArchibaldLomSaveHandler::deleteVocabular('education:interactivityLevel', $eduid);
    ArchibaldLomSaveHandler::deleteVocabular('education:semanticDensity', $eduid);
    ArchibaldLomSaveHandler::deleteVocabular('education:difficult', $eduid);

    ArchibaldLomSaveHandler::deleteVocabular('education:learningResourceType_' . $eduid);
    ArchibaldLomSaveHandler::deleteVocabular('education:intendedEndUserRole_' . $eduid);
    ArchibaldLomSaveHandler::deleteVocabular('education:context_' . $eduid);

    $techid = $lom->getTechnical()->technical_id;
    $db = db_delete("archibald_technical")
      ->condition('technical_id', $techid)
      ->execute();
    $db = db_delete("archibald_technical_formats")
      ->condition('technical_id', $techid)
      ->execute();
    $db = db_delete("archibald_technical_locations")
      ->condition('technical_id', $techid)
      ->execute();

    $lifeid = $lom->getLifeCycle()->lifecycle_id;

    $db = db_delete("archibald_lifecycle")
      ->condition('lifecycle_id', $lifeid)
      ->execute();

    $db = db_delete("archibald_lifecycle_contributes")
      ->condition('lifecycle_id', $lifeid)
      ->execute();

    ArchibaldLomSaveHandler::deleteVocabular("lifeCycle:status", $lifeid);
    foreach ($lom->getLifeCycle()->getContribute() as $contribute) {
      ArchibaldLomSaveHandler::deleteContribute($contribute, 'lifecycle');
    }

    $genid = $lom->getGeneral()->general_id;


    $db = db_delete("archibald_general")
      ->condition('general_id', $genid)
      ->execute();
    $db = db_delete("archibald_general_identifier")
      ->condition('general_id', $genid)
      ->execute();
    $db = db_delete("archibald_general_languages")
      ->condition('general_id', $genid)
      ->execute();
    $db = db_delete("archibald_general_keywords")
      ->condition('general_id', $genid)
      ->execute();
    $db = db_delete("archibald_general_coverage")
      ->condition('general_id', $genid)
      ->execute();
    $db = db_delete("archibald_relations")->condition('lom_id', $lom->getLomId())
      ->condition('lom_version', $lom->getVersion())
      ->execute();

    ArchibaldLomSaveHandler::deleteVocabular('general:structure', $genid);
    ArchibaldLomSaveHandler::deleteVocabular('general:aggregationLevel', $genid);

    $db = db_delete("archibald_lom")->condition('lom_id', $lom->getLomId())
      ->condition('version', $lom->getVersion())
      ->execute();
  }

  /**
   * insert ArchibaldLom object into database
   *
   * @param ArchibaldLom $lom
   *   reference
   * @param string $version
   *   called by reference
   *   the new version id after saving
   *
   * @param boolean $reset_translation
   *   Wether we want to reset the translation bit or not (optional, default = true)
   * @return string
   *   Returns the inserted lomid as a string
   */
  public static function insert(ArchibaldLom &$lom, &$version = '', $reset_translations = TRUE) {
    global $user;
    $userid = $user->uid;
    if (empty($userid)) {
      $userid = 1;
    }

    $lom->version = ArchibaldLomSaveHandler::generateUuid();
    $version = $lom->version;
    ArchibaldLomSaveHandler::$lastLomVersion = $version;
    if ($reset_translations == TRUE && $lom->getLomId()!= NULL) {
      //disable reset of translation which generates a new revision if translation is not in progress
      $translation_in_progrss = db_select('archibald_lom_stats', 'ls')
        ->fields('ls')
        ->condition('lom_id', $lom->getLomId())
        ->execute()
        ->fetchAssoc();

      if ($translation_in_progrss['new_revision_required'] == 0) {
        $reset_translations = FALSE;
      }
    }
    elseif ($lom->getLomId()!=NULL) {
      $reset_translations = FALSE;
    }

    /**
     * @var ArchibaldLomDataGeneral
     */
    $general = $lom->getGeneral();
    $fields = array(
      'title' => new ArchibaldLomDataLangString(),
      'description' => "",
      'structure' => NULL,
      'aggregationLevel' => NULL,
    );

    if ($general->getTitle() instanceof ArchibaldLomDataLangString) {
      $fields['title'] = ArchibaldLomSaveHandler::saveOrInsertLangString($general->getTitle());
    }

    if ($general->getDescription() instanceof ArchibaldLomDataLangString) {
      $fields['description'] = ArchibaldLomSaveHandler::saveOrInsertLangString($general->getDescription());
    }

    if ($general->getStructure() instanceof ArchibaldLomDataVocabulary) {
      $fields['structure'] = $general->getStructure()->getValue();
    }

    if ($general->getAggregationLevel() instanceof ArchibaldLomDataVocabulary) {
      $fields['aggregationLevel'] = $general->getAggregationLevel()->getValue();
    }

    $general_inserted_id = db_insert('archibald_general')
      ->fields($fields)
      ->execute();


    ArchibaldLomSaveHandler::insertVocabular($general->getStructure(), 'general:structure', $general_inserted_id);
    ArchibaldLomSaveHandler::insertVocabular($general->getAggregationLevel(), 'general:aggregationLevel', $general_inserted_id);

    foreach ($general->getIdentifier() as $value) {
      $fields = array(
        'catalog' => $value->getCatalog(),
        'entry' => $value->getEntry(),
        'general_id' => $general_inserted_id,
      );
      if ($value->getTitle() instanceof ArchibaldLomDataLangString) {
        $fields['title'] = ArchibaldLomSaveHandler::saveOrInsertLangString(
          $value->getTitle()
        );
      }

      db_insert("archibald_general_identifier")
        ->fields($fields)
        ->execute();
    }

    foreach ($general->getLanguage() as $value) {
      $fields = array(
        'language' => $value,
        'general_id' => $general_inserted_id,
      );
      db_insert("archibald_general_languages")
        ->fields($fields)
        ->execute();
    }

    foreach ($general->getKeyword() as $value) {
      $fields = array(
        'keyword' => ArchibaldLomSaveHandler::saveOrInsertLangString($value),
        'general_id' => $general_inserted_id,
      );
      db_insert("archibald_general_keywords")
        ->fields($fields)
        ->execute();
    }



    foreach ($general->getCoverage() as $value) {
      $fields = array(
        'coverage' => ArchibaldLomSaveHandler::saveOrInsertLangString($value),
        'general_id' => $general_inserted_id,
      );
      db_insert("archibald_general_coverage")
        ->fields($fields)
        ->execute();
    }
    unset($general);

    /**
     * @var ArchibaldLomDataLifeCycle
     */
    $lifecycle = $lom->getLifeCycle();
    $fields = array(
      'version' => ArchibaldLomSaveHandler::saveOrInsertLangString(
        $lifecycle->getVersion()
      ),
      'status' => $lifecycle->getStatus()->getValue(),
    );

    $lifecycle_inserted_id = db_insert('archibald_lifecycle')
      ->fields($fields)
      ->execute();

    ArchibaldLomSaveHandler::insertVocabular($lifecycle->getStatus(), 'lifeCycle:status', $lifecycle_inserted_id);

    foreach ($lifecycle->getContribute() as $k => $value) {
      $fields = array(
        'contribute_id' => ArchibaldLomSaveHandler::insertContribute(
          $value,
          'lifecycle'
        ),
        'lifecycle_id' => $lifecycle_inserted_id,
      );
      db_insert("archibald_lifecycle_contributes")
        ->fields($fields)
        ->execute();
      $lifecycle->setContribute($value, $k);
    }
    $lifecycle->lifecycleId = $lifecycle_inserted_id;
    $lom->setLifeCycle($lifecycle);
    unset($lifecycle);

    /**
     * @var ArchibaldLomDataTechnical
     */
    $technical_inserted_id = 0;
    $technical = $lom->getTechnical();

    $fields = array(
      'size' => 0,
      'other_plattform_requirements' => new ArchibaldLomDataLangString(),
      'duration_duration' => '',
      'duration_description' => new ArchibaldLomDataLangString(),
    );

    if ($technical instanceof ArchibaldLomDataTechnical) {
      $fields = array(
        'size' => $technical->getSize(),
        'other_plattform_requirements' => ArchibaldLomSaveHandler::saveOrInsertLangString(
          $technical->getOtherPlattformRequirements()
        ),
        'duration_duration' => $technical->getDuration()->getDuration(),
        'duration_description' => ArchibaldLomSaveHandler::saveOrInsertLangString(
          $technical->getDuration()->getDescription()
        ),
        'preview_image' => $technical->getPreviewImage(),
      );

      $technical_inserted_id = db_insert('archibald_technical')
        ->fields($fields)
        ->execute();

      $alreadyinserted = array();
      foreach (@$technical->getFormat() as $value) {
        $hash_key = md5($value . '|' . $technical_inserted_id);

        $fields = array(
          'format' => $value,
          'technical_id' => $technical_inserted_id,
        );

        if (isset($alreadyinserted[$hash_key])) {
          continue;
        }

        db_insert('archibald_technical_formats')
          ->fields($fields)
          ->execute();

        $alreadyinserted[$hash_key] = TRUE;
      }

      foreach (@$technical->getLocation() as $value) {
        $fields = array(
          'location' => $value,
          'technical_id' => $technical_inserted_id,
        );
        db_insert('archibald_technical_locations')
          ->fields($fields)
          ->execute();
      }
      unset($technical);
    }

    /**
     * @var $education ArchibaldLomDataEducation
     */
    $education = $lom->getEducation();
    $fields = array(
      'interactiveType' => $education->getInteractiveType()->getValue(),
      'interactivityLevel' => $education->getInteractivityLevel()->getValue(),
      'semanticDensity' => $education->getSemanticDensity()->getValue(),
      'difficult' => $education->getDifficult()->getValue(),
      'description' => ArchibaldLomSaveHandler::saveOrInsertLangString($education->getDescription()),
      'typicalLearningTime_duration' => $education->getTypicalLearningTime()->getDuration(),
      'typicalLearningTime_description' => ArchibaldLomSaveHandler::saveOrInsertLangString(
        $education->getTypicalLearningTime()->getDescription()
      ),
    );
    $education_inserted_id = db_insert('archibald_education')
      ->fields($fields)
      ->execute();

    ArchibaldLomSaveHandler::insertVocabular($education->getInteractiveType(), 'education:interactiveType', $education_inserted_id);
    ArchibaldLomSaveHandler::insertVocabular($education->getInteractivityLevel(), 'education:interactivityLevel', $education_inserted_id);
    ArchibaldLomSaveHandler::insertVocabular($education->getSemanticDensity(), 'education:semanticDensity', $education_inserted_id);

    ArchibaldLomSaveHandler::insertVocabular($education->getDifficult(), 'education:difficult', $education_inserted_id);

    foreach ($education->getLearningResourceType() as $value) {
      $fields = array(
        'learningResourceType_id' => $value->getValue(),
        'education_id' => $education_inserted_id,
      );

      db_insert('archibald_education_learningresourcetype')
        ->fields($fields)
        ->execute();

      ArchibaldLomSaveHandler::insertVocabular(
        $value, 'education:learningResourceType_' . $education_inserted_id,
        $value->getValue()
      );
    }

    foreach ($education->getIntendedEndUserRole() as $value) {
      $fields = array(
        'intendedEndUserRole_id' => $value->getValue(),
        'education_id' => $education_inserted_id,
      );
      db_insert('archibald_education_intendedenduserrole')
        ->fields($fields)
        ->execute();

      ArchibaldLomSaveHandler::insertVocabular(
        $value,
        'education:intendedEndUserRole_' . $education_inserted_id, $value->getValue()
      );
    }

    foreach ($education->getContext() as $value) {
      $fields = array(
        'context_id' => $value->getValue(),
        'education_id' => $education_inserted_id,
      );
      db_insert("archibald_education_context")
        ->fields($fields)
        ->execute();
      ArchibaldLomSaveHandler::insertVocabular($value, 'education:context_' . $education_inserted_id, $value->getValue());
    }

    foreach ($education->getTypicalAgeRange() as $value) {
      $fields = array(
        'typicalAgeRange' => ArchibaldLomSaveHandler::saveOrInsertLangString($value),
        'education_id' => $education_inserted_id,
      );
      db_insert("archibald_education_typicalagerange")
        ->fields($fields)
        ->execute();
    }

    foreach ($education->getLanguage() as $value) {
      $fields = array(
        'language' => $value,
        'education_id' => $education_inserted_id,
      );
      db_insert("archibald_education_language")
        ->fields($fields)
        ->execute();
    }
    unset($education);

    /**
     * @var ArchibaldLomDataMetaMetadata
     */
    $meta_metadata = $lom->getMetaMetadata();
    $fields = array(
      'metadataschema' => $meta_metadata->getMetadataSchema(),
      'language' => $meta_metadata->getLanguage(),
    );
    $meta_metadata_inserted_id = db_insert('archibald_meta_metadata')
      ->fields($fields)
      ->execute();

    foreach ($meta_metadata->getContribute() as $value) {
      $fields = array(
        'contribute_id' => ArchibaldLomSaveHandler::insertContribute($value, 'meta_metadata'),
        'meta_metadata_id' => $meta_metadata_inserted_id,
      );
      db_insert('archibald_meta_metadata_contributes')
        ->fields($fields)
        ->execute();
    }
    $lom_id = ArchibaldLomSaveHandler::requestLomId($lom->getLomId());

    if (empty($lom->version)) {
      $lom->version = 0;
    }

    ArchibaldLomSaveHandler::insertVocabular($lom->getRights()->getCost(), 'lom:rights_cost', $lom_id . '_' . $lom->version);
    ArchibaldLomSaveHandler::insertVocabular(
      $lom->getRights()->getCopyrightAndOtherRestrictions(),
      'lom:rights_copyrightAndOtherRestrictions', $lom_id . '_' . $lom->getVersion()
    );

    $fields = array(
      'lom_id' => $lom_id,
      'version' => $lom->getVersion(),
      'uid' => $userid,
      'general_id' => $general_inserted_id,
      'lifecycle_id' => $lifecycle_inserted_id,
      'technical_id' => $technical_inserted_id,
      'education_id' => $education_inserted_id,
      'meta_metadata_id' => $meta_metadata_inserted_id,
      'save_time' => time(),
    );

    foreach ($lom->getRelation() as $relation) {
      $kind = $relation->getKind();
      $identifiers = $relation->getResource()->getIdentifier();

      foreach ($identifiers as $identifier) {
        $relation_id = $identifier->getEntry();

        $vocublar_id = md5($lom_id . '_' . $lom->getVersion() . '_' . $relation_id);
        ArchibaldLomSaveHandler::insertVocabular($kind, 'lom:relations', $vocublar_id);

        $relation_fields = array(
          'relation_id' => $relation_id,
          'lom_id' => $lom_id,
          'lom_version' => $lom->getVersion(),
          'kind' => $vocublar_id,
        );
        $relation_id = db_insert('archibald_relations')
          ->fields($relation_fields)
          ->execute();
      }
    }

    if ($lom->getRights()->getCost() instanceof ArchibaldLomDataVocabulary) {
      $fields['rights_cost'] = $lom->getRights()->getCost()->getValue();
    }

    if ($lom->getRights()->getCopyrightAndOtherRestrictions() instanceof ArchibaldLomDataVocabulary) {
      $fields['rights_copyrightAndOtherRestrictions'] = $lom->getRights()->getCopyrightAndOtherRestrictions()->getValue();
    }

    if ($lom->getRights()->getDescription() instanceof ArchibaldLomDataLangString) {
      $fields['rights_description'] = ArchibaldLomSaveHandler::saveOrInsertLangString($lom->getRights()->getDescription());
    }

    db_insert("archibald_lom")
      ->fields($fields)
      ->execute();

    $tmp = $lom->getEducation();
    $tmp->educationId = $education_inserted_id;
    $lom->setEducation($tmp);

    $tmp = $lom->getTechnical();
    $tmp->technicalId = $technical_inserted_id;
    $lom->setTechnical($tmp);

    $tmp = $lom->getGeneral();
    $tmp->generalId = $general_inserted_id;
    $lom->setGeneral($tmp);

    $tmp = $lom->getLifeCycle();
    $tmp->lifecycleId = $lifecycle_inserted_id;
    $lom->setLifeCycle($tmp);

    $tmp = $lom->getMetaMetadata();
    $tmp->metaMetadataId = $meta_metadata_inserted_id;
    $lom->setMetaMetadata($tmp);

    $meta_metadata->addIdentifier(new ArchibaldLomDataIdentifier('educa', $lom_id));

    foreach ($meta_metadata->getIdentifier() as $value) {
      $fields = array(
        'catalog' => $value->getCatalog(),
        'entry' => $value->getEntry(),
        'meta_metadata_id' => $meta_metadata_inserted_id,
      );

      db_insert("archibald_meta_metadata_identifier")
        ->fields($fields)
        ->execute();
    }


    foreach ($lom->getAnnotation() as $k => $annotation) {

      $fields = array(
        'lom_id' => $lom_id,
        'lom_version' => $lom->getVersion(),
        #@TODO vcard parsing
        'entity' => $annotation->getEntity(),
        'date_date' => $annotation->getDate()->getDatetime(),
        'date_description' => ArchibaldLomSaveHandler::saveOrInsertLangString($annotation->getDate()->getDescription()),
        'description' => ArchibaldLomSaveHandler::saveOrInsertLangString($annotation->getDescription()),
      );
      $annotation->annotationId = db_insert('archibald_annotations')
        ->fields($fields)
        ->execute();
      $annotation->lomId = $lom_id;
      $lom->setAnnotation($annotation, $k);
    }

    foreach ($lom->getClassification() as $k => $classification) {
      $fields = array(
        'lom_id' => $lom_id,
        'lom_version' => $lom->getVersion(),
        'purpose' => $classification->getPurpose()->getValue(),
        'description' => ArchibaldLomSaveHandler::saveOrInsertLangString($classification->getDescription()),
      );

      $classification_inserted_id = db_insert('archibald_classification')
        ->fields($fields)
        ->execute();
      $classification->classificationId = $classification_inserted_id;
      $classification->lomId = $lom_id;
      ArchibaldLomSaveHandler::insertVocabular($classification->getPurpose(), 'lom:classification', $classification_inserted_id);

      foreach ($classification->getTaxonPath() as $taxon_path) {
        $fields = array(
          'source' => ArchibaldLomSaveHandler::saveOrInsertLangString($taxon_path->getSource()),
        );
        $taxon_path_inserted_id = db_insert('archibald_taxon_path')
          ->fields($fields)
          ->execute();

        $taxon_path->taxonPathId = $taxon_path_inserted_id;

        $weight = 0;
        foreach ($taxon_path->getTaxon() as $taxon) {
          $weight++;
          $fields = array(
            'taxon_path_id' => $taxon_path_inserted_id,
            'id' => $taxon->getId(),
            'entry' => ArchibaldLomSaveHandler::saveOrInsertLangString($taxon->getEntry()),
            'weight' => $weight,
          );
          db_insert('archibald_taxon')
            ->fields($fields)
            ->execute();
          $taxon->taxonPathId = $taxon_path_inserted_id;
        }

        $fields = array(
          'taxon_path_id' => $taxon_path_inserted_id,
          'classification_id' => $classification_inserted_id,
        );
        db_insert('archibald_classification_taxon')
          ->fields($fields)
          ->execute();
      }

      foreach ($classification->getKeyword() as $value) {
        $fields = array(
          'keyword' => ArchibaldLomSaveHandler::saveOrInsertLangString($value),
          'classification_id' => $classification_inserted_id,
        );
        db_insert('archibald_classification_keywords')
          ->fields($fields)
          ->execute();
      }
    }

    foreach ($lom->getCurriculum() as $k => $curriculum) {
      $fields = array(
        'lom_id' => $lom_id,
        'lom_version' => $lom->getVersion(),
        'source' => $curriculum->getSource(),
        'entity' => $curriculum->getEntity(),
      );

      $curriculum_inserted_id = db_insert('archibald_curriculum')
        ->fields($fields)
        ->execute();
      $curriculum->curriculumId = $curriculum_inserted_id;
      $curriculum->lomId = $lom_id;
    }

    // check if a entry in stats table exists and create it if nessesary
    $entry_exists = db_query(
      'SELECT COUNT(lom_id) FROM {archibald_lom_stats} WHERE lom_id=:lom_id',
      array(
        ':lom_id' => $lom_id,
      )
    )->fetchField(0);

    if ($entry_exists == 0) {

      // determine content partner
      $content_partners = archibald_load_content_partner();
      $content_partner_id = 0;

      if (count($content_partners) == 1) {
        $first_content_partner = reset($content_partners);
        if (!empty($first_content_partner['content_partner_id'])) {
          $content_partner_id = $first_content_partner['content_partner_id'];
        }
      }

      $fields = array(
        'lom_id' => $lom_id,
        'content_partner_id' => $content_partner_id,
        'new_revision_required' => 0,
      );
      db_insert('archibald_lom_stats')
        ->fields($fields)
        ->execute();

      db_delete('archibald_change_log')
        ->condition('lom_id', $lom_id)
        ->execute();
    }

    if ($reset_translations == TRUE) {
      archibald_reset_new_revision_is_required(FALSE, FALSE, $lom_id);
    }
    return $lom_id;
  }

  /**
   * updates the lom object and also increment the version id
   *
   * @param ArchibaldLom $lom reference
   * @param string $version reference
   *
   * @return String the updated lomid as a string
   */
  public static function update(ArchibaldLom $lom, &$version) {
    return ArchibaldLomSaveHandler::insert($lom, $version);
  }

  /**
   * mark lom object as deleted
   *
   * @param ArchibaldLom $lom
   */
  public static function delete(ArchibaldLom $lom) {
    // mark as deleted
    db_update('archibald_lom')
      ->fields(array('deleted' => 1))
      ->condition('lom_id', $lom->getLomId())
      ->condition('version', $lom->getVersion())
      ->execute();

    // delete all relations
    db_delete('archibald_relations')
      ->condition('lom_id', $lom->getLomId())
      ->execute();

    // reset lom_stats to let solr renindex it
    db_update('archibald_lom_stats')
      ->fields(array('indexed_version' => NULL))
      ->condition('lom_id', $lom->getLomId())
      ->execute();

    cache_set('lom:' . $lom->getLomId() . '__version:' . $lom->getVersion(), NULL, 'cache', CACHE_TEMPORARY);
  }

  /**
   * get newest version key for a lom object
   *
   * @param string $lom_id
   *
   * @return string
   */
  public static function getActualVersion($lom_id) {
    return db_query('SELECT version
      FROM {archibald_lom}
      WHERE lom_id=:lom_id
      ORDER BY save_time DESC
      LIMIT 1',
      array(
        'lom_id' => $lom_id,
      )
    )->fetchColumn(0);
  }

  /**
   * Loads a lom class object from given lomid and version,
   * if version is '' it will load the recent version
   *
   * @param string $lom_id
   * @param string $version
   * @param boolean $use_cache
   *   default TRUE
   *
   * @return ArchibaldLom
   */
  public static function load($lom_id, $version = '', $use_cache = TRUE) {

    $where = '';
    $args = array(
      ':lom_id' => $lom_id,
    );

    // get newest if no version given
    if (empty($version)) {
      $version = self::getActualVersion($lom_id);
    }

    $cid = 'lom:' . $lom_id . '__version:' . $version;
    if ($use_cache == TRUE && $cache = cache_get($cid)) {
      if (!empty($cache->data)) {
        return $cache->data;
      }
    }

    $where = 'AND version = :version ';
    $args[':version'] = $version;

    $lom = new ArchibaldLom();

    $lom_db = db_query("SELECT * FROM {archibald_lom}
      WHERE lom_id = :lom_id " . $where . "
      ORDER BY save_time DESC
      LIMIT 1",
      $args
    );
    $lom_db = $lom_db->fetch();

    if (empty($lom_db)) {
      // if lom id is invalid
      return NULL;
    }

    $lom->setFields($lom_db);
    $version      = $lom_db->version;
    $lom->version = $version;
    $rights       = new ArchibaldLomDataRights();
    $rights->setCost(ArchibaldLomSaveHandler::loadVocabular('lom:rights_cost', $lom->getLomId() . '_' . $lom->version));

    $rights->setCopyrightAndOtherRestrictions(
      ArchibaldLomSaveHandler::loadVocabular('lom:rights_copyrightAndOtherRestrictions', $lom->getLomId() . '_' . $lom->version)
    );

    $rights->setDescription(
      ArchibaldLomSaveHandler::requestLangStringObj($lom_db->rights_description)
    );

    $lom->setRights($rights);
    unset($rights);

    $general_db = db_query(
      'SELECT * FROM {archibald_general} WHERE general_id = :general_id',
      array(
        ':general_id' => $lom_db->general_id,
      )
    );

    $row = $general_db->fetchAssoc();
    $tmp_obj = new ArchibaldLomDataGeneral();
    $tmp_obj->setFields($row);

    $tmp_obj->setStructure(ArchibaldLomSaveHandler::loadVocabular('general:structure', $lom_db->general_id));
    $tmp_obj->setAggregationLevel(ArchibaldLomSaveHandler::loadVocabular('general:aggregationLevel', $lom_db->general_id));


    $tmp_obj->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($row['description']));
    #print_r(ArchibaldLomSaveHandler::requestLangStringObj($row['title']));die();
    $tmp_obj->setTitle(ArchibaldLomSaveHandler::requestLangStringObj($row['title']));

    $sub_db = db_query(
      'SELECT * FROM {archibald_general_identifier}
        WHERE general_id = :general_id',
      array(
        ':general_id' => $lom_db->general_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $title = '';
      if (!empty($row['title'])) {
        $title = ArchibaldLomSaveHandler::requestLangStringObj($row['title']);
      }
      $tmp_obj->addIdentifier(new ArchibaldLomDataIdentifier($row['catalog'], $row['entry'], $title));
    }
    unset($sub_db);

    $sub_db = db_query('SELECT * FROM {archibald_general_keywords} WHERE general_id = :general_id', array(
      ':general_id' => $lom_db->general_id,
    ));

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addKeyword(ArchibaldLomSaveHandler::requestLangStringObj($row['keyword']));
    }
    unset($sub_db);

    $sub_db = db_query(
      'SELECT *
        FROM {archibald_general_languages}
        WHERE general_id = :general_id
        ORDER BY language',
      array(
        ':general_id' => $lom_db->general_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addLanguage($row['language']);
    }
    unset($sub_db);

    $sub_db = db_query(
      'SELECT *
        FROM {archibald_general_coverage}
        WHERE general_id = :general_id
        ORDER BY coverage',
      array(
        ':general_id' => $lom_db->general_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addCoverage(ArchibaldLomSaveHandler::requestLangStringObj($row['coverage']));
    }
    unset($sub_db);

    $lom->setGeneral($tmp_obj);

    $lifecycle_db = db_query(
      'SELECT *
        FROM {archibald_lifecycle}
        WHERE lifecycle_id = :lifecycle_id',
      array(
        ':lifecycle_id' => $lom_db->lifecycle_id,
      )
    );

    $tmp_obj = new ArchibaldLomDataLifeCycle();
    $tmp_obj->setFields($lifecycle_db->fetchAssoc());
    $tmp_obj->setVersion(ArchibaldLomSaveHandler::requestLangStringObj($tmp_obj->version));

    $sub_db = db_query('
      SELECT *
        FROM {archibald_lifecycle_contributes}
        WHERE lifecycle_id = :lifecycle_id
        ORDER BY contribute_id',
      array(
        ':lifecycle_id' => $lom_db->lifecycle_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addContribute(ArchibaldLomSaveHandler::loadContribute($row['contribute_id'], 'lifecycle'));
    }
    unset($sub_db);
    $tmp_obj->setStatus(ArchibaldLomSaveHandler::loadVocabular('lifeCycle:status', $lom_db->lifecycle_id));

    $lom->setLifeCycle($tmp_obj);



    $meta_metadata_db = db_query('
      SELECT *
        FROM {archibald_meta_metadata}
        WHERE meta_metadata_id = :meta_metadata_id',
      array(
        ':meta_metadata_id' => $lom_db->meta_metadata_id,
      )
    );

    $tmp_obj = new ArchibaldLomDataMetaMetadata();
    $tmp_obj->setFields($meta_metadata_db->fetchAssoc());

    $sub_db = db_query('
      SELECT *
      FROM {archibald_meta_metadata_identifier}
      WHERE meta_metadata_id = :meta_metadata_id',
      array(
        ':meta_metadata_id' => $lom_db->meta_metadata_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addIdentifier(new ArchibaldLomDataIdentifier($row['catalog'], $row['entry']));
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_meta_metadata_contributes}
        WHERE meta_metadata_id = :meta_metadata_id',
      array(
        ':meta_metadata_id' => $lom_db->meta_metadata_id,
      )
    );
    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addContribute(ArchibaldLomSaveHandler::loadContribute($row['contribute_id'], 'meta_metadata'));
    }
    unset($sub_db);

    $lom->setMetaMetadata($tmp_obj);

    $technical_db = db_query('
      SELECT *
        FROM {archibald_technical}
        WHERE technical_id = :technical_id',
      array(
        ':technical_id' => $lom_db->technical_id,
      )
    );

    $tmp_obj = new ArchibaldLomDataTechnical();
    $technical_db = $technical_db->fetchAssoc();
    $tmp_obj->setFields($technical_db);
    $tmp_obj->setOtherPlattformRequirements(
      ArchibaldLomSaveHandler::requestLangStringObj($technical_db['other_plattform_requirements'])
    );
    $tmp_obj->setSize($tmp_obj->getSize());
    $sub_db = db_query('
      SELECT *
        FROM {archibald_technical_formats}
        WHERE technical_id = :technical_id',
      array(
        ':technical_id' => $lom_db->technical_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addFormat($row['format']);
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_technical_locations}
        WHERE technical_id = :technical_id',
      array(
        ':technical_id' => $lom_db->technical_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addLocation($row['location']);
    }
    unset($sub_db);

    $tmp_obj->setDuration(
      new ArchibaldLomDataDuration(
        $technical_db['duration_duration'],
        ArchibaldLomSaveHandler::requestLangStringObj($technical_db['duration_description']))
    );
    $lom->setTechnical($tmp_obj);



    $education_db = db_query('
      SELECT *
        FROM {archibald_education}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );

    $row = $education_db->fetchAssoc();

    $tmp_obj = new ArchibaldLomDataEducation();
    $tmp_obj->setFields($row);

    $tmp_obj->setInteractiveType(ArchibaldLomSaveHandler::loadVocabular('education:interactiveType', $lom_db->education_id));
    $tmp_obj->setInteractivityLevel(ArchibaldLomSaveHandler::loadVocabular('education:interactivityLevel', $lom_db->education_id));
    $tmp_obj->setSemanticDensity(ArchibaldLomSaveHandler::loadVocabular('education:semanticDensity', $lom_db->education_id));
    $tmp_obj->setDifficult(ArchibaldLomSaveHandler::loadVocabular('education:difficult', $lom_db->education_id));

    $duration = new ArchibaldLomDataDuration();
    $duration->setDuration($row['typicallearningtime_duration']);
    $duration->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($row['typicallearningtime_description']));
    $tmp_obj->setTypicalLearningTime($duration);
    unset($duration);

    $tmp_obj->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($row['description']));

    $sub_db = db_query('
      SELECT *
        FROM {archibald_education_learningresourcetype}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {

      $tmp_obj->addLearningResourceType(
        ArchibaldLomSaveHandler::loadVocabular(
          'education:learningResourceType_' . $lom_db->education_id,
          $row['learningresourcetype_id']
        )
      );
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_education_intendedenduserrole}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );
    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addIntendedEndUserRole(
        ArchibaldLomSaveHandler::loadVocabular(
          'education:intendedEndUserRole_' . $lom_db->education_id,
          $row['intendedenduserrole_id']
        )
      );
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_education_context}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addContext(
        ArchibaldLomSaveHandler::loadVocabular('education:context_' . $lom_db->education_id, $row['context_id'])
      );
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_education_typicalagerange}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addTypicalAgeRange(ArchibaldLomSaveHandler::requestLangStringObj($row['typicalagerange']));
    }
    unset($sub_db);

    $sub_db = db_query('
      SELECT *
        FROM {archibald_education_language}
        WHERE education_id = :education_id',
      array(
        ':education_id' => $lom_db->education_id,
      )
    );

    while ($row = $sub_db->fetchAssoc()) {
      $tmp_obj->addLanguage($row['language']);
    }
    unset($sub_db);


    $lom->setEducation($tmp_obj);

    $res = db_query('
      SELECT *
      FROM {archibald_annotations}
      WHERE lom_id = :lom_id
      AND lom_version = :lom_version',
      array(
        ':lom_id' => $lom_db->lom_id,
        ':lom_version' => $lom_db->version,
      )
    );
    while ($annotion_db = $res->fetchAssoc()) {
      $tmp_obj = new ArchibaldLomDataAnnotation();
      $tmp_obj->setFields($annotion_db);

      $datetime = new ArchibaldLomDataDateTime($annotion_db['date_date']);
      $datetime->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($annotion_db['date_description']));
      $tmp_obj->setDate($datetime);
      unset($datetime);

      $tmp_obj->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($annotion_db['description']));
      $lom->addAnnotation($tmp_obj);
    }

    $res = db_query('
      SELECT *
        FROM {archibald_relations}
        WHERE lom_id = :lom_id
        AND lom_version = :lom_version
        ORDER BY relation_id',
      array(
        ':lom_id' => $lom_db->lom_id,
        ':lom_version' => $lom_db->version,
      )
    );

    while ($relation_db = $res->fetchAssoc()) {
      $new_resource = new ArchibaldLomDataResource();
      $new_resource->addIdentifier(new ArchibaldLomDataIdentifier('educa', $relation_db['relation_id']));

      $tmp_obj = new ArchibaldLomDataRelation();
      $tmp_obj->setKind(ArchibaldLomSaveHandler::loadVocabular('lom:relations', $relation_db['kind']));
      $tmp_obj->setResource($new_resource);

      $lom->addRelation($tmp_obj);
    }

    $res = db_query('
      SELECT *
        FROM {archibald_classification}
        WHERE lom_id = :lom_id
        AND lom_version = :lom_version
        ORDER BY classification_id',
      array(
        ':lom_id' => $lom_db->lom_id,
        ':lom_version' => $lom_db->version,
      )
    );
    while ($classification_db = $res->fetchAssoc()) {
      $tmp_obj = new ArchibaldLomDataClassification();
      $tmp_obj->setFields($classification_db);
      $tmp_obj->setPurpose(ArchibaldLomSaveHandler::loadVocabular('lom:classification', $tmp_obj->classification_id));
      $tmp_obj->setDescription(ArchibaldLomSaveHandler::requestLangStringObj($classification_db['description']));

      $sub_db = db_query('
        SELECT *
          FROM {archibald_classification_keywords}
          WHERE classification_id = :classification_id',
        array(
          ':classification_id' => $tmp_obj->classification_id,
        )
      );
      while ($row = $sub_db->fetchAssoc()) {
        $tmp_obj->addKeyword(ArchibaldLomSaveHandler::requestLangStringObj($row['keyword']));
      }
      unset($sub_db);

      $tp_res = db_query('
        SELECT tp.*
        FROM {archibald_classification_taxon} ctp
        JOIN {archibald_taxon_path} tp
          ON (ctp.taxon_path_id = tp.taxon_path_id)
        WHERE classification_id = :classification_id
        ORDER BY ctp.taxon_path_id',
        array(
          ':classification_id' => $classification_db['classification_id'],
        )
      );
      while ($taxonpath_db = $tp_res->fetchAssoc()) {

        $tmp_taxonpath_obj = new ArchibaldLomDataTaxonPath();
        $tmp_taxonpath_obj->setFields($taxonpath_db);
        $tmp_taxonpath_obj->setSource(ArchibaldLomSaveHandler::requestLangStringObj($taxonpath_db['source']));

        $t_res = db_query('
          SELECT *
            FROM {archibald_taxon}
            WHERE taxon_path_id = :taxon_path_id
            ORDER BY weight ASC',
          array(
            ':taxon_path_id' => $taxonpath_db['taxon_path_id'],
          )
        );
        while ($taxon_db = $t_res->fetchAssoc()) {
          $taxon = new ArchibaldLomDataTaxon();
          $taxon->setFields($taxon_db);
          $taxon->setEntry(ArchibaldLomSaveHandler::requestLangStringObj($taxon_db['entry']));
          $tmp_taxonpath_obj->addTaxon($taxon);
        }


        $tmp_obj->addTaxonPath($tmp_taxonpath_obj);
      }
      $lom->addClassification($tmp_obj);
    }

    $res = db_query('
      SELECT *
        FROM {archibald_curriculum}
        WHERE lom_id = :lom_id
        AND lom_version = :lom_version
        ORDER BY curriculum_id',
      array(
        ':lom_id' => $lom_db->lom_id,
        ':lom_version' => $lom_db->version,
      )
    );
    while ($curriculum_db = $res->fetchAssoc()) {
      $tmp_obj = new ArchibaldLomDataCurriculum();
      $tmp_obj->setFields($curriculum_db);
      $lom->addCurriculum($tmp_obj);
    }

    cache_set($cid, $lom, 'cache', CACHE_TEMPORARY);

    return $lom;
  }

  /**
   * insert the given contribute for given type into database
   * and returns the inserted id
   *
   * @param ArchibaldLomDataContribute $contribute
   * @param string $type
   *
   * @return integer
   *   the inserted contribute id
   */
  private static function insertContribute(ArchibaldLomDataContribute &$contribute, $type) {

    $fields = array(
      'type' => $type,
      'role' => $contribute->getRole()->getValue(),
      'date_date' => $contribute->getDate()->getDatetime(),
      'date_description' => ArchibaldLomSaveHandler::saveOrInsertLangString(
        $contribute->getDate()->getDescription()
      ),
    );
    $contribute_inserted_id = db_insert('archibald_contribute')
      ->fields($fields)
      ->execute();

    ArchibaldLomSaveHandler::insertVocabular($contribute->getRole(), 'contribute:' . $type, $contribute_inserted_id);

    $contribute->contribute_id = $contribute_inserted_id;
    foreach ($contribute->getEntity() as $entity) {
      $entity_id = '';
      if (class_exists('ArchibaldLomContributer')) {
        $archibald_lom_contributer = new ArchibaldLomContributer($entity);
        $entity_id = $archibald_lom_contributer->getId();
      }
      $fields = array(
        'entity' => $entity,
        'entity_id' => $entity_id,
        'contribute_id' => $contribute_inserted_id,
      );
      db_insert('archibald_contribute_entity')
        ->fields($fields)
        ->execute();
    }
    return $contribute_inserted_id;
  }

  /**
   * load contribute from database
   *
   * @param int $contribute_id
   *
   * @return ArchibaldLomDataContribute
   */
  private static function loadContribute($contribute_id, $type) {
    $contribute = new ArchibaldLomDataContribute();
    $contribute_db = db_query('
      SELECT *
        FROM {archibald_contribute}
        WHERE contribute_id = :contribute_id',
      array(
        ':contribute_id' => $contribute_id,
      )
    )->fetchAssoc();
    $contribute->setFields($contribute_db);
    $contribute->setDate(
      new ArchibaldLomDataDateTime(
        $contribute_db['date_date'],
        ArchibaldLomSaveHandler::requestLangStringObj($contribute_db['date_description'])
      )
    );

    $contribute->setRole(ArchibaldLomSaveHandler::loadVocabular('contribute:' . $type, $contribute->contribute_id));

    $res = db_query('
      SELECT *
        FROM {archibald_contribute_entity}
        WHERE contribute_id = :contribute_id
        ORDER BY contribute_id',
      array(
        ':contribute_id' => $contribute_id,
      )
    );

    while ($row = $res->fetchAssoc()) {
      $contribute->addEntity($row['entity']);
    }
    return $contribute;
  }

  /**
   * delte contribute from database
   *
   * @param integer $contribute_id
   * @param string $type
   */
  private static function deleteContribute($contribute_id, $type) {
    if ($contribute_id instanceof ArchibaldLomDataContribute) {
      $id = $contribute_id->contributeIid;
    }
    else {
      $id = $contribute_id;
      $contribute_id = ArchibaldLomSaveHandler::loadContribute($contribute_id, $type);
    }

    ArchibaldLomSaveHandler::deleteVocabular('contribute:' . $type, $contribute_id->contributeId);

    $db = db_delete('archibald_contribute_entity')
      ->condition('contribute_id', $contribute_id->contributeId, '=')
      ->execute();

    $db = db_delete('archibald_contribute')
      ->condition('contribute_id', $contribute_id->contributeId, '=')
      ->execute();
  }

  /**
   *
   * @param ArchibaldLomDataVocabulary $vocabular
   * @param string $type
   * @param integer $id
   *   uuid
   *
   * @return string
   *   the inserted contribute value
   */
  private static function insertVocabular($vocabular, $type, $id) {

    if (!($vocabular instanceof ArchibaldLomDataVocabulary)) {
      return '';
    }

    $value = $vocabular->getValue();
    if (empty($value)) {
      return '';
    }
    $fields = array(
      'id' => $id,
      'type' => $type,
      'source' => $vocabular->getSource(),
      'value' => $value,
    );

    try {
      db_insert('archibald_vocabulars')
        ->fields($fields)
        ->execute();
    }
    catch (Exception $e) {
      # do nothing if we insert it twice
      # $e->getMessage()
    }
    return $vocabular->getValue();
  }

  /**
   *
   * @param String $type
   * @param Int $id
   */
  private static function deleteVocabular($type, $id = 0) {
    $db = db_delete("archibald_vocabulars")->condition('type', $type, '=');
    if ($id != 0) {
      $db = $db->condition('id', $id, '=');
    }
    $db->execute();
  }

  /**
   *
   * @param int $lom_id
   * @param string $type
   *
   * @return ArchibaldLomDataVocabulary
   */
  private static function loadVocabular($type, $id) {

    $vocabular = new ArchibaldLomDataVocabulary();
    $vocabular_db = db_select("archibald_vocabulars", 'archibaldv')
      ->fields('archibaldv')
      ->condition('id', $id, '=')
      ->condition('type', $type, '=')
      ->execute();

    $vocabular->setFields($vocabular_db->fetch());
    return $vocabular;
  }

  /**
   * generate new uuid
   *
   * @return string
   *   uuid
   */
  private static function generateUuid() {
    return md5(uniqid(mt_rand(), TRUE));
  }

  /**
   * Load from given id the langstring object and return it
   *
   * @param string $id
   *
   * @return ArchibaldLomDataLangString
   */
  public static function requestLangStringObj($id) {
    if (is_object($id)) {
      return $id;
    }
    $return_obj = new ArchibaldLomDataLangString();
    $return_obj->strings = array();
    $return_obj->setId($id);
    $res = db_select('archibald_langstring_terms', 'archibaldls')
      ->fields('archibaldls')
      ->condition('langstring_terms_id', $id)
      ->execute();
    while ($row = $res->fetchAssoc()) {
      $return_obj->setString($row['term'], $row['language']);
    }
    if (is_null($return_obj->strings)) {
      $return_obj->strings = array();
    }
    return $return_obj;
  }

  /**
   * save a ArchibaldLomDataLangString to databse and return id
   *
   * @param string $langstring
   *
   * @return string
   *    the langstring id
   */
  public static function saveOrInsertLangString(&$langstring) {
    if (empty($langstring)) {
      return FALSE;
    }

    if (is_array($langstring)) {
      $tlangstring = new ArchibaldLomDataLangString();
      $tlangstring->setFields($langstring);
      $langstring = $tlangstring;
      unset($tlangstring);
    }

    if (($langstring instanceof ArchibaldLomDataLangString) === FALSE) {
      return '';
    }

    $is_empty = TRUE;
    $strings = $langstring->getStrings();
    if (empty($strings)) {
      return FALSE;
    }
    else {

      foreach ($strings as $language => $term) {
        if (!empty($term)) {
          $is_empty = FALSE;
        }
      }

    }
    if ($is_empty == TRUE) {
      return FALSE;
    }

    $insert = FALSE;
    if ($langstring->getId() == '') {
      // is a new string
      // check for duplicates in db
      // a
      $cur_letter = 97;
      foreach ($strings as $language => $term) {
        if ($cur_letter === 97) {
          $query = db_select('archibald_langstring_terms', 'a');
        }
        else {
          $on = 'a.langstring_terms_id = ' . chr($cur_letter) . '.langstring_terms_id';
          $query->join('archibald_langstring_terms',  chr($cur_letter), $on);
        }
        $query->condition(chr($cur_letter) . '.term', $term);
        $query->condition(chr($cur_letter) . '.language', $language);
        $cur_letter++;
      }
      $query->addField('a', 'langstring_terms_id', 'langstring_terms_id');
      $res_test = $query->execute()->fetchAssoc();

      if (!empty($res_test['langstring_terms_id'])) {
        // found match
        $langstring->setId($res_test['langstring_terms_id']);
        return $res_test['langstring_terms_id'];
      }
      else {
        // no duplicates found
        $insert = TRUE;
        $langstring->setId(ArchibaldLomSaveHandler::generateUuid());
      }
    }

    foreach ($strings as $language => $term) {
      $res_test = db_select('archibald_langstring_terms', 'n')
        ->fields('n')->condition('langstring_terms_id', $langstring->getId())
        ->condition('language', $language)
        ->execute()
        ->fetchAssoc();

      if (empty($res_test)) {
        $fields = array(
          'langstring_terms_id' => $langstring->getId(),
          'language' => $language,
          'term' => $term,
        );

        try {
          db_insert('archibald_langstring_terms')
            ->fields($fields)
            ->execute();
        }
        catch (Exception $e) {
          // its only for debuging, it should never trown
          echo "\n\n### {$e->getMessage()}\n";
          foreach (debug_backtrace() as $e) {
            echo $e['file'] . ':' . $e['line'] . "\t" . $e['function'] . "\n";
          }
          die('archibald_langstring_terms error');
        }
      }
      else {
        if (empty($term)) {
          $res = db_delete('archibald_langstring_terms')
            ->condition('langstring_terms_id', $langstring->getId())
            ->condition('language', $language)
            ->execute();
        }
        else {
          $res = db_update('archibald_langstring_terms')
            ->fields(array('term' => $term))
            ->condition('langstring_terms_id', $langstring->getId())
            ->condition('language', $language)
            ->execute();
        }
      }
    }
    return $langstring->getId();
  }

  /**
   * Get a lom id, if 0 given it will generate a uuid
   *
   * @param string $lom_id
   *
   * @return string
   */
  private static function requestLomId($lom_id = 0) {
    if (empty($lom_id)) {
      $lom_id = ArchibaldLomSaveHandler::generateUuid();
    }
    return $lom_id;
  }

  /**
   * get responsible uid
   *
   * @param string $lom_id
   *
   * @return integer $uid
   */
  public static function getResponsibleUid($lom_id) {
    return db_query('
      SELECT responsible_uid
        FROM {archibald_lom_stats}
        WHERE lom_id = :lom_id',
      array(
        ':lom_id' => $lom_id,
      )
    )->fetchField(0);
  }

  /**
   * set responsible uid
   *
   * @param string $lom_id
   * @param integer $uid
   */
  public static function setResponsibleUid($lom_id, $uid) {
    $res = db_update('archibald_lom_stats')->fields(array('responsible_uid' => $uid))->condition('lom_id', $lom_id);
    $res->execute();
  }

  /**
   * get if local published or not
   *
   * @param string $lom_id
   *
   * @return string version_id
   */
  public static function getLocalPublished($lom_id) {
    $result = db_query('
      SELECT local_published
      FROM {archibald_lom_stats}
      WHERE lom_id = :lom_id',
      array(
        ':lom_id' => $lom_id,
      )
    )
    ->fetchField(0);

    return (empty($result)) ? FALSE : $result;
  }

  /**
   * set local publish status
   *
   * @param string $lom_id
   * @param string $published
   *   lom_version string or
   *   NULL for unpublish
   */
  public static function setLocalPublished($lom_id, $published) {
    $res = db_update('archibald_lom_stats')->fields(
      array(
        'local_published' => $published,
      )
    )
    ->condition('lom_id', $lom_id)
    ->execute();
  }
}

