<?php

/**
 * @file
 * The settings formulars for archibald 2
 */

/**
 * Implements hook_settings_language().
 */
function archibald_admin_settings_language() {

  $form = array();

  $form['archibald'] = array(
    '#type' => 'fieldset',
    '#title' => t('Language settings'),
    '#collapsible' => TRUE,
  );

  include_once 'languages/en.php';

  $form['archibald'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Language settings'),
    '#default_value' => variable_get('archibald_language_settings', array()),
    '#options' => $languages,
    '#description' => t('This languages will be available for descriptions.'),
    '#theme' => 'archibald_language_settings',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  #$form['#validate'][] = 'archibald_admin_settings_language_validate';

  return $form;
}

/**
 * validate archibald_admin_settings admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_language_validate(&$form, &$form_state) {
  $valid = FALSE;
  foreach ($form_state['values']['archibald'] as $v) {
    if ($v != "0") {
      $valid = TRUE;
      break;
    }
  }
  if (!$valid) {
    form_set_error('archibald', t('Please choose a minimum of 1 language'));
  }
}

/**
 * form submit handler for archibald_admin_settings admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_language_submit($form, &$form_state) {
  $values = array();
  foreach ($form_state['values']['archibald'] as $v) {
    if ($v == '0') {
      continue;
    }
    $values[] = $v;
  }
  variable_set('archibald_language_settings', $values);

  drupal_set_message(t('The configuration has been saved.'));
}

/**
 * settings form for archibald solr search interface
 * Implements hook_settings().
 */
function archibald_admin_settings_fields() {
  $form = array();
  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');

  $fields = archibald_get_field_minimum_requirements();

  $form['options'] = array(
    '#type' => 'fieldset',
    '#collapsible' => FALSE,
    '#tree' => FALSE,
  );

  $can_central_publish = archibald_can_publish_central();


  foreach (archibald_get_field_info() as $section_id => $section) {
    if ($section_id == 'relation') {
      continue;
    }

    foreach ($section['fields'] AS $field_id => $field) {

      $name = $section_id . '_' . $field_id;

      $form['options'][$name] = array(
        '#title' => t('Field: @field', array('@field' => $section['label'] . ' > ' . $field['label'])),
        '#description' => t('Please choose the required option for each field.'),
        '#type' => 'radios',
        '#options' => array(
          ARCHIBALD_RES_FIELD_OPTIONAL => t('Optional'),
          ARCHIBALD_RES_FIELD_RECOMMEND => t('Recommend'),
          ARCHIBALD_RES_FIELD_REQUIRED => t('Required'),
        ),
        '#default_value' => archibald_get_field_requirements($name),
      );

      if (archibald_get_field_minimum_requirements($section_id . '.' . $field_id) > ARCHIBALD_RES_FIELD_RECOMMEND) {
        unset($form['options'][$name]['#options'][ARCHIBALD_RES_FIELD_OPTIONAL]);
        unset($form['options'][$name]['#options'][ARCHIBALD_RES_FIELD_RECOMMEND]);
      }

      if ($can_central_publish && in_array($name, array('general_description', 'education_learningresourcetype', 'classification_context'))) {
        $form['options'][$name]['#disabled'] = TRUE;
        $form['options'][$name]['#default'] = ARCHIBALD_RES_FIELD_REQUIRED;
        $form['options'][$name]['#title'] .= '<br /> (' . t('This field is always required, because national publishing is available.') . ')';
      }
    }
  }

  /**
   * Not used. It is not a good idea to require a relation. hh 03.07.2012
   *
  $form['options']['relation'] = array(
    '#title' => t('Field: @field', array('@field' => archibald_get_field_label('relation'))),
    '#description' => t('Please choose the required option for each field.'),
    '#type' => 'radios',
    '#options' => array(
      ARCHIBALD_RES_FIELD_OPTIONAL => t('Optional'),
      ARCHIBALD_RES_FIELD_RECOMMEND => t('Recommend'),
      ARCHIBALD_RES_FIELD_REQUIRED => t('Required'),
    ),
    '#default_value' => archibald_get_field_requirements('relation'),
  ); */

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * theme a archibald form admin settings field
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_admin_settings_fields($variables) {
  $form = reset($variables);

  $html = '';

  $table = array(
    'header' => array(
      t('Field'),
      array(
        'data' => t('Options'),
        'colspan' => 3,
      ),
    ),
    'rows' => array(),
    'class' => 'archibald_admin_fields',
  );

  foreach (element_children($form['options'], TRUE) as $k) {
    $field = $form['options'][$k];

    $row = array();
    $row[] = $field['#title'];

    for ($i=0;$i<=2;$i++) {
      if (empty($field[$i])) {
        $row[] = '-';
      }
      else {
        $row[] = drupal_render($field[$i]);
      }
    }

    $table['rows'][] = $row;
  }

  $html .= theme('table', $table);
  $html .= drupal_render($form['submit']);

  // prevent endless loop
  $form['#theme'] = '';
  unset($form['options']);
  $html .= drupal_render($form);

  return $html;
}

/**
 * form submit handler for archibald_admin_settings_solr
 * archibald solr admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_fields_submit($form, &$form_state) {
  foreach ($form_state['values'] as $field => $value) {
    $in_array = in_array($field, array('submit', 'form_build_id', 'form_token', 'form_id', 'op'));
    if ($in_array) {
      continue;
    }

    variable_set('archibald_field_require_type_' . drupal_strtolower($field), $value);
  }
  drupal_set_message(t('Field configuration saved successfully'), 'ok');
}

/**
 * settings form for archibald solr search interface
 * Implements hook_settings().
 */
function archibald_admin_settings_solr() {
  $form = array();

  if (!module_exists('apachesolr')) {
    form_set_error(
      'archibald',
      t(
        'Sorry, !module module was not installed. Please install it first.',
        array(
          '!module' => '<a href="http://drupal.org/project/apachesolr" ' .
          'target="_blank">apachesolr</a>',
        )
      )
    );
    return array();
  }

  try {
    // Get the $solr object
    $solr = apachesolr_get_solr();
    // If there is no server available, don't continue.
    if (!$solr->ping(variable_get('apachesolr_ping_timeout', 4))) {
      throw new Exception(t('No Solr instance available.'));
    }
  }
  catch (Exception $e) {
    form_set_error(
      'archibald',
      t('Sorry, no Solr instance available. Please configurate apachesolr module first.') . '<br />' . l(t('configure'), 'admin/config/search/apachesolr')
    );
    return array();
  }

  if (variable_get('archibald_solr_search_activ') == 0) {
    drupal_set_message(t('Solr search is currently disabled, please save to activate it.'), 'warning');
  }

  $weights = drupal_map_assoc(
    array(
      '21.0',
      '16.0',
      '11.0',
      '7.0',
      '5.0',
      '4.0',
      '3.5',
      '3.0',
      '2.8',
      '2.6',
      '2.4',
      '2.2',
      '2.0',
      '1.9',
      '1.8',
      '1.7',
      '1.6',
      '1.5',
      '1.4',
      '1.3',
      '1.2',
      '1.1',
      '1.0',
      '0.9',
      '0.8',
      '0.7',
      '0.6',
      '0.5',
      '0.4',
      '0.3',
      '0.2',
      '0.1',
    )
  );
  $weights['0'] = t('Ignore');

  $form['archibald_solr_search_activ'] = array(
    '#title' => t('Activate solr search'),
    '#description' => t('If this option is disabled, the search function will work by default sql search system.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_solr_search_activ', 1),
  );

  $form['archibald_content_collections'] = array(
    '#title' => t('Search in collections'),
    '#description' => t('"Search in collections" implies that children of collections will only display in combination with the collection head.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_content_collections', 0),
  );

  $form['biases'] = array(
    '#type' => 'fieldset',
    '#title' => t('Result biasing'),
    '#description' => t('Give bias to certain properties when ordering the search results. Any value except "Ignore" will increase the score of the given type in search results. Choose "Ignore" to ignore any given property.'),
    '#collapsible' => TRUE,
  );

  $form['biases']['archibald_solr_boosting_normal'] = array(
    '#type' => 'fieldset',
    '#title' => t('Main items'),
    '#collapsible' => FALSE,
  );

  $keys = array('label' => 'Title',
    'content' => t('Description'),
    'tm_keywords' => t('Keywords'),
    'tm_coverage' => t('Coverage'),
    'ts_otherPlattformRequirements' => t('Technical requirements'),
    'ts_education_description' => t('Educational description'),
    'tm_contributer' => t('Contributors'),
    'tm_vocabular_text' => t('Vocabulary text'),
  );
  foreach ($keys as $key => $title) {
    $form['biases']['archibald_solr_boosting_normal']['archibald_solr_bias_' . $key] = array(
      '#type' => 'select',
      '#options' => $weights,
      '#title' => $title,
      '#default_value' => archibald_get_solr_bias_value($key),
    );
  }

  $form['biases']['archibald_solr_boosting_childs'] = array(
    '#type' => 'fieldset',
    '#title' => t('Collection child items'),
    '#description' => t('This options looks up for collection child within search and is only used if collection search is activated.'),
    '#collapsible' => FALSE,
  );

  $keys = array('tm_child_label' => t('Title'),
    'tm_child_content' => t('Description'),
    'tm_child_keywords' => t('Keywords'),
    'tm_child_coverage' => t('Coverage'),
    'tm_child_otherPlattformRequirements' => t('Technical requirements'),
    'tm_child_education_description' => t('Educational description'),
    'tm_child_contributer' => t('Contributors'),
    'tm_child_vocabular_text' => t('Vocabulary text'),
  );
  foreach ($keys as $key => $title) {
    $form['biases']['archibald_solr_boosting_childs']['archibald_solr_bias_' . $key] = array(
      '#type' => 'select',
      '#options' => $weights,
      '#title' => t($title),
      '#default_value' => archibald_get_solr_bias_value($key),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * form submit handler for archibald_admin_settings_solr
 * archibald solr admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_solr_submit($form, &$form_state) {
  $values = array();

  variable_set('archibald_solr_search_activ', $form_state['values']['archibald_solr_search_activ']);

  variable_set('archibald_content_collections', $form_state['values']['archibald_content_collections']);

  foreach ($form_state['values'] as $key => $val) {
    if (drupal_substr($key, 0, 15) == 'archibald_solr_bias_') {
      $org_val = archibald_get_solr_bias_value(drupal_substrsubstr($key, 15), TRUE);

      if ($org_val == $val) {
        // if val like educa preset dont save it
        variable_del(drupal_substr($key, 15));
      }
      else {
        variable_set($key, $val);
      }
    }
  }

  drupal_set_message(t('The configuration has been saved.'));
}

/**
 * Implements hook_settings().
 */
function archibald_admin_settings() {

  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');

  $form = array();

  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 3
  );

  $form['general']['archibald_switch_language_while_add'] = array(
    '#title' => t('Switch to language of added description.'),
    '#description' => t('Before a new description is added the user must choose the main language of the description. The user interface will switch to the selected language.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_switch_language_while_add', 1),
  );

  $form['general']['archibald_resource_save_able_with_errors'] = array(
    '#title' => t('Allow saving a resource even when mandatory fields are missing'),
    '#description' => t('In case this option is disabled the user is not allowed to save a resource until all mandatory fields are available.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_resource_save_able_with_errors', 1),
  );

  $form['general']['archibald_coverage_autocomplete_behaviour'] = array(
    '#title' => t('Search for keywords only within the selected language'),
    '#description' => t('When a description is edited only keywords of the selected language are available.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_coverage_autocomplete_behaviour', 1),
  );

  $form['general']['archibald_form_blur_message'] = array(
    '#title' => t('Display a warning when user leaves the description edit form without saving.'),
    '#description' => t('This option will only work with Internet Explorer 7 or newer and Firefox.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_form_blur_message', 0),
  );


  $form['general']['identifier_fileExtensions'] = array(
    '#type' => 'textfield',
    '#title' => t('Identifer file extensions'),
    '#description' => t('Space separated list of file extensions
      which are allowed to upload as resource identifer.') . '<br />' .
      t('Default') . ': ' . ARCHIBALD_DEFAULT_IDENTIFIER_FILE_EXTENSIONS,
    '#default_value' => variable_get('archibald_identifier_fileExtensions', ''),
    '#size' => 50,
    '#maxlength' => 500,
  );

  // save able with error
  $form['front_end_search'] = array(
    '#type' => 'fieldset',
    '#title' => t('Frontend search interface'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 4
  );

  $form['front_end_search']['archibald_search_in_current_language'] = array(
    '#title' => t('Search only for descriptions in selected language.'),
    '#description' => t('Search only for descriptions in selected language.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('archibald_search_in_current_language', 1),
  );

  $form['archibald'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connection to the Swiss national school library'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 1,
  );

  $content_partner = archibald_load_content_partner();
  $rows = array();
  $can_change_working_mode = TRUE;
  foreach ($content_partner as $content_partner_id => $data) {
    // Define all the Operations
    $ops = array();

    $content_partner_name = l($data['name'], 'admin/config/services/archibald/settings/edit/' . $data['content_partner_id'],
      array(
        'query' => array(
          'destination' => $_GET['q'],
        ),
      )
    );

    $ops['edit'] = array(
      'class' => 'operation',
      'data' => l(t('Edit'), 'admin/config/services/archibald/settings/edit/' . $data['content_partner_id']),
    );

    // Check if content partner is in use
    // and if it is the last one
    if (!empty($data['usage_count'])) {
      $can_change_working_mode = FALSE;
      $ops['delete'] = t('With this account there are @usage_count descriptions published.',
        array(
          '@usage_count' => $data['usage_count'],
        )
      );
    }
    elseif (count($content_partner) < 2) {
      $ops['delete'] = t('The last account, cannot be deleted');
    }
    else {
      $ops['delete'] = array(
        'class' => 'operation',
        'data' => l(
          t('Delete'), 'admin/config/services/archibald/settings/delete/' . $data['content_partner_id'],
          array(
            'query' => array(
              'destination' => current_path(),
            ),
          )
        ),
      );
    }

    $headers = array(
      t('Name'),
      t('Icon'),
      t('Username'),
      array(
        'data' => t('Operations'),
        'colspan' => count($ops),
      ),
    );

    // get partner icon and render it if possible
    $icon = '';
    if (!empty($data['icon_file']->uri)) {
      $icon = theme(
        'image_style', array(
          'style_name' => 'archibald_contributer',
          'path' => $data['icon_file']->uri,
          'getsize' => FALSE,
        )
      );
    }

    $rows[$content_partner_id] = array(
      'data' => array(
        // Cells
        array(
          'data' => $content_partner_name,
        ),
        $icon,
        check_plain($data['username']),
      ),
    );
    // Add the links to the page
    $rows[$content_partner_id]['data'] = array_merge($rows[$content_partner_id]['data'], $ops);
  }

  $form['archibald']['table'] = array(
    '#theme' => 'table',
    '#header' => $headers,
    '#rows' => array_values($rows),
    '#attributes' => array('class' => array('admin-archibald')),
  );

  $form['archibald']['add_new_content_partner'] = array(
    '#markup' => l(t('Add new content partner'), 'admin/config/services/archibald/settings/add') . '<br /><br />'
  );

  $form['publication'] = array(
    '#type' => 'fieldset',
    '#title' => t('The publication of descriptions'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 2,
  );

  if ( archibald_can_publish_central() != TRUE ) {
    $form['publication']['archibald_publish_type_info'] = array(
      '#markup' => t('As long no content partner is configured, resources can only be published local')
    );
  }
  else {
    $form['publication']['archibald_publish_type'] = array(
      '#type' => 'radios',
      '#options' => array(
      ARCHIBALD_PUBLISH_SEPERATE => t('Descriptions can be published locally or nationally. Use this option if you want to publish  only selected descriptions to the national catalogue.'),
      ARCHIBALD_PUBLISH_TOGETHER => t('Descriptions must be published locally and nationally. Use this option if you want to publish all your descriptions to the national catalogue.'),
      ),
      '#default_value' => variable_get('archibald_publish_type', ARCHIBALD_PUBLISH_SEPERATE),
    );
  }

  $can_change_working_mode_notice = '';
  if (!$can_change_working_mode) {
    $can_change_working_mode_notice = t('You can\'t change the working mode because you have published descriptions. Please unpublish all descriptions first.');
  }
  $working_mode_description = t('Please specify the working mode, use demo if you want to play in a sandbox environment.<br />If you think you are ready to go switch to live mode.<br />Within the sandbox demo mode you can do anything you want.');
  $form['archibald']['working_mode'] = array(
    '#type' => 'radios',
    '#default_value' => variable_get('archibald_working_mode', ARCHIBALD_WORKING_MODE_DEMO),
    '#title' => t('Working mode'),
    '#disabled' => !$can_change_working_mode,
    '#options' => array(
      ARCHIBALD_WORKING_MODE_DEVELOPMENT => t('Development'),
      ARCHIBALD_WORKING_MODE_DEMO => t('Demo'),
      ARCHIBALD_WORKING_MODE_LIVE => t('Live'),
    ),
    '#description' => $can_change_working_mode_notice . $working_mode_description,
  );

  $form['ontolog'] = array(
    '#type' => 'fieldset',
    '#title' => t('Ontology settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 5
  );

  $form['ontolog']['archibald_ontology_url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('Set URL for the ontology service'),
    '#default_value' => variable_get('archibald_ontology_url', 'http://ontology.biblio.educa.ch'),
    '#size' => 50,
    '#maxlength' => 250,
    '#required' => TRUE,
  );

  $form['email_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('E-Mails'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#weight' => 6
  );


  $form['email_fieldset']['email'] = array(
    '#type' => 'vertical_tabs',
  );

  // These email tokens are shared for all settings, so just define
  // the list once to help ensure they stay in sync.
  $email_token_help = t('Available variables are: [site:name], [site:url], [user:name], [user:mail], [lom:id], [lom:url], [lom:title], [overview:proposal-url], [overview:to-publish-url], [overview:own-url].');

  $roles = array_map('check_plain', user_roles(TRUE));

  $description = t('Edit the notification e-mail message sent to users with editor role. When a new resource has been created by an user who is not in editor group');
  $description .= ' ' . $email_token_help;

  $description .= '<br /><h5>' . t('Please do only insert English content into content fields. For translation please click the "Translate" link.') . '</h5>';
  $form['email_fieldset']['email_new_unavailable'] = array(
    '#type' => 'fieldset',
    '#title' => t('New description (created by non editor role)'),
    '#collapsible' => TRUE,
    '#description' => $description,
    '#group' => 'email',
  );

  $form['email_fieldset']['email_new_unavailable']['archibald_mail_new_unavailable_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => archibald_mail_text('new_unavailable_subject', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('new_unavailable_subject'),
    '#maxlength' => 180,
  );

  $form['email_fieldset']['email_new_unavailable']['archibald_mail_new_unavailable_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => archibald_mail_text('new_unavailable_body', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('new_unavailable_body'),
    '#rows' => 15,
  );

  $form['email_fieldset']['email_new_unavailable']['archibald_mail_new_unavailable_group'] = array(
    '#type' => 'select',
    '#title' => t('Role'),
    '#default_value' => variable_get('archibald_mail_new_unavailable_group', 1),
    '#options' => $roles,
    '#description' => t('Define here which user role should receive the E-Mail'),
  );

  $form['email_fieldset']['email_status2final'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status of description has been changed to final.'),
    '#collapsible' => TRUE,
    '#description' => t('Edit the notification e-mail message sent to users with publisher role. When the status of a resource has been changed from draft to final.') . ' ' . $email_token_help,
    '#group' => 'email',
  );

  $form['email_fieldset']['email_status2final']['archibald_mail_status2final_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => archibald_mail_text('status2final_subject', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('status2final_subject'),
    '#maxlength' => 180,
  );

  $form['email_fieldset']['email_status2final']['archibald_mail_status2final_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => archibald_mail_text('status2final_body', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('status2final_body'),
    '#rows' => 15,
  );

  $form['email_fieldset']['email_status2final']['archibald_mail_status2final_group'] = array(
    '#type' => 'select',
    '#title' => t('Role'),
    '#default_value' => variable_get('archibald_mail_status2final_group', 1),
    '#options' => $roles,
    '#description' => t('Define here which user role should receive the E-Mail'),
  );

  $form['email_fieldset']['email_status2draft'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status of description has been changed from final to draft.'),
    '#collapsible' => TRUE,
    '#description' => t('Edit the notification e-mail message sent to the responsible editor. When the status of a resource has been changed by a publisher to draft.') . ' ' . $email_token_help,
    '#group' => 'email',
  );

  $form['email_fieldset']['email_status2draft']['archibald_mail_status2draft_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => archibald_mail_text('status2draft_subject', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('status2draft_subject'),
    '#maxlength' => 180,
  );

  $form['email_fieldset']['email_status2draft']['archibald_mail_status2draft_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => archibald_mail_text('status2draft_body', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('status2draft_body'),
    '#rows' => 15,
  );

  $form['email_fieldset']['email_status2draft']['archibald_mail_status2draft_group'] = array(
    '#type' => 'select',
    '#title' => t('Role'),
    '#default_value' => variable_get('archibald_mail_status2draft_group', 1),
    '#options' => $roles,
    '#description' => t('Define here which user role should receive the E-Mail'),
  );

  $form['email_fieldset']['email_set_responsible'] = array(
    '#type' => 'fieldset',
    '#title' => t('Set a new responsible editor for a description'),
    '#collapsible' => TRUE,
    '#description' => t('Edit the notification e-mail message sent to the new responsible editor. When the current responsible editor passes this task.') . ' ' . $email_token_help,
    '#group' => 'email',
  );

  $form['email_fieldset']['email_set_responsible']['archibald_mail_set_responsible_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => archibald_mail_text('set_responsible_subject', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('set_responsible_subject'),
    '#maxlength' => 180,
  );

  $form['email_fieldset']['email_set_responsible']['archibald_mail_set_responsible_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => archibald_mail_text('set_responsible_body', NULL, array(), TRUE),
    '#description' => archibald_mail_translate_description('set_responsible_body'),
    '#rows' => 15,
  );

  $available_curriculums = archibald_get_available_curriculums();
  if (!empty($available_curriculums)) {

    $form['curriculum_fieldset'] = array(
      '#type' => 'fieldset',
      '#title' => t('Curriculum'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => 6,
      '#tree' => TRUE,
    );


    $form['curriculum_fieldset']['curriculum'] = array(
      '#type' => 'vertical_tabs',
    );

    foreach ($available_curriculums as $curriculum) {
      $classname = 'ArchibaldCurriculum' . drupal_ucfirst($curriculum);

      $curr_handler = new $classname();

      $form['curriculum_fieldset'][$curriculum] = array(
        '#type' => 'fieldset',
        '#title' => t('Settings for @name', array('@name' => $curr_handler->getName())),
        '#collapsible' => TRUE,
        '#group' => 'curriculum',
        '#tree' => TRUE,
      );

      $form['curriculum_fieldset'][$curriculum] += $curr_handler->getSettingsForm();
    }
  }


  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 7
  );

  $form['#validate'][] = 'archibald_admin_settings_validate';

  return $form;
}

/**
 * Returns the text which includes a translation link for the given mail key
 * @param string $key
 *  the email key
 * @return string
 */
function archibald_mail_translate_description($key) {
  return t('To translate the E-Mail text please click on the following link: !link', array(
    '!link' => l(t('Translate'), archibald_mail_get_translate_link($key), array('attributes' => array('target' => '_Blank')))
  ));
}

/**
 * Returns the path to the translation link
 *
 * @param string $en_text
 *  the english text
 * @return string
 *  the url to the translation
 */
function archibald_mail_get_translate_link($key) {

  $en_text = archibald_mail_text($key, NULL, array(), TRUE);

  $row = db_query("SELECT lid FROM {locales_source} WHERE textgroup = 'default' AND source = :source", array(
    ':source' => $en_text
  ))->fetchAssoc();

  return 'admin/config/regional/translate/edit/' . $row['lid'];
}
/**
 * form submit handler for archibald_admin_setting admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_submit($form, &$form_state) {

  variable_set('archibald_ontology_url', $form_state['values']['archibald_ontology_url']);

  // reset cron look after big processing error
  variable_set('archibald_upload_error', 0);

  // save archibald_working_mode
  variable_set('archibald_working_mode', $form_state['values']['working_mode']);

  switch (variable_get("archibald_working_mode")) {
    case ARCHIBALD_WORKING_MODE_DEVELOPMENT:
      variable_set("archibald_api_url", 'https://dsb_api.ubuntu');
      variable_set("archibald_fileserver_upload_url", 'http://files.ubuntu/file_server/upload.ajax');
      break;

    case ARCHIBALD_WORKING_MODE_DEMO:
      variable_set("archibald_api_url", 'https://demo-api-dsb.educa.ch');
      variable_set("archibald_fileserver_upload_url", 'https://demo-file-api-dsb.educa.ch/file_server/upload.ajax');
      break;

    case ARCHIBALD_WORKING_MODE_LIVE:
      variable_set("archibald_api_url", 'https://main-api-dsb.educa.ch');
      variable_set("archibald_fileserver_upload_url", 'https://file-api-dsb.educa.ch/file_server/upload.ajax');
      break;
  }

  // save archibald_identifier_fileExtensions
  variable_set('archibald_identifier_fileExtensions', $form_state['values']['identifier_fileExtensions']);

  // save archibald_identifier_fileExtensions
  if (!isset($form_state['values']['archibald_publish_type'])) {
    variable_set('archibald_publish_type', ARCHIBALD_PUBLISH_TOGETHER);
  }
  else {
    variable_set('archibald_publish_type', $form_state['values']['archibald_publish_type']);
  }

  // save archibald_switch_language_while_add
  variable_set('archibald_switch_language_while_add', $form_state['values']['archibald_switch_language_while_add']);

  // save archibald_resource_save_able_with_errors
  variable_set('archibald_resource_save_able_with_errors', $form_state['values']['archibald_resource_save_able_with_errors']);

  // save archibald_coverage_autocomplete_behaviour
  variable_set('archibald_coverage_autocomplete_behaviour', $form_state['values']['archibald_coverage_autocomplete_behaviour']);

  // save archibald_form_blur_message
  variable_set('archibald_form_blur_message', $form_state['values']['archibald_form_blur_message']);

  // save archibald_only_final_at_home
  /* variable_set(
    'archibald_only_final_at_home',
    $form_state['values']['archibald_only_final_at_home']
    ); */


  // save archibald_search_in_current_language
  variable_set('archibald_search_in_current_language', $form_state['values']['archibald_search_in_current_language']);


  // save mail configurations
  if ($form_state['values']['archibald_mail_new_unavailable_subject'] == archibald_mail_text('new_unavailable_subject', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_new_unavailable_subject'] = NULL;
  }

  if ($form_state['values']['archibald_mail_new_unavailable_body'] == archibald_mail_text('new_unavailable_body', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_new_unavailable_body'] = NULL;
  }

  if ($form_state['values']['archibald_mail_status2final_subject'] == archibald_mail_text('status2final_subject', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_status2final_subject'] = NULL;
  }

  if ($form_state['values']['archibald_mail_status2final_body'] == archibald_mail_text('status2final_body', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_status2final_body'] = NULL;
  }

  if ($form_state['values']['archibald_mail_status2draft_subject'] == archibald_mail_text('status2draft_subject', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_status2draft_subject'] = NULL;
  }

  if ($form_state['values']['archibald_mail_status2draft_body'] == archibald_mail_text('status2draft_body', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_status2draft_body'] = NULL;
  }

  if ($form_state['values']['archibald_mail_set_responsible_subject'] == archibald_mail_text('set_responsible_subject', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_set_responsible_subject'] = NULL;
  }

  if ($form_state['values']['archibald_mail_set_responsible_body'] == archibald_mail_text('set_responsible_body', NULL, array(), TRUE, TRUE)) {
    $form_state['values']['archibald_mail_set_responsible_body'] = NULL;
  }

  variable_set('archibald_mail_new_unavailable_subject', $form_state['values']['archibald_mail_new_unavailable_subject']);
  variable_set('archibald_mail_new_unavailable_body', $form_state['values']['archibald_mail_new_unavailable_body']);
  variable_set('archibald_mail_new_unavailable_group', $form_state['values']['archibald_mail_new_unavailable_group']);
  variable_set('archibald_mail_status2final_subject', $form_state['values']['archibald_mail_status2final_subject']);
  variable_set('archibald_mail_status2final_body', $form_state['values']['archibald_mail_status2final_body']);
  variable_set('archibald_mail_status2final_group', $form_state['values']['archibald_mail_status2final_group']);
  variable_set('archibald_mail_status2draft_subject', $form_state['values']['archibald_mail_status2draft_subject']);
  variable_set('archibald_mail_status2draft_body', $form_state['values']['archibald_mail_status2draft_body']);
  variable_set('archibald_mail_status2draft_group', $form_state['values']['archibald_mail_status2draft_group']);
  variable_set('archibald_mail_set_responsible_subject', $form_state['values']['archibald_mail_set_responsible_subject']);
  variable_set('archibald_mail_set_responsible_body', $form_state['values']['archibald_mail_set_responsible_body']);

  foreach (archibald_get_available_curriculums() as $curriculum) {
    $classname = 'ArchibaldCurriculum' . drupal_ucfirst($curriculum);

    $curr_handler = new $classname();
    $curr_handler->submitSettingsForm($form_state['values']['curriculum_fieldset'][$curriculum], $form_state);
  }

  drupal_set_message(t('The configuration has been saved.'));

  // to rebuild the menu
  menu_rebuild();
}

/**
 * validate archibald_admin_settings admin settings form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_admin_settings_validate(&$form, &$form_state) {
  $form_values = $form_state['values'];

  foreach (archibald_get_available_curriculums() as $curriculum) {
    $classname = 'ArchibaldCurriculum' . drupal_ucfirst($curriculum);

    $curr_handler = new $classname();
    $curr_handler->validateSettingsForm($form_state['values']['curriculum_fieldset'][$curriculum], $form_state);
  }

  if (!valid_url($form_values['archibald_ontology_url'])) {
    form_set_error('ontolog][archibald_ontology_url', t('Please enter a valid URL.'));
  }
}

/**
 * theme archibald language settings
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_language_settings($variables) {
  $html = '';
  $form = &$variables['form'];

  $table = array(
    'rows' => array(),
  );
  asort($form['#options']);
  $languag_count = count($form['#options']);
  $k2code        = array_keys($form['#options']);
  $cols          = 6;
  $max_rows      = ceil($languag_count / $cols);
  $i             = 0;

  for ($i = 0; $i <= $max_rows; $i++) {

    $col = array();
    for ($x = 0; $x < $cols; $x++) {
      $k = $i + ($x * $max_rows);
      if (!isset($k2code[$k])) {
        $col[] = '';
        continue;
      }
      $k = $k2code[$k];
      $col[] = "<div class='form-item form-type-checkbox " . "form-item-archibald-" . $k . "'>" . theme("checkbox", $form[$k]) . " <label class='option' for='" . $form[$k]['#id'] . "'>" . $form[$k]['#title'] . "</label></div>";
    }
    $table['rows'][] = $col;
  }


  $html = theme('table', $table);

  return $html;
}

/**
 * menu callback
 * show the content partner add / edit form
 *
 * @param array $form
 * @param array $form_state
 * @param integer $content_partner_id
 *
 * @return array
 *   a drupal form
 */
function archibald_content_partner_edit_form($form, $form_state, $content_partner_id = NULL) {
  $content_partner_id = (INT) $content_partner_id;

  $content_partner = archibald_load_content_partner($content_partner_id);

  $form['account'] = array(
    '#type' => 'fieldset',
    '#title' => t('Create an account to publish resources into the national catalogue'),
    '#collapsible' => FALSE,
  );


  $form['account']['content_partner_id'] = array(
    '#type' => 'hidden',
    '#value' => $content_partner['content_partner_id'],
  );

  $form['account']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('Choose an arbitrary name for the account.'),
    '#default_value' => $content_partner['name'],
    '#size' => 50,
    '#maxlength' => 128,
    '#weight' => -15,
    '#required' => TRUE,
  );



  $is_content_partner_editable = TRUE;
  if ( !empty($content_partner['usage_count']) ) {
    $is_content_partner_editable = FALSE;

    $form['account']['user_info'] = array(
      '#markup' => t('Your are not allowed to change the username as long as descriptions
        are published under this account!'),
    );


  }

  $form['account']['username'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Enter your username which was provided by educa.ch'),
    '#default_value' => $content_partner['username'],
    '#size' => 50,
    '#maxlength' => 128,
    '#weight' => -10,
    '#disabled' => !$is_content_partner_editable
  );

  $form['account']['key'] = array(
    '#type' => 'managed_file',
    '#title' => t('Private key file'),
    '#description' => t('Upload the private key file provided by educa.ch. This key enables Archibald to publish descriptions to the national catalogue.'),
    '#upload_validators' => array(
    'file_validate_extensions' => array(
      'pem key private',
      ),
    ),
    '#default_value' => $content_partner['key'],
    '#upload_location' => 'public://files/lom_client_private_key',
  );

  if (variable_get('file_private_path', FALSE)) {
    $form['account']['key']['#upload_location'] = 'private://files/lom_client_private_key';
  }

  $form['account']['password'] = array(
    '#type' => 'textfield',
    '#title' => t('Private key file Password'),
    '#description' => t('Enter the password for the private key file which was provided by educa.ch'),
    '#default_value' => '',
    // we dont like to show the password  $content_partner['password'],
    '#size' => 50,
    '#maxlength' => 120,
  );

  if (!empty($content_partner['content_partner_id'])) {
    $form['account']['password']['#description'] .= '<br />' .
      t('Dont mind an empty field, you only need to enter the password to change it.');
  }




  $form['logo'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content Partner Logo'),
    '#collapsible' => FALSE,
  );


  $form['logo']['icon'] = array(
    '#type' => 'managed_file',
    //'#title' => t('Company Logo'),
    '#description' => t('Upload the logo of your institution. It will be uploaded with every published description. After changing the logo you need to republish all your descriptions!'),
    '#upload_validators' => array(
      'file_validate_extensions' => array('png jpg jpeg gif'),
    ),
    '#default_value' => $content_partner['icon'],
    '#upload_location' => 'public://files/company_logo',
    '#field_suffix' => '<div class="archibald_preview_image" style_name="archibald_contributer">&nbsp;</div>',
  );

  $form['file_upload'] = array(
    '#type' => 'fieldset',
    '#title' => t('File upload'),
    '#collapsible' => TRUE,
  );


  $file_upload_options = array('no' => t('Not permmited'));

  if (!empty($form_state['values']['file_server_type'])) {
    $content_partner['file_server_type'] = $form_state['values']['file_server_type'];
  }

  $dir = new ArchibaldDir('file_uploader', FALSE, dirname(__FILE__));
  $dir->justFiles();
  $dir->fileRegexp('.class.php');

  $selected_plugin = NULL;
  foreach ($dir as $entry) {
    require_once $entry->path;
    if (!preg_match("/^uploader\.(.*)\.class\.php/", $entry->filename, $matches)) {
      continue;
    }

    $classname = 'ArchibaldUploader' . drupal_ucfirst($matches[1]);
    $class = new $classname();
    $file_upload_options = array_merge($file_upload_options, $class->getUploaderName());
    if ($content_partner['file_server_type'] == $matches[1]) {
      $selected_plugin = $class;
    }
  }

  $form['file_upload']['file_server_type'] = array(
    '#type' => 'select',
    '#title' => t('Upload method for digital resources'),
    '#options' => $file_upload_options,
    '#default_value' => $content_partner['file_server_type'],
    '#description' =>
      t('Choose the upload method for digital resources') .
      '<br />' .
      t('IMPORTANT: In case you choose - FTP server - you must configure the settings of the FTP server!'),
    '#ajax' => array(
      // #ajax has two required keys: callback and wrapper.
      // 'callback' is a function that will be called when this element changes.
      'callback' => 'archibald_content_partner_change_plugin_callback',
      // 'wrapper' is the HTML id of the page element that will be replaced.
      'wrapper' => 'replace_plugin_config_div',
      // There are also several optional keys - see ajax_example_autocheckboxes
      // below for details on 'method', 'effect' and 'speed' and
      // ajax_example_dependent_dropdown for 'event'.
    ),
  );

  if (!empty($selected_plugin) || $content_partner['file_server_type'] == 'no') {
    $form['file_upload']['file_uploader_config'] = array(
      '#type' => 'fieldset',
      '#title' => t("Server configuration"),
      '#tree' => TRUE,
      '#prefix' => '<div id="replace_plugin_config_div">',
      '#suffix' => '</div>',
    );
    if ($content_partner['file_server_type'] != 'no') {
      $form['file_upload']['file_uploader_config'] = array_merge(
        $form['file_upload']['file_uploader_config'],
        $selected_plugin->getSettingsForm($form, $form_state, $content_partner_id)
      );
    }
  }

  // Button for the corresponding actions
  $form['actions'] = array(
    '#type' => 'actions',
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  $form['actions']['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/services/archibald/settings',
  );

  return $form;
}

/**
 * Callback.
 *
 * On an ajax submit, the form builder function is called again, then the $form
 * and $form_state are passed to this callback function so it can select which
 * portion of the form to send on to the client.
 *
 * @return renderable array (the textfield element)
 */
function archibald_content_partner_change_plugin_callback($form, $form_state) {
  // The form has already been submitted and updated. We can return the replaced
  // item as it is.
  if (empty($form['file_upload']['file_uploader_config'])) {
    return array("#markup" => '');
  }
  return array(
    "#markup" => '<div id="replace_plugin_config_div">' . drupal_render($form['file_upload']['file_uploader_config']) . '</div>',
  );
  return $form['file_upload']['file_uploader_config'];
}

/**
 * validate archibald_content_partner_edit_form admin settings form for
 * content partner
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_partner_edit_form_validate(&$form, &$form_state) {
  //validate private file

  if (empty($form_state['values']['password']) && !empty($form_state['values']['content_partner_id'])) {
    $form_state['values']['content_partner_id'] = (INT) $form_state['values']['content_partner_id'];
    $content_partner = archibald_load_content_partner($form_state['values']['content_partner_id']);
    $form_state['values']['password'] = $content_partner['password'];
  }

  if (!empty($form_state['values']['key'])) {
    if (empty($form_state['values']['password'])) {
      form_set_error('password', t('Private key password cannot be empty if key file is provided'));
    }
    else {
      $file = file_load($form_state['values']['key']);
      $client_keyfile = file_get_contents($file->uri);
      if (openssl_pkey_get_private($client_keyfile, $form_state['values']['password']) === FALSE) {
        form_set_error('password', t('Private key password is wrong'));
      }
    }
  }

  if (!empty($form_state['values']['file_server_type']) && !empty($form_state['values']['file_uploader_config']) && $form_state['values']['file_server_type'] != 'no') {
    require_once(dirname(__FILE__) . '/file_uploader/uploader.' . $form_state['values']['file_server_type'] . '.class.php');

    $classname = 'ArchibaldUploader' . drupal_ucfirst($form_state['values']['file_server_type']);
    $class = new $classname();
    $class->validateSettingsForm($form['file_upload']['file_uploader_config'], $form_state['values']['file_uploader_config']);
  }
}

/**
 * save archibald_content_partner_edit_form admin settings form for
 * content partner
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_partner_edit_form_submit(&$form, &$form_state) {

  $form_state['values']['content_partner_id'] = (INT) $form_state['values']['content_partner_id'];
  $form_state['redirect'] = 'admin/config/services/archibald/settings';


  //Save company logo
  if (!empty($form_state['values']['icon'])) {
    $file         = file_load($form_state['values']['icon']);
    $file->status = FILE_STATUS_PERMANENT;
    $file         = file_save($file);
    file_usage_add($file, 'archibald', 'company_logo', 1);

    // In case company logo is updated, all published resource must be republished.
    $content_partner = archibald_load_content_partner($form_state['values']['content_partner_id']);
    $old_icon_fid = $content_partner['icon'];
    $content_partner_usage_count = $content_partner['usage_count'];

    if ( $form_state['values']['icon'] != $old_icon_fid && $content_partner_usage_count > 0 ) {
      $form_state['redirect'] = 'admin/archibald/content_partner/' . $form_state['values']['content_partner_id'] . '/republish/central';
    }
    elseif ( $form_state['values']['icon'] != $old_icon_fid) {
      $form_state['redirect'] = 'admin/archibald/content_partner/' . $form_state['values']['content_partner_id'] . '/republish/local';
    }
  }

  //Save private file
  if (!empty($form_state['values']['key'])) {
    $file         = file_load($form_state['values']['key']);
    $file->status = FILE_STATUS_PERMANENT;
    $file         = file_save($file);
    file_usage_add($file, 'archibald', 'lom_client_private_key', 1);
  }

  $content_partner = array(
    'content_partner_id' => $form_state['values']['content_partner_id'],
    'name' => $form_state['values']['name'],
    'icon_fid' => (INT) $form_state['values']['icon'],
    'file_server_type' => $form_state['values']['file_server_type'],
  );

  if (isset($form_state['values']['username'])) {
    $content_partner['username'] = $form_state['values']['username'];
  }

  if (isset($form_state['values']['key'])) {
    $content_partner['key_fid'] = $form_state['values']['key'];
  }

  if (!empty($form_state['values']['password'])) {
    $content_partner['password'] = $form_state['values']['password'];
  }

  if (empty($content_partner['content_partner_id'])) {
    unset($content_partner['content_partner_id']);
    $partner_id = db_insert('archibald_settings_content_partner')->fields($content_partner)->execute();
  }
  else {
    db_update('archibald_settings_content_partner')->fields($content_partner)->condition('content_partner_id', $content_partner['content_partner_id'])->execute();
    $partner_id = $content_partner['content_partner_id'];
  }

  if (!empty($form_state['values']['file_server_type']) && $form_state['values']['file_server_type'] != 'no') {
    require_once(dirname(__FILE__) . '/file_uploader/uploader.' . $form_state['values']['file_server_type'] . '.class.php');
    $classname = 'ArchibaldUploader' . drupal_ucfirst($form_state['values']['file_server_type']);
    $class = new $classname();
    $class->validateSettingsForm($form['file_upload']['file_uploader_config'], $form_state['values']['file_uploader_config'], $partner_id);
  }

  // reset cron look after big processing error
  variable_set('archibald_upload_error', 0);

  //Reset menu cache
  menu_rebuild();



}

/**
 * menu callback
 * show the content partner delte form
 *
 * @param array $form
 * @param array $form_state
 * @param integer $content_partner_id
 *
 * @return array
 *   a drupal form
 */
function archibald_content_partner_delete_form($form, $form_state, $content_partner_id = NULL) {
  $content_partner = archibald_load_content_partner((INT) $content_partner_id);

  $content_partners = archibald_load_content_partner();


  $form['content_partner_id'] = array(
    '#type' => 'value',
    '#value' => $content_partner['content_partner_id'],
  );

  $form['name'] = array(
    '#markup' => t(
      'Do you really want to delete "@name"?',
      array(
        '@name' => $content_partner['name'],
      )
    ),
  );

  // Button for the corresponding actions
  $form['actions'] = array(
    '#type' => 'actions',
  );

  if (!empty($content_partner['usage_count'])) {
    $form['actions']['delete'] = array(
      '#markup' => t(
        'With this account there are @usage_count descriptions published.',
        array(
          '@usage_count' => $data['usage_count'],
        )
      ),
    );
  }
  elseif (count($content_partners) < 2) {
    $form['actions']['delete'] = array(
      '#markup' => t('The last account, cannot be deleted'),
    );
  }
  else {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
  }

  $form['actions']['cancel'] = array(
    '#type' => 'link',
    '#title' => t('Cancel'),
    '#href' => 'admin/config/services/archibald/settings',
  );

  return $form;
}

/**
 * save archibald_content_partner_delete_form admin settings form for
 * content partner
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_partner_delete_form_submit(&$form, &$form_state) {

  if (!empty($form_state['values']['content_partner_id'])) {
    db_delete('archibald_settings_content_partner')
      ->condition('content_partner_id', $form_state['values']['content_partner_id'])
      ->execute();
  }

  $form_state['redirect'] = 'admin/config/services/archibald/settings';
}

/**
 * This function generates a form which ask user toi confirm republication of all resource of the
 * content partner.
 * Mind that this function uses the function "arg(2)".
 * The third path argument must be the content partner id!
 *
 * @param int $content_partner
 *   the content partner id
 * @param string $type
 *   the publish type (central or local)
 *
 * @return string
 *   the form
 */
function archibald_admin_content_partner_republish($content_partner_id, $type) {
  return drupal_get_form("archibald_admin_content_partner_republish_form", $content_partner_id, $type);
}

/**
 * Menu callback.
 * show the content partner add / edit form
 *
 * @param array $form
 *   the form
 * @param array $form_state
 *   the form state
 * @param int $content_partner_id
 *   the content partner id
 * @param string $type
 *   the type (central or local)
 *
 * @return array
 *   a drupal form
 */
function archibald_admin_content_partner_republish_form($form, $form_state, $content_partner_id, $type) {
  // Check access for the given type.
  switch ($type) {
    case 'local':
    case 'central':
      if (!user_access('publish archibald content ' . $type)) {
        drupal_access_denied();
        die();
      }
      break;
    default:
      drupal_access_denied();
      die();

  }

  $form['republish'] = array(
    '#type' => 'fieldset',
    '#title' => t('Republish all descriptions of a content partner (@type)', array(
      "@type" => t("@publish_type", array("@publish_type" => $type)),
    ))
  );

  $form['republish']['info'] = array(
    '#markup' => t('You changed the company logo of a content partner!
      Each description comes with the logo of the company.
      You have to republish all descriptions to update ') . '<br/><br/>'
  );

  $form['republish']['content_partner_id'] = array(
    '#type'  => 'hidden',
    '#value' => $content_partner_id,
  );

  $form['republish']['type'] = array(
    '#type'  => 'hidden',
    '#value' => $type,
  );

  $form['republish']['yes'] = array(
    '#type'  => 'submit',
    '#value' => t('Republish all descriptions'),
  );

  $form['republish']['no'] = array(
    '#type'  => 'submit',
    '#value' => t('Do not republish'),
  );

  return $form;
}

/**
 * This function republish all resources of a content partner.
 * Mind that the same revision of resource is republished (and not the newest revision)!
 *
 * @param array $form
 * @param array $form_state
 *
 * @todo after updating dsbc_lom_stats table, redirect to republishing page
 */
function archibald_admin_content_partner_republish_form_submit($form, &$form_state) {

  if ($form_state['clicked_button']['#id'] == 'edit-yes') {

    $content_partner_id = $form_state['values']['content_partner_id'];
    $type = $form_state['values']['type'];

    if (!empty($content_partner_id)) {

      $fields = array(
        'indexed_version' => NULL,
      );

      if ($type == 'central') {
        $fields['publication_progress'] = NULL;
        $fields['publication_log'] = NULL;
      }

      db_update('archibald_lom_stats')
        ->fields($fields)
        ->condition('content_partner_id', $content_partner_id)
        ->execute();

      if ($type == 'central') {
        $form_state['redirect'] = 'archibald/publish/status/central';
      }
      else {
        $form_state['redirect'] = 'admin/config/services/archibald/settings';
      }
    }
  }
  else {
    $form_state['redirect'] = 'admin/config/services/archibald/settings';
  }
}

