<?php

/**
 * @file
 * handler for contributer / vCards
 * mange and choose contributer
 */
define('ARCHIBALD_CONRIBUTER_PER_PAGE', 14);

require 'archibald.contributer_edit.inc';

/**
 * contributer/vcard list/search
 *
 * @param string $operation_type
 *   (chooser|standalone)  default:chooser
 * @param string $js
 *   (nojs|ajax)
 *   default:nojs
 * @param string $query
 * @param integer $page
 *   default:0
 *
 * @return string html
 */
function archibald_choose_contributer($operation_type = 'chooser', $js = 'nojs', $query = '', $page = 0) {

  archibald_set_header($operation_type);
  $_SESSION['archibald_contributer-last_query'] = $query;

  // pager
  $page = (INT) $page;
  if ($page >= 1) {
    $_SESSION['archibald_contributer-last_page'] = $page;
    $output = archibald_choose_contributer_search_page($query, $page, $operation_type);

    if ($js == 'ajax') {
      print $output;
      exit;
    }
    else {
      return $output;
    }
  }

  // normal ctools overlay
  $_SESSION['archibald_contributer-last_page'] = 1;
  $output = archibald_choose_contributer_search($query, $operation_type);
  if ($js == 'ajax') {
    $commands = array(ctools_modal_command_display(t('Choose contributor'), $output));

    drupal_add_http_header('Content-Type', 'application/json');
    print ajax_render($commands);
    exit;
  }

  return $output;
}

/**
 * include js and css files and set header vars
 *
 * @param string $operation_type
 *  (chooser|standalone)
 *   default:NULL
 */
function archibald_set_header($operation_type = '') {
  // Include the CTools tools that we need.
  ctools_include('ajax');
  ctools_include('modal');

  // Add CTools' javascript to the page.
  ctools_modal_add_js();

  // include standalone css + js
  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');
  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald_form.css');

  // only set variable if all needen vars are set
  if (!empty($operation_type)) {
    drupal_add_js(
      array(
        'archibald' => array(
          'urls' => array(
            'contributer_list' => url('archibald_contributer/' . $operation_type . '/ajax'),
            'contributer_preview' => url('archibald_contributer/preview'),
          ),
        ),
      ),
      'setting'
    );
  }
}

/**
 * ajax callback to display vCard preview
 *
 * @param string $contributer_id
 *   the contributer id
 */
function archibald_contributer_preview($contributer_id, $return = FALSE) {
  $contributer = new ArchibaldLomContributer($contributer_id);
  $vcard       = $contributer->getVcardObject();
  $result      = theme('archibald_choose_contributer_preview', array('vcard' => $vcard));
  if ($return) {
    return $result;
  }
  echo $result;
  die();
}

/**
 * get the contributer and return it with ajax
 *
 * @param string $contributer_id the contributer id
 *
 * @return array result set
 */
function archibald_contributer_get($contributer_id) {
  $sh = db_select('archibald_contributers ', 'archibaldc')
    ->fields(
      'archibaldc',
      array(
        'lastname',
        'firstname',
        'organisation',
      )
    );

  if (!empty($contributer_id)) {
    $condition = db_or();
    $condition->condition('contributer_id', $contributer_id);
    $sh->condition($condition);
  }

  $db_result = $sh->execute()->fetchAssoc();
  if ($db_result) {
    return drupal_json_output($db_result);
  }

  return drupal_json_output('');
}

/**
 * performace a contributer search
 *
 * @param string $query
 * @param integer $page
 *   default=1
 * @param integer $per_page
 *   default=25
 * @param integer $pages
 *   default=1
 *   called by refference
 *   number of pages
 *
 * @return array
 *   result set
 */
function archibald_contributer_search($query, $page = 1, $per_page = 25, &$pages = 1) {

  $sh = db_select('archibald_contributers ', 'contrers')
    ->fields(
      'contrers',
      array(
        'contributer_id',
        'lastname',
        'firstname',
        'organisation',
        'country',
        'zip',
        'city',
        'address1',
        'address2',
        'email',
        'url',
        'tel',
      )
    );

  $shc = db_select('archibald_contributers ', 'contrers');
  $shc->addExpression('COUNT(contributer_id)', 'count');

  if (!empty($query)) {
    $condition = db_or();
    $words = preg_split('/[\s\.\?\!\-\_]+/', $query);
    foreach ($words as $word) {
      if (!empty($word)) {
        $condition->condition('firstname', '%' . $word . '%', 'LIKE');
        $condition->condition('lastname', '%' . $word . '%', 'LIKE');
        $condition->condition('organisation', '%' . $word . '%', 'LIKE');
        $condition->condition('country', '%' . $word . '%', 'LIKE');
        $condition->condition('city', '%' . $word . '%', 'LIKE');
        $condition->condition('address1', '%' . $word . '%', 'LIKE');
        $condition->condition('email', '%' . $word . '%', 'LIKE');
        $condition->condition('url', '%' . $word . '%', 'LIKE');
      }
    }
    $sh->condition($condition);
    $shc->condition($condition);
  }

  $pages = ceil($shc->execute()->fetchField() / $per_page);
  if ($pages < 1) {
    $pages = 1;
  }

  if ($page > $pages) {
    $page = $pages;
  }

  if ($page < 1) {
    $page = 1;
  }

  $sh->orderBy('firstname', 'ASC')
    ->orderBy('lastname', 'ASC')
    ->orderBy('organisation', 'ASC')
    ->range(($page - 1) * $per_page, $per_page);

  $result = array();
  $db_result = $sh->execute();
  while ($row = $db_result->fetchAssoc()) {
    $result[] = $row;
  }

  return $result;
}

/**
 * search for contributers
 *
 * @param string $query
 *  search string
 * @param string $operation_type
 *  (chooser|standalone)  default:chooser
 * @param string $contributer_id
 *  to select
 *
 * @return string html
 */
function archibald_choose_contributer_search($query = '', $operation_type = 'chooser', $contributer_id = '') {

  $search_result = archibald_contributer_search($query, 1, ARCHIBALD_CONRIBUTER_PER_PAGE, $pages);

  return theme(
    'archibald_choose_contributer_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'query' => (!empty($_POST['query']) ? check_plain($_POST['query']) : ''),
      'contributer_id' => $contributer_id,
      'operation_type' => $operation_type,
    )
  );
}

/**
 * search for contributers ajax request
 *
 * @param array $form
 * @param array $form_state
 *
 * @return string html
 */
function archibald_choose_contributer_search_ajax($form, $form_state) {
  $search_result = archibald_contributer_search($form_state['values']['query'], 1, ARCHIBALD_CONRIBUTER_PER_PAGE, $pages);

  return theme(
    'archibald_choose_contributer_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'query' => $form_state['values']['query'],
      'ajax' => TRUE,
      'operation_type' => $form_state['values']['operation_type'],
    )
  );
}

/**
 * search for contributers ajax request
 *
 * @param string $query
 * @param integer $page
 * @param string $operation_type   (chooser|standalone)  default:chooser
 *
 * @return string html
 */
function archibald_choose_contributer_search_page($query = '', $page = 1, $operation_type = 'chooser') {

  $search_result = archibald_contributer_search($query, $page, ARCHIBALD_CONRIBUTER_PER_PAGE, $pages);

  return theme(
    'archibald_choose_contributer_search',
    array(
      'search_result' => $search_result,
      'pages' => $pages,
      'page' => $page,
      'query' => $query,
      'ajax' => TRUE,
      'operation_type' => $operation_type,
    )
  );
}

/**
 * theme archibald contributer search
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_choose_contributer_search($variables) {
  ctools_include('ajax');

  if (!isset($variables['contributer_id'])) {
    $variables['contributer_id'] = '';
  }

  if (!isset($variables['page'])) {
    $variables['page'] = 1;
  }

  // if we came from the add form and saved this success full,
  // the new item needs to be in list to be choosen
  if (!empty($variables['contributer_id'])) {
    $res_row = db_select('archibald_contributers ', 'contrers')
      ->fields(
        'contrers',
        array(
          'contributer_id',
          'lastname',
          'firstname',
          'organisation',
          'country',
          'zip',
          'city',
          'address1',
          'address2',
          'email',
          'url',
          'tel',
        )
      )
      ->condition('contributer_id', $variables['contributer_id'], '=')
      ->execute()
      ->fetchAssoc();

    $variables['search_result'][] = $res_row;
  }


  $html = '';
  if (empty($variables['ajax'])) {
    $html .= drupal_render(
      drupal_get_form(
        'archibald_choose_contributer_search_form',
        $variables['query'],
        $variables['operation_type']
      )
    );
  }

  $table = array(
    'header' => array(
      t('Person'),
      t('Options'),
    ),
    'rows' => array(),
  );

  // generate table from result set
  foreach ($variables['search_result'] as $res_row) {
    $edit_url = 'archibald_contributer/edit/' . $variables['operation_type'] . '/nojs/' . $res_row['contributer_id'] . '/0/' . (!empty($_POST['self_lom_id']) ? check_plain($_POST['self_lom_id']) : '');
    // "/nojs" will replaced by ctools to "/ajax" if possible

    $row = array(0 => '', 1 => array());

    if ($variables['operation_type'] == 'standalone') {
      $row[0] = ctools_ajax_text_button(
        trim($res_row['firstname'] . ' ' . $res_row['lastname'] . ' ' . $res_row['organisation']), $edit_url, t('Edit contributor')
      );
    }
    else {
      $row[0] = array(
        'data' => trim($res_row['firstname'] . ' ' . $res_row['lastname'] . ' ' . $res_row['organisation']),
        'class' => array(
          'archibald_contributer_list_name archibald_contributer_choose',
        ),
        'contributer_id' => $res_row['contributer_id'],
      );
    }

    if (!empty($_POST['entity']) && $_POST['entity'] == $res_row['contributer_id']) {
      $row[0]['class'][] = 'archibald_contributer_list_current_activ';
    }

    $attributes = array(
      'class' => array(
        'archibald_contributer_choose',
      ),
      'contributer_id' => $res_row['contributer_id'],
    );

    if ($variables['contributer_id'] == $res_row['contributer_id']) {
      $attributes['class'][] = ' archibald_contributer_choose_this';
    }

    if ($variables['operation_type'] == 'chooser') {
      $row[1][] = l(
        t('Choose'), 'archibald_contributer/choose/' . $res_row['contributer_id'], array('attributes' => $attributes));
    }
    $row[1][] = ctools_ajax_text_button(t('Edit'), $edit_url, t('Edit contributor'));
    $row[1] = implode(' | ', $row[1]);

    $table['rows'][] = $row;
  }

  $content = theme('table', $table);

  // Start of Pager
  $items = array();
  for ($p = 1; $p <= $variables['pages']; $p++) {
    if ($p == $variables['page']) {
      $items[] = array(
        'class' => array(
          'pager-current',
        ),
        'data' => $p,
      );
    }
    else {
      $items[] = array(
        'class' => array(
          'pager-item',
        ),
        'data' => l(
          $p,
          'archibald_contributer/' . $variables['operation_type'] . '/nojs/' . urlencode($variables['query']) . '/' . $p,
          array(
            'attributes' => array(
              'class' => array(
                'archibald_contriberter_page',
              ),
            ),
          )
        ),
      );
    }
  }

  $content .= '<h2 class="element-invisible">' . t('Pages') . '</h2>' .
    theme(
      'item_list',
      array(
        'items' => $items,
        'attributes' => array(
          'class' => array(
            'pager',
          ),
        ),
      )
    );

  $html .= '<div id="archibald_choose_contributer_search_result">' . $content . '</div>';

  return $html;
}

/**
 * search formular for archibald contributer search
 *
 * @param array $form
 * @param array $form_state
 * @param string $query
 * @param string $operation_type
 *  (chooser|standalone)  default:chooser
 *
 * @return array
 */
function archibald_choose_contributer_search_form($form, &$form_state, $query, $operation_type = 'chooser') {

  $form['operation_type'] = array(
    '#type' => 'hidden',
    '#value' => $operation_type,
  );

  $self_lom_id = '';
  if (!empty($_POST['self_lom_id'])) {
    $self_lom_id = check_plain($_POST['self_lom_id']);
  }

  $form['self_lom_id'] = array(
    '#type' => 'hidden',
    '#value' => $self_lom_id,
  );

  $current_entity = '';
  if (!empty($_POST['entity'])) {
    $current_entity = check_plain($_POST['entity']);
  }

  $form['entity'] = array(
    '#type' => 'hidden',
    '#value' => $current_entity,
  );


  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Term'),
    '#size' => 40,
    '#default_value' => $query,
    '#attributes' => array(
      'class' => array(
        'archibald_choose_contributer_query',
      ),
    ),
    '#ajax' => array(
      'callback' => 'archibald_choose_contributer_search_ajax',
      'wrapper' => 'archibald_choose_contributer_search_result',
      'method' => 'replace',
      'effect' => 'fade',
      'keypress' => TRUE,
    ),
  );

  $form['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#weight' => 9999,
    '#ajax' => array(
      'callback' => 'archibald_choose_contributer_search_ajax',
      'wrapper' => 'archibald_choose_contributer_search_result',
      'method' => 'replace',
      'effect' => 'fade'
    ),
  );

  $form['#theme'] = 'archibald_choose_contributer_search_form';

  return $form;
}

/**
 * theme the search form
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_choose_contributer_search_form($variables) {
  ctools_include('ajax');

  $form = reset($variables);

  $add_url = 'archibald_contributer/add/' . $form['operation_type']['#value'] . '/nojs';
  // "/nojs" will replaced by ctools to "/ajax" if possible

  if ($form['operation_type']['#value'] == 'standalone') {
    $width = 450;
  }
  else {
    $width = 550;
  }

  unset($form['query']['#title']);
  $html = '';
  if (isset($_POST['entity']) && !empty($_POST['entity'])) {
    $html .= '<input type="button" class="archibald_contributer_reset form-submit" style="float:left" value="' . t('Reset') . '">';
  }
  $html .= '<div style="text-align:right; float:right;">';
  $html .= drupal_render($form['query']) . drupal_render($form['search']);
  $form['query']['#printed'] = TRUE;
  $html .= '</div>';

  $button = ctools_ajax_text_button(t('Add vCard'), $add_url, t('Add vCard'));
  $html .= '<ul class="action-links"><li>' . $button . '</li></ul>';
  $html .= drupal_render_children($form);

  return $html;
}

/**
 * menu callback for choosing a menu item
 *
 * @param string $contributer_id
 *
 * @return string html
 */
function archibald_contributer_choose($contributer_id) {
  // display search result and mark edited items as choosen
  // cause of this by js the modal frame will dismissed
  return archibald_choose_contributer_search($_SESSION['archibald_contributer-last_query'], 'chooser', $contributer_id);
}

/**
 * clean up telefonumber and try to normalize it
 *
 * @param string $org_nr
 *
 * @return string
 */
function archibald_contributer_clean_tel($org_nr) {
  if (empty($org_nr)) {
    return '';
  }

  $nr = preg_replace('~[^0-9]+~i', '', $org_nr);

  if (drupal_substr($org_nr, 0, 1) == '+') {
    return '+' . $nr;
  }

  $nr = preg_replace('~[^0-9]+~i', '', $nr);

  $x1 = drupal_substr($nr, 0, 1);
  $x2 = drupal_substr($nr, 1, 2);

  if ($x1 == '0' && $x2 == '0') {
    return '+' . drupal_substr($nr, 2);
  }
  elseif ($x1 == '0' && $x2 > 0) {
    return '+41' . drupal_substr($nr, 1);
  }
  else {
    return $org_nr;
  }
}

/**
 * theming function for contributer preview ajax caller
 *
 * @param $variables array(vcard=>ArchibaldAppDataVcard)
 *
 * @return string html
 */
function theme_archibald_choose_contributer_preview($variables) {
    #'header' => array(t('Title'), t('Value')),
  $table = array(
    'colgroups' => array(array(array('width' => '40%'), array())),
    'rows' => array(),
  );

  $table['rows'][] = array(t('First name'), $variables['vcard']->firstname);
  $table['rows'][] = array(t('Last name'), $variables['vcard']->lastname);
  $table['rows'][] = array(t('Organisation'), $variables['vcard']->organization);

  $address = $variables['vcard']->address;
  if (!empty($address[0])) {
    $table['rows'][] = array(t('Country'), @$address[0]['country']);
    $table['rows'][] = array(t('ZIP'), @$address[0]['zip']);
    $table['rows'][] = array(t('City'), @$address[0]['city']);
    $table['rows'][] = array(t('Address'), @$address[0]['street']);
    $table['rows'][] = array(t('Additional address'), @$address[0]['extendedaddress']);
  }

  $email = $variables['vcard']->email;
  if (!empty($email[0]['value'])) {
    $table['rows'][] = array(t('E-Mail'), @$email[0]['value']);
  }

  $url = $variables['vcard']->url;
  if (!empty($url[0]['value'])) {
    $table['rows'][] = array(t('URL'), @$url[0]['value']);
  }

  $telephone = $variables['vcard']->telephone;
  if (!empty($telephone[0]['value'])) {
    $table['rows'][] = array(t('Work telephone'), @$telephone[0]['value']);
  }

  /**
   * HH 2012-05-04 vcard icons are not longer supported
   * CHA 2012-06-15 vcard enabled again
   */
  $default_value = NULL;
  if (isset($variables['vcard']->logo->uri)) {
    $default_value = $variables['vcard']->logo->uri;
  }
  preg_match('/archibald_file\/([0-9]+)\//', $default_value, $old_fid);
  if (!empty($old_fid[1])) {
    $file            = file_load($old_fid[1]);
    $style_name      = 'thumbnail';
    $table['rows'][] = array(
      t('Logo'),
      theme('image_style',
        array(
          'style_name' => (empty($style_name)) ? 'thumbnail' : $style_name,
          'path' => $file->uri,
          'getsize' => FALSE,
          'attributes' => array(
            'fid' => $file->fid
          ),
        )
      ),
    );
  }

  return theme('table', $table);
}

