<?php

/**
 * @file
 * provides the cronributer / vcard edit form
 */

/**
 * show and process contributer form
 *
 * @param string $operation_type
 *   (chooser|standalone)
 *   default:chooser
 * @param string $js
 *   (nojs|ajax)
 *   default:nojs
 *
 * @return string html
 */
function archibald_contributer_add($operation_type = 'chooser', $js = 'nojs') {
  return archibald_contributer_edit($operation_type, $js, NULL);
}

/**
 * show and process contributer form
 *
 * @param string $operation_type
 *   (chooser|standalone)
 *   default:chooser
 * @param string $js
 *   (nojs|ajax) default:nojs
 * @param string $contributer_id
 * @param string $step
 *
 * @return string html
 */
function archibald_contributer_edit($operation_type = 'chooser', $js = 'nojs', $contributer_id = NULL, $step = NULL, $self_lom_id = '') {

  archibald_set_header($operation_type);

  if (!empty($_REQUEST['contributer_id'])) {
    $contributer_id = $_REQUEST['contributer_id'];
  }

  if (!empty($contributer_id)) {
    // edit mode
    $contributer = new ArchibaldLomContributer($contributer_id);
    $vcard = $contributer->getVcardObject();
  }
  else {
    // add mode
    $vcard = new ArchibaldAppDataVcard();
  }

  if (empty($contributer_id)) {
    $button_label = t('Add new contributor');
  }
  else {
    $button_label = t('Edit contributor');
  }

  $form_info = array(
    'id' => 'contributer',
    'path' => 'archibald_contributer/edit/' . $operation_type . '/' . ($js ? 'ajax' : 'nojs') . '/' . $contributer_id . '/%step',
    'show trail' => TRUE,
    'show back' => TRUE,
    'show cancel' => TRUE,
    'show return' => FALSE,
    'next callback' => 'archibald_contributer_form_next',
    'finish callback' => 'archibald_contributer_form_finish',
    'cancel callback' => 'archibald_contributer_form_cancel',
    // this controls order, as well as form labels
    'order' => array(
      'edit' => $button_label,
    ),
    // here we map a step to a form id.
    'forms' => array(
      // e.g. this for the step at wombat/create
      'edit' => array(
        'form id' => 'archibald_contributer_edit_form',
      ),
    ),
  );

  // only will editing we have to look up for changed lom resourced.
  // Not will adding
  if (!empty($contributer_id)) {
    $form_info['order']['update_resource'] = t('Nationally published descriptions using this vCard must be republished after editing.');
    $form_info['forms']['update_resource'] = array(
      'form id' => 'archibald_contributer_update_resource_form',
    );
  }

  // this means you only can have one add form at same time
  $object_id = (empty($contributer_id)) ? 'add' : $contributer_id;

  if (empty($step)) {
    // We reset the form when $step is NULL because that means they have
    // for whatever reason started over.
    archibald_contributer_form_cache_clear($object_id);
    $step = 'edit';
  }

  // This automatically gets defaults if there wasn't anything saved.
  $object = archibald_contributer_form_cache_get($object_id, $contributer_id, $self_lom_id);

  $form_state = array(
    'ajax' => $js,
    // Put our object and ID into the form state cache so we can easily find
    // it.
    'object_id' => $object_id,
    'object' => & $object,
    'contributer_id' => $contributer_id,
    'vcard' => $vcard,
    'operation_type' => $operation_type,
  );

  // Send this all off to our form. This is like drupal_get_form only wizardy.
  ctools_include('wizard');
  $form = ctools_wizard_multistep_form($form_info, $step, $form_state);
  $output = drupal_render($form);

  // If $output is FALSE, there was no actual form.
  if ($js == 'ajax') {
    // If javascript is active, we have to use a render array.
    $commands = array();
    if ($output === FALSE || !empty($form_state['complete'])) {
      // Dismiss the modal.
      if ($operation_type == 'standalone') {
        if (!empty($form_state['redirect'])) {
          $commands = array(ctools_ajax_command_redirect($form_state['redirect']));
        }
        else {
          $commands = array(
            ctools_modal_command_dismiss(),
            ajax_command_replace(
              '#archibald_choose_contributer_search_result',
              archibald_choose_contributer_search_page(
                @$_SESSION['archibald_contributer-last_query'], @$_SESSION['archibald_contributer-last_page'], $operation_type
              )
            ),
          );
        }
        drupal_add_js(
          array(
            'archibald' => array(
              'new_contributer_id' => $form_state['object']->new_contributer_id,
            ),
          ),
          'setting'
        );

      }
      else {
        // display search result and mark edited items as choosen
        // cause of this by js the modal frame will dismissed
        $commands = array(
          ctools_modal_command_display(
            t('Choose contributor'),
            archibald_choose_contributer_search(
              @$_SESSION['archibald_contributer-last_query'], $operation_type, $form_state['object']->new_contributer_id
            )
          ),
        );
      }
      drupal_add_js(
        array(
          'archibald' => array(
            'new_contributer_id' => $form_state['object']->new_contributer_id,
          ),
        ),
        'setting'
      );

    }
    elseif (!empty($form_state['cancel'])) {
      // If cancelling, return to the activity.
      $commands[] = ctools_modal_command_dismiss();
    }
    else {
      $commands = ctools_modal_form_render($form_state, $output);
    }

    $commands[] = array(
      'command' => 'archibald_contributer_preview_hide',
    );

    print ajax_render($commands);
    exit;
  }
  else {
    if ($output === FALSE || !empty($form_state['complete'])) {
      return $animal;
    }
    elseif (!empty($form_state['cancel'])) {
      if ($operation_type == 'standalone') {
        $form_state['redirect'] = 'archibald_contributer/standalone';
      }
      else {
        $form_state['redirect'] = 'archibald_contributer/choose/' . $contributer_id . '/' . @$object->self_lom_id;
      }
    }
    else {
      return $output;
    }
  }
}

// ---------------------------------------------------------------------------
// Wizard caching helpers.

/**
 * Store our little cache so that we can retain data from form to form.
 */
function archibald_contributer_form_cache_set($id, $object) {
  ctools_include('object-cache');
  ctools_object_cache_set('archibald_contributer_form', $id, $object);
}

/**
 * Get the current object from the cache, or default.
 */
function archibald_contributer_form_cache_get($id, $contributer_id = '', $self_lom_id = '') {

  ctools_include('object-cache');
  $object = ctools_object_cache_get('archibald_contributer_form', $id);
  if (!$object) {
    // Create a default object.
    $object = new stdClass;
    $object->start_contributer_id = $contributer_id;
    if (!empty($self_lom_id)) {
      $object->self_lom_id = $self_lom_id;
    }
  }

  return $object;
}

/**
 * Clear the wizard cache.
 */
function archibald_contributer_form_cache_clear($id) {
  ctools_include('object-cache');
  ctools_object_cache_clear('archibald_contributer_form', $id);
}

// ---------------------------------------------------------------------------
// Wizard in-between helpers; what to do between or after forms.

/**
 * Handle the 'next' click on the add/edit pane form wizard.
 *
 * All we need to do is store the updated pane in the cache.
 */
function archibald_contributer_form_next(&$form_state) {
  archibald_contributer_form_cache_set($form_state['object_id'], $form_state['object']);
}

/**
 * Handle the 'finish' click on teh add/edit pane form wizard.
 *
 * All we need to do is set a flag so the return can handle adding
 * the pane.
 */
function archibald_contributer_form_finish(&$form_state) {
  $form_state['complete'] = TRUE;
}

/**
 * Handle the 'cancel' click on the add/edit pane form wizard.
 */
function archibald_contributer_form_cancel(&$form_state) {
  $form_state['cancel'] = TRUE;
}

// ---------------------------------------------------------------------------
// Wizard forms for our simple info collection wizard.

/**
 * Wizard start form. Edit the contributer
 */
function archibald_contributer_edit_form($form, &$form_state) {
  if (empty($form_state['contributer_id'])) {
    $form_state['title'] = t('Create new contributor');
  }
  else {
    $form_state['title'] = t('Edit contributor');
  }

  $form['#attributes'] = array(
    'class' => array(
      'archibald-choose-contributer-add-form-' . $form_state['operation_type'],
    ),
  );

  $form['contributer_id'] = array(
    '#type' => 'hidden',
    '#value' => $form_state['contributer_id'],
  );

  $form['operation_type'] = array(
    '#type' => 'hidden',
    '#value' => $form_state['operation_type'],
    '#attributes' => array(
      'class' => array(
        'archibald_choose_contributer_form_operation_type',
      ),
    ),
  );

  $form['firstname'] = array(
    '#type' => 'textfield',
    '#title' => t('First name'),
    '#required' => FALSE,
    '#maxlength' => 180,
    '#default_value' => $form_state['vcard']->firstname,
  );

  $form['lastname'] = array(
    '#type' => 'textfield',
    '#title' => t('Last name'),
    '#required' => FALSE,
    '#maxlength' => 180,
    '#description' => t('Please enter name of person or organisation.'),
    '#default_value' => $form_state['vcard']->lastname,
  );

  $form['organisation'] = array(
    '#type' => 'textfield',
    '#title' => t('Organisation'),
    '#maxlength' => 255,
    '#default_value' => $form_state['vcard']->organization,
  );

  $address = $form_state['vcard']->address;
  $form['country'] = array(
    '#type' => 'textfield',
    '#title' => t('Country'),
    '#maxlength' => 60,
    '#default_value' => @$address[0]['country'],
  );

  $form['zip'] = array(
    '#type' => 'textfield',
    '#title' => t('ZIP'),
    '#maxlength' => 20,
    '#default_value' => @$address[0]['zip'],
  );

  $form['city'] = array(
    '#type' => 'textfield',
    '#title' => t('City'),
    '#maxlength' => 180,
    '#default_value' => @$address[0]['city'],
  );

  $form['address1'] = array(
    '#type' => 'textfield',
    '#title' => t('Address'),
    '#maxlength' => 255,
    '#default_value' => @$address[0]['street'],
  );

  $form['address2'] = array(
    '#type' => 'textfield',
    '#title' => t('Additional address'),
    '#maxlength' => 255,
    '#default_value' => @$address[0]['extendedaddress'],
  );

  $email = $form_state['vcard']->email;
  $form['email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-Mail'),
    '#required' => FALSE,
    '#maxlength' => 255,
    '#default_value' => @$email[0]['value'],
  );

  $url = $form_state['vcard']->url;
  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#required' => FALSE,
    '#maxlength' => 255,
    '#default_value' => @$url[0]['value'],
  );

  $telephone = $form_state['vcard']->telephone;
  $form['tel'] = array(
    '#type' => 'textfield',
    '#title' => t('Work telephone'),
    '#required' => FALSE,
    '#description' => t('Please enter phone number like following example +41313005500.'),
    '#maxlength' => 32,
    '#default_value' => @$telephone[0]['value'],
  );

  /**
   * HH 2012-05-04 vcard icons are not longer supported
   * CHA 2012-06-15 vcard enabled again
   */
  $default_value = NULL;
  if (isset($form_state['vcard']->logo->uri)) {
    $default_value = $form_state['vcard']->logo->uri;
  }


  preg_match('/archibald_file\/([0-9]+)\//', $default_value, $old_fid);
  $default_value = isset($old_fid[1]) ? $old_fid[1] : NULL;
  $form['logo'] = array(
    '#type' => 'managed_file',
    '#title' => t('Logo'),
    '#default_value' => $default_value,
    '#upload_validators' => array(
      'file_validate_extensions' => array(
        'png jpg jpeg gif'
      ),
    ),
    '#upload_location' => 'public://files/preview_image',
    '#field_suffix' => '<div class="archibald_preview_image" style_name="thumbnail">&nbsp;</div>',
  );

  return $form;
}

/**
 * search formular for archibald contributer add validator
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function archibald_contributer_edit_form_validate($form, &$form_state) {
  archibald_set_header();

  if (empty($form_state['values']['organisation'])) {
    if (empty($form_state['values']['firstname'])) {
      form_set_error('firstname', t('Enter first name'));
    }

    if (empty($form_state['values']['lastname'])) {
      form_set_error('lastname', t('Enter last name'));
    }
  }

  if (!empty($form_state['values']['email'])) {
    if (!valid_email_address($form_state['values']['email'])) {
      form_set_error('email', t('Please enter a valid E-Mail address'));
    }
  }

  if (!empty($form_state['values']['url'])) {
    if (!valid_url($form_state['values']['url'], TRUE)) {
      form_set_error('url', t('Please enter a valid URL'));
    }
  }
}

/**
 * search formular for archibald contributer add save handler
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function archibald_contributer_edit_form_submit($form, &$form_state) {
  $form_state['values']['tel'] = archibald_contributer_clean_tel($form_state['values']['tel']);

  ##### handle the logo
  if (!empty($form_state['values']['logo'])) {
    // The new file's status is set to 0 or temporary and in order to ensure
    // that the file is not removed after 6 hours we need to change it's status
    // to 1. Save the ID of the uploaded image for later use.
    $file = file_load($form_state['values']['logo']);
    $file->status = FILE_STATUS_PERMANENT;
    file_save($file);

    // When a module is managing a file, it must manage the usage count.
    // Here we increment the usage count with file_usage_add().
    file_usage_add($file, 'archibald', 'contributer_logo', 1);

    $form_state['values']['logo'] = archibald_file_get_url($file);
  }

  $lom_contributer = new ArchibaldLomContributer($form_state['values']['contributer_id']);
  $lom_contributer->renderDataObject($form_state['values']);

  $contributer_id = $lom_contributer->save();

  $form_state['object']->new_contributer_id = $contributer_id;

  $form_state['object']->linked_resources = archibald_contributer_get_linked_resources($contributer_id);

  // remove to lom reousce with we currently edit from the list
  if (!empty($form_state['object']->self_lom_id) && !empty($form_state['object']->linked_resources[$form_state['object']->self_lom_id])) {
    unset($form_state['object']->linked_resources[$form_state['object']->self_lom_id]);
  }

  // if there are no resources to update sip this next step and end action
  if (empty($form_state['object']->linked_resources)) {
    $form_state['complete'] = TRUE;
  }
}

/**
 * Display form to let user choose witch resoruce er like to update
 * Cause updated contriubter vCard was used in given lsit of resources
 */
function archibald_contributer_update_resource_form($form, &$form_state) {
  $form_state['title'] = t('Update descriptions linked to this contributor.');

  $options = array();
  foreach ($form_state['object']->linked_resources as $lom_id => $tmp) {
    $options[$lom_id] = ArchibaldLom::determinTitle($lom_id);
  }

  $form['to_update'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Update descriptions'),
    '#default_value' => array_keys($form_state['object']->linked_resources),
    '#options' => $options,
    '#description' => t('Following nationally published descriptions using the updated vCard. In case a description should not be updated, uncheck it.'),
  );



  return $form;
}

/**
 * process users choise
 * update all marked resources
 */
function archibald_contributer_update_resource_form_submit(&$form, &$form_state) {

  $redirect = FALSE;
  foreach ($form_state['values']['to_update'] as $lom_id => $decision) {
    if (!empty($decision)) {
      foreach ($form_state['object']->linked_resources[$lom_id] as $location => $NULL) {
        // @TODO perhabs we should do this as batch job
        // http://www.hashbangcode.com/forum/playing-drupal-queues-538.html
        archibald_contributer_update_resource(
          $lom_id,
          $location,
          $form_state['object']->start_contributer_id,
          $form_state['object']->new_contributer_id
        );
        archibald_republish_resource($lom_id);
        $redirect = TRUE;
      }
    }
  }
  if ($redirect == TRUE) {
    $form_state['redirect'] = 'archibald/publish/status/central';
  }
  $form_state['complete'] = TRUE;
}

/**
 * get list of lom_id by contributer_id
 * where actual version lom resource contains a
 * vcard mathing with contributer_id
 *
 * @param string $contributer_id
 *
 * @return array
 */
function archibald_contributer_get_linked_resources($contributer_id) {
  $version_subquery = '
    SELECT la.version
    FROM archibald_lom la
    WHERE lom.lom_id=la.lom_id
    ORDER BY la.save_time DESC
    LIMIT 1';

  $result = array();

  // cat lom_id`s of matching lom resources for lifecycle
  $sql = 'SELECT lom.lom_id
    FROM {archibald_contribute_entity} as cont_entity
    INNER JOIN {archibald_contribute} cont ON (cont_entity.contribute_id = cont.contribute_id)
    INNER JOIN {archibald_lifecycle_contributes} lc_cont ON (cont.contribute_id = lc_cont.contribute_id)
    INNER JOIN {archibald_lom} lom ON (lc_cont.lifecycle_id = lom.lifecycle_id)
    WHERE cont_entity.entity_id=:contributer_id
    AND cont.type=:type
    AND lom.version = (' . $version_subquery . ') ';

  $q_res = db_query($sql, array(
      ':contributer_id' => $contributer_id,
      ':type' => 'lifecycle',
    ));

  while ($row = $q_res->fetchObject()) {
    $result[$row->lom_id]['lifecycle'] = TRUE;
  }

  // cat lom_id`s of matching lom resources for meta meta data
  $sql = 'SELECT lom.lom_id
    FROM {archibald_contribute_entity} as cont_entity
    INNER JOIN {archibald_contribute} cont ON (cont_entity.contribute_id = cont.contribute_id)
    INNER JOIN {archibald_meta_metadata_contributes} mm_cont ON (cont.contribute_id = mm_cont.contribute_id)
    INNER JOIN {archibald_lom} lom ON (mm_cont.meta_metadata_id = lom.meta_metadata_id)
    WHERE cont_entity.entity_id=:contributer_id
    AND cont.type=:type
    AND lom.version = (' . $version_subquery . ') ';

  $q_res = db_query($sql, array(
      ':contributer_id' => $contributer_id,
      ':type' => 'meta_metadata',
    ));

  while ($row = $q_res->fetchObject()) {
    $result[$row->lom_id]['meta_metadata'] = TRUE;
  }

  return $result;
}

/**
 * update contributer in resource.
 * replace old_contributer to new_contributer
 *
 * @param string $lom_id
 * @param string $location
 * @param string $old_contributer_id
 * @param string $new_contributer_id
 */
function archibald_contributer_update_resource($lom_id, $location, $old_contributer_id, $new_contributer_id) {

  $lom = archibald_load($lom_id);

  switch ($location) {
    case 'lifecycle':
      $contributes = $lom->getLifeCycle()->getContribute();
      break;

    case 'meta_metadata':
      $contributes = $lom->getMetaMetadata()->getContribute();
      break;
  }

  // Wall trough all contributs
  // and find matching id and replace contribut to new one.
  foreach ($contributes as $cid => $contribute) {
    $lom_contributer = new ArchibaldLomContributer();
    $lom_contributer->load($contribute->getEntity(0));

    $contribute_id = $lom_contributer->getId();

    if ($contribute_id == $old_contributer_id) {

      $lom_contributer = new ArchibaldLomContributer($new_contributer_id);
      $vcard = $lom_contributer->getVcard();

      if (empty($vcard)) {
        continue;
      }

      $contributes[$cid]->setEntity($vcard, 0);
    }
  }

  switch ($location) {
    case 'lifecycle':
      $life_cycle = $lom->getLifeCycle();
      $life_cycle->setContribute($contributes);
      $lom->setLifeCycle($life_cycle);
      break;

    case 'meta_metadata':
      $meta_metadata = $lom->getMetaMetadata();
      $meta_metadata->setContribute($contributes);
      $lom->setMetaMetadata($meta_metadata);
      break;
  }

  archibald_save($lom);
}

