<?php

/**
 * @file
 * controller for the frontend search
 */

/**
 * menu callback for archibald content overview
 *
 * @return string html
 */
function archibald_content_overview() {
  global $language;

  drupal_add_js(
    drupal_get_path('module', 'archibald') . '/js/archibald.js'
  );

  drupal_add_css(
    drupal_get_path('module', 'archibald') . '/css/archibald.css'
  );

  // for link search like vocabular items
  if (empty($_POST) && isset($_GET['query'])) {
    $_SESSION['archibald_search_query'] = $_GET['query'];
  }

  $num_per_page = variable_get('archibald_content_num_per_page', 25);

  $query = '';
  if (isset($_POST['query'])) {
    $_SESSION['archibald_search_query'] = filter_xss($_POST['query']);
  }
  elseif (isset($_GET['query'])) {
    $_SESSION['archibald_search_query'] = filter_xss($_GET['query']);
  }

  if (!empty($_SESSION['archibald_search_query'])) {
    $query = $_SESSION['archibald_search_query'];
  }

  if (variable_get('archibald_solr_search_activ', 0) == 1) {
    $search = new ArchibaldLomSolrSearch();
    $search->init(FALSE);
    drupal_add_js(
      drupal_get_path('module', 'archibald') .
      '/js/client_overview_facets.js'
    );

    $search->setLimit($num_per_page);
    $search->setStart(pager_find_page() * $num_per_page);

    $search->enableFacets(variable_get('archibald_content_collections', 0));

    if (!empty($query)) {
      if (variable_get('archibald_content_collections', 0) == 1) {
        $in_query_filters = array(
          'learning_resourcetype' => 'sm_child_learning_resourcetype',
          'intendet_enduserrole' => 'sm_child_intendet_enduserrole',
          'difficulty' => 'sm_child_difficulty',
          'education_context' => 'tm_child_context',
          'keywords' => 'tm_child_keywords',
          'coverage' => 'tm_child_coverage',
          'author' => 'sm_child_author',
        );
      }
      else {
        $in_query_filters = array(
          'learning_resourcetype' => 'sm_learning_resourcetype',
          'intendet_enduserrole' => 'sm_intendet_enduserrole',
          'difficulty' => 'ss_difficulty',
          'education_context' => 'sm_context',
          'keywords' => 'tm_keywords',
          'coverage' => 'tm_coverage',
          'author' => 'sm_author',
        );
      }
      $query = strtr($query, $in_query_filters);

      $search->setQuery($query);
    }

    if (variable_get('archibald_search_in_current_language', 1) == TRUE) {
      $search->setFilter('ss_language', $language->language);
    }

    // field to look for search keywords with boost options
    $qf_values = array(
      'base' => array(
        'label',
        'content',
        'tm_keywords',
        'tm_coverage',
        'ts_otherPlattformRequirements',
        'ts_education_description',
        'tm_contributer',
        'tm_vocabular_text',
      ),
      'collection' => array(
        'tm_child_label',
        'tm_child_content',
        'tm_child_keywords',
        'tm_child_coverage',
        'tm_child_otherPlattformRequirements',
        'tm_child_education_description',
        'tm_child_contributer',
        'tm_child_vocabular_text',
      ),
    );

    $qf_base = '';
    foreach ($qf_values['base'] as $key) {
      $value = archibald_get_solr_bias_value($key);
      if (!empty($value)) {
        $qf_base .= $key . '^' . $value . ' ';
      }
    }
    $qf_collection = $qf_base;
    foreach ($qf_values['collection'] as $key) {
      $value = archibald_get_solr_bias_value($key);
      if (!empty($value)) {
        $qf_collection .= $key . '^' . $value . ' ';
      }
    }
    $qf_base = trim($qf_base);
    $qf_collection = trim($qf_collection);

    if (variable_get('archibald_content_collections', 0) == 1) {
      $search->setQueryFields($qf_collection);

      // dont show hts_collection:1 cause this are child nodes,
      // we only like to find the collections or stand alones
      // the child we dertermine with the $qf_collection
      $search->setFilter('hts_collection', array(0, 2));
    }
    else {
      $search->setQueryFields($qf_base);
    }

    // get facet checkboxes and process them,
    // this we needs to do bevor ->search() and ->get_filter_options()
    foreach ($_POST as $key => $val) {
      if (drupal_substr($key, 0, 6) == 'facet_') {
        $search->setFacetFilter(
          drupal_substr($key, 6),
          array_keys($val)
        );
      }
    }

    $search_result = $search->search();
    pager_default_initialize($search->getTotal(), $num_per_page);

    $search_form = drupal_get_form(
      'archibald_content_overview_form',
      $search->getFacetOptions()
    );

    $html = theme(
      'archibald_content_overview',
      array(
        'search_result' => $search_result,
        'search_form' => $search_form,
        'search_result_count' => $search->getTotal(),
      )
    );
  }
  else {
    // use the default sql search

    if (variable_get('archibald_search_in_current_language', 1) == TRUE) {
      $search_in_lang = $language->language;
    }
    else {
      $search_in_lang = 'any';
    }

    $search = new ArchibaldLomSQLSearch();
    $search->setLimit($num_per_page);
    $search->setQuery(
      $query,
      $search_in_lang
    );
    $search->setLocalPublishedFilter(TRUE);
    $search->getLocalPublishedVersion();

    $total_record_count = $search->getTotalRecordCount();

    $page = pager_default_initialize($total_record_count, $num_per_page);
    $search->setOffset($page * $num_per_page);

    $search_form = drupal_get_form('archibald_content_overview_form');

    $html = theme(
      'archibald_content_overview',
      array(
        'search_result' => $search->search(),
        'search_form' => $search_form,
        'search_result_count' => $total_record_count,
      )
    );
  }

  if (!empty($_POST['ajax'])) {
    echo $html;
    die();
  }

  return $html;
}

/**
 * search formular for archibald resources
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function archibald_content_overview_form($form, &$form_state, $filter_options = array()) {
  $query = '';
  if (!empty($_SESSION['archibald_search_query'])) {
    $query = $_SESSION['archibald_search_query'];
  }

  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Term'),
    '#default_value' => $query,
    '#size' => 40,
  );

  if (!empty($filter_options)) {
    $weight = 5;
    foreach ($filter_options as $filter_option_key => $filter_option) {
      $key = 'facet_' . $filter_option_key;
      $form[$key] = array(
        '#type' => 'fieldset',
        '#title' => t('Filter by') . ' ' .
          trim(
            strtr(
              $filter_option_key,
              array(
                'educa' => '',
                '_' => ' ',
              )
            )
          ),
        '#weight' => $weight,
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#tree' => TRUE,
      );
      archibald_content_overview_form_facets2form_recursiv(
        $form[$key],
        $filter_option,
        $key
      );
      $weight++;
    }
  }

  $form['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#weight' => 9999,
  );
  $form['#theme'] = 'archibald_content_overview_form';

  return $form;
}

/**
 * process facets recursiv and display them
 *
 * @param array $form
 * @param array $items
 * @param string $parent_key
 *   to get back an well formed post array
 */
function archibald_content_overview_form_facets2form_recursiv(&$form, $items, $parent_key) {

  foreach ($items as $item) {
    $form[$item->key] = array(
      '#type' => 'checkbox',
      '#title' => $item->name,
      '#default_value' => $item->activ,
      '#result_count' => $item->count,
      '#attributes' => array(
        'key' => $item->key,
        'class' => array('facet_id_' . $item->key),
      ),
      '#tree' => TRUE,
      '#parents' => array($parent_key, $item->key),
    );

    archibald_content_overview_form_facets2form_recursiv(
      $form[$item->key],
      $item->vals,
      $parent_key
    );
  }
}

/**
 * theme the search form
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_content_overview_form($form) {
  $form = reset($form);
  unset($form['query']['#title']);
  $html = '';
  $html .= '<div>';
  $html .= theme('textfield', $form['query']) . drupal_render($form['search']);
  $html .= '</div>';

  foreach (element_children($form) as $key) {
    if (drupal_substr($key, 0, 6) == 'facet_') {
      $html .= '<div class="details">';
      $html .= ' <p><a href="JavaScript:void(0);" id="' . $key . '" class="btndetails inactive remember_choice sorl_search_facet">' . '<span>' . t($form[$key]['#title']) . '</span></a></p>';
      $html .= ' <div style="display: block;" class="detailscontainer recusrivcontainer">';
      $html .= '  <ul class="' . $key . '">';
      $html .= theme_archibald_content_overview_facet_form($form[$key]);
      $html .= '  </ul>';
      $html .= '  <div class="clear"></div>';
      $html .= ' </div>';
      $html .= '</div>';
    }
  }

  return $html;
}

/**
 * not an offical theme functions, it ill called direcktly.
 * Here we render the filter checkboxes
 *
 * @param array $form
 * @param integer $sum_count
 *
 * @return string html
 */
function theme_archibald_content_overview_facet_form($form, &$sum_count = 0) {
  $content = '';
  $found_enabled = FALSE;
  foreach (element_children($form) as $checkbox_key) {
    $found_enabled_child = FALSE;

    // this nee ned to find out if a child have any matches
    $child_count = 0;
    $tmp_content = theme_archibald_content_overview_facet_form(
      $form[$checkbox_key],
      $child_count
    );

    // a child is enabled but all have not counts
    if ($child_count == -1) {
      $found_enabled       = TRUE;
      $found_enabled_child = TRUE;
      $child_count         = 0;
    }

    if ($form[$checkbox_key]['#default_value'] == TRUE) {
      // current child is enabled
      $found_enabled = TRUE;
    }

    $result_count = (INT) $form[$checkbox_key]['#result_count'];
    $child_count += $result_count;
    $sum_count += $child_count;

    // if current has 0 relts but its child ,get the result sum of child
    // work around for lerning resurce type
    if (empty($result_count) && !empty($child_count)) {
      $result_count = $child_count;
      $form[$checkbox_key]['#disabled'] = TRUE;
    }

    // append count to title
    $form[$checkbox_key]['#title'] .= ' (' . $result_count . ')';
    $form[$checkbox_key]['#attributes']['class'][] = 'sorl_search_facet_select';

    /* display item if:
     *  item or child have facet hits
     * OR
     *  item or child is activ (checkbox was checked)
     */

    if (!empty($child_count) ||
      $form[$checkbox_key]['#default_value'] == TRUE ||
      $found_enabled_child == TRUE
    ) {
      $content .= '<li>';
      if (!empty($form[$checkbox_key]['#disabled'])) {
        // only show title not the text box if it is disabled
        $content .= $form[$checkbox_key]['#title'];
      }
      else {
        $content .= drupal_render($form[$checkbox_key]);
      }

      if (!empty($tmp_content)) {
        $content .= '<ul>' . $tmp_content . '</ul>';
      }
      $content .= '</li>';
    }
  }

  // if a child is enabled but all have no facet hits
  // set to -1 to notify parent instance to let this tree open
  if ($found_enabled == TRUE && $sum_count < 1) {
    $sum_count = -1;
  }

  return $content;
}

