<?php

/**
 * @file
 * here in are all thing which are shows some content
 * and are to small to have an own file / folder
 */

/**
 * display lom object
 *
 * @param string $lom_id
 * @param string $version_id
 */
function archibald_content_view($lom_id, $version_id = '') {
  global $language;

  if (empty($version_id)) {
    // get local published version if no version was given
    $status = archibald_get_lom_stats($lom_id, TRUE);
    if (!empty($status->local_published)) {
      $version_id = $status->local_published;
    }
  }

  $lom = archibald_load($lom_id, $version_id);

  if (empty($lom)) {
    return t('No description available');
  }
  $additional          = array();
  $additional['first'] = array();
  $res                 = db_query('
    SELECT save_time, uid
    FROM {archibald_lom}
    WHERE lom_id=:lom_id
    ORDER BY save_time ASC
    LIMIT 1', array(
      'lom_id' => $lom_id,
    )
  );
  $return = $res->fetchAssoc();

  $additional['first']['save_time'] = date("d.m.Y H:i", $return['save_time']);
  $res_user = db_query(
    'SELECT name FROM {users} WHERE uid=:uid',
    array(
      'uid' => $return['uid'],
    )
  );
  $additional['first']['username'] = $res_user->fetchColumn(0);

  $res = db_query(
    'SELECT save_time FROM {archibald_lom} WHERE lom_id=:lom_id ORDER BY save_time DESC',
    array(
      'lom_id' => $lom_id,
    )
  );
  $additional['last'] = array(
    'save_time' => date('d.m.Y H:i', $res->fetchColumn(0)),
  );

  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
  drupal_add_js(
    array(
      'archibald' => array(
        'lom' => array(
          'lom_id' => $lom->getLomId(),
          'version' => $lom->getVersion(),
        ),
      ),
    ), 'setting'
  );

  $lom_general = $lom->getGeneral();
  $lang_string = $lom_general->getTitle();
  if ($lang_string instanceof ArchibaldLomDataLangString) {
    $title = ArchibaldLom::determinTitle($lang_string, $language->language);
    drupal_set_title(filter_xss($title));
  }

  return theme(
    'archibald_content_view',
    array(
      'lom' => $lom,
      'additional' => $additional,
    )
  );
}

/**
 * returns a list with all revisions listed for given lom ressource
 *
 * @param string $lom_id the lom_id
 *
 * @return string html
 */
function archibald_revision_overview($lom_id) {
  global $language;

  drupal_set_title(t('Revisions'));

  $table = array(
    'header' => array(
      t('Date - Time'),
      t('Title of description'),
      t('Published'),
      t('User'),
      t('Options'),
    ),
    'rows' => array(),
  );

  $sql_result = db_query('
    SELECT archibaldl.version, archibaldg.title, archibaldl.save_time, usr.name as username
    FROM archibald_lom as archibaldl
    JOIN archibald_general as archibaldg ON (archibaldl.general_id = archibaldg.general_id)
    JOIN users as usr ON (archibaldl.uid = usr.uid)
    WHERE archibaldl.lom_id = :lom_id
     ORDER BY save_time DESC', array(
      ':lom_id' => $lom_id,
    )
  );

  while ($row = $sql_result->fetchAssoc()) {
    $lom_langstring = ArchibaldLomSaveHandler::requestLangStringObj($row['title']);

    $published = '-';

    switch (archibald_lom_is_published($lom_id, $row['version'])) {
      case ARCHIBALD_PUBLISH_STATUS_LOCAL:
        $published = t('Local');
        break;
      case ARCHIBALD_PUBLISH_STATUS_CENTRAL:
        $published = t('National');
        break;
      case ARCHIBALD_PUBLISH_STATUS_LOCAL_CENTRAL:
        $published = t('Local and national');
        break;
    }

    $links = array();
    $links[] = l(t('View'), 'archibald/' . $lom_id . '/revisions/' . $row['version'] . '/view');
    $links[] = l(t('Edit'), 'archibald/' . $lom_id . '/edit/empty/' . $row['version']);

    $row = array(
      format_date($row['save_time'], 'short'),
      ArchibaldLom::determinTitle($lom_langstring, $language->language),
      $published,
      $row['username'],
      implode(' | ', $links),
    );
    $table['rows'][] = $row;
  }

  return theme('table', $table);
}

/**
 * generate form to add / edit archibald resource
 *
 * @param string $lom_id
 * @param string $language
 * @param string $version
 *
 * @return string html
 */
function archibald_content_edit($lom_id = 'add', $language = '', $version = '', $copy_version = '') {

  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');
  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald_form.css');


  drupal_add_js(
    array(
      'archibald' =>
      array(
        'urls' => array(
          'classification' => url('admin/archibald/classification'),
          'contributer_list' => url('archibald_contributer/chooser/ajax'),
          'contributer_preview' => url('archibald_contributer/preview/ajax'),
          'relation_list' => url('admin/archibald/relation/list/ajax'),
        ),
      ),
    ), 'setting'
  );

  ctools_include('ajax');
  ctools_include('modal');

  // Add CTools' javascript to the page.
  ctools_modal_add_js();

  $title = t('Add new description');
  if ($language == 'empty') {
    $language = '';
  }

  $lang_list = array();
  $languages = language_list('enabled');
  if (!empty($lom_id) && drupal_strtolower($lom_id) != 'add') {
    $lom = archibald_load($lom_id, $version, FALSE);
    if (empty($lom)) {
      drupal_set_message(t("No description available"), 'error');
      return "";
    }
    // try to load original language from meta meta data record
    if (empty($language)) {
      $meta_metadata = $lom->getMetaMetadata();
      if ($meta_metadata instanceof ArchibaldLomDataMetaMetadata) {
        $language = $meta_metadata->getLanguage();
      }
    }

    if (empty($language)) {
      // if metametadata.language is not set,
      // try to get it from title langstring and
      // set it to metametadata.language if it is unique

      $possible_languages = array_keys(
        $lom->getGeneral()->getTitle()->getStrings()
      );

      if (count($possible_languages) == 1) {
        $language = reset($possible_languages);
        $meta_metadata->setLanguage($language);
        $lom->setMetaMetadata($meta_metadata);
      }
      elseif (count($possible_languages) > 1) {
        // display language chooser to user
        $lang_list = array();
        foreach ($possible_languages as $lang) {
          $lang_list[] = $languages[1][$lang];
        }
      }
      else {
        // ooops here goes something horrible wrong,
        // show all languages to user
        $lang_list = array();
        foreach ($languages[1] as $lang) {
          $lang_list[] = $lang;
        }
      }
    }

    $lom_general = $lom->getGeneral();
    $lang_string = $lom_general->getTitle();
    if ($lang_string instanceof ArchibaldLomDataLangString) {
      $lom_title = ArchibaldLom::determinTitle($lang_string, $language);
      $title = t(
        'Edit "!title" (!language)',
        array(
          '!title' => $lom_title,
          '!language' => $language,
        )
      );
    }
  }
  else {

    if (!empty($version)) {
      // use an existing lom object as tmeplete to creating a new one
      $preload_lomid = $version;
      $lom           = archibald_load($preload_lomid, $copy_version, FALSE);
      $lom->lom_id   = '';
      $lom->version  = '';
      // try to load original language from meta meta data record
      if (empty($language)) {
        $meta_metadata = $lom->getMetaMetadata();
        if ($meta_metadata instanceof ArchibaldLomDataMetaMetadata) {
          $language = $meta_metadata->getLanguage();
        }
      }

      if (empty($language)) {
        foreach ($lom->general->getLanguage() as $lang) {
          $lang_list[] = $languages[1][$lang];
        }
      }

      $lom_general = $lom->getGeneral();
      $lang_string = $lom_general->getTitle();
      if ($lang_string instanceof ArchibaldLomDataLangString) {
        $lom_title = ArchibaldLom::determinTitle($lang_string, $language);
        $title = t(
          'Add "!title" (!language)',
          array(
            '!title' => $lom_title,
            '!language' => $language,
          )
        );
      }

      /// unset title
      $lom_general->setTitle(NULL);
      $lom->setGeneral($lom_general);
    }
    else {
      $lom = new ArchibaldLom();
    }

    $lang_list = array();
    foreach ($languages[1] as $lang) {
      $lang_list[] = $lang;
    }
  }

  if (empty($language)) {
    if (count($lang_list) == 1) {
      $language = $lang_list[0]->language;
    }
    else {
      drupal_set_title(t('Choose default language for descriptions'));
      return theme(
        'archibald_content_choose_language',
        array(
          'lang_list' => $lang_list,
          'lom_id' => $lom_id,
        )
      );
    }
  }

  drupal_set_title($title);

  return drupal_get_form('archibald_content_form', $lom, $language);
}

/**
 * delete (mark as deleted) a lom object
 *
 * @param string $lom_id
 */
function archibald_content_delete($lom_id) {
  $lom = archibald_load($lom_id);
  $title = ArchibaldLom::determinTitle($lom);

  $status = archibald_get_lom_stats($lom_id);

  if (!empty($status->publication_version) && $status->publication_log->lom_status == 'success') {
    drupal_set_title(filter_xss($title));
    return t('This description cannot be deleted, unpublish first.');
  }

  return drupal_render(drupal_get_form('archibald_content_delete_form', $lom_id, $title));
}

/**
 * delete formular for archibald resources
 *
 * @param array $form
 * @param array $form_state
 * @param string $title
 *   title of resource which is to delete
 *
 * @return array
 *   form
 */
function archibald_content_delete_form($form, &$form_state, $lom_id, $title) {
  $form['ressource'] = array(
    '#prefix' => '<ul>',
    '#suffix' => '</ul>',
    '#tree' => TRUE,
  );
  $form['ressource'][$lom_id] = array(
    '#type' => 'hidden',
    '#value' => $lom_id,
    '#prefix' => '<li>',
    '#suffix' => check_plain($title) . "</li>\n",
  );
  $form['operation'] = array('#type' => 'hidden', '#value' => 'delete');
  $form['#submit'][] = 'archibald_content_delete_form_confirm_submit';

  return confirm_form(
    $form, t('Are you sure you want to delete this description?'), 'admin/archibald', t('This action cannot be undone.'), t('Yes'), t('No')
  );
}

/**
 * Delete one or multiple lom ressources after confimation
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_content_delete_form_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    foreach (array_keys($form_state['values']['ressource']) as $lom_id) {
      $lom = archibald_load($lom_id);
      ArchibaldLomSaveHandler::delete($lom);
      $title = ArchibaldLom::determinTitle($lom);
      watchdog(
        'Content', 'Description "!title" has been deleted',
        array(
          '!title' => $title,
          '@lom_id' => $lom_id,
        ), WATCHDOG_NOTICE, 'archibald/' . $lom_id
      );

      drupal_set_message(
        t(
          'Description "!title" has been deleted',
          array(
            '!title' => $title,
            '@lom_id' => $lom_id,
          )
        )
      );
    }
  }
  $form_state['redirect'] = 'admin/archibald';
}

/**
 * the language chooser
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_content_choose_language($variables) {
  if (!empty($variables['lom_id']) && $variables['lom_id'] != 'add') {
    $type = 'edit';
    $output = '<b>' . t('Choose the language in which you want to edit this description') . '</b><br />';
  }
  else {
    $type = 'add';
    $output = '<b>' . t('Choose the language in which you want to add a new description') . '</b><br />';
  }

  foreach ($variables['lang_list'] as $lang) {
    $link_params = array();
    if (variable_get('archibald_switch_language_while_add', 1) == 1) {
      $link_params = array(
        'language' => $lang
      );
    }

    if ($type == 'add') {
      $url = 'archibald/add/' . $lang->language;
    }
    else {
      $url = 'archibald/' . $variables['lom_id'] . '/edit/' . $lang->language;
    }
    $output .= l(t($lang->name), $url, $link_params) . '<br />';
  }

  return $output;
}

/**
 * display the content overview and perform the search
 *
 * @param string $type
 *   type to display
 *   unavailable
 *   own
 *   draft
 *   published
 *   published_local
 *   to_publish_local
 *   to_publish
 * @param $get_only_table boolean
 *   for ajax, set it to TRUE, to get only content
 *
 * @return string
 */
function archibald_admin_content_overview($type = 'all', $get_only_table = FALSE) {
  global $user;

  drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');

  // Include the CTools tools that we need.
  ctools_include('ajax');
  ctools_include('modal');

  // Add CTools' javascript to the page.
  ctools_modal_add_js();

  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
  drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald_admin_content_overview.js');

  if (isset($_POST['query'])) {
    archibald_admin_content_overview_form_submit();
  }

  $form_params = array(
    'show_status_filter' => FALSE,
    'show_own_filter' => TRUE,
  );

  $publish_type = variable_get("archibald_publish_type", 0);
  $search = new ArchibaldLomSQLSearch();
  $is_published = FALSE;
  switch ($type) {
    case 'unavailable':
      $search->setStatusFilter('unavailable');
      break;

    case 'own':
      $search->setResponsibleFilter($user->uid);
      $search->setLocalPublishedFilter(FALSE);
      break;

    case 'draft':
      $search->setStatusFilter('draft');
      break;

    case 'published':
      $search->addWasPublishedFilter();
      $is_published = TRUE;
      break;

    case 'published_local':
      $search->setLocalPublishedFilter(TRUE);
      $is_published = TRUE;
      break;

    case 'to_publish_local':
      $search->setLocalPublishedFilter(FALSE);
      break;

    case 'to_publish':
      $search->addCanPublishedFilter();
      $search->setStatusFilter('final');
      // $search->setLocalPublishedFilter(TRUE);
      break;

    default:
      if (!empty($_SESSION['archibald_search_status'])) {
        if ($_SESSION['archibald_search_status'] == 'deleted') {
          $search->showDeleted();
        }
        else {
          $search->setStatusFilter(archibald_select_options_default_value_filter_splitter($_SESSION['archibald_search_status']));
        }
      }
      $form_params['show_status_filter'] = TRUE;
      break;
  }

  if (!empty($_SESSION['archibald_search_own'])) {
    $search->setResponsibleFilter($user->uid);
  }
  $search_form = drupal_get_form('archibald_admin_content_overview_form', $form_params);


  $num_per_page = variable_get('archibald_admin_content_num_per_page', 25);

  $search->setLimit($num_per_page);
  if (!empty($_SESSION['archibald_search_query'])) {
    $search->setQuery($_SESSION['archibald_search_query']);
  }
  $page = pager_default_initialize($search->getTotalRecordCount(), $num_per_page);
  $search->setOffset($page * $num_per_page);

  return theme('archibald_admin_content_overview', array(
    'published' => $is_published,
    'search_result' => $search->search(),
    'search_form' => $search_form,
    'search_result_count' => $search->getTotalRecordCount(),
    'type' => $type,
    'get_only_table' => (!empty($get_only_table)) ? 1 : 0,
  ));
}

/**
 * admin search formular for archibald resources
 *
 * @param array $form
 * @param array $form_state
 * @param array $form_params
 *
 * @return array
 */
function archibald_admin_content_overview_form($form, &$form_state, $form_params) {

  $query = '';
  if (!empty($_SESSION['archibald_search_query'])) {
    $query = $_SESSION['archibald_search_query'];
  }

  $form['query'] = array(
    '#type' => 'textfield',
    '#title' => t('Term'),
    '#default_value' => $query,
    '#size' => 40,
  );

  if ($form_params['show_own_filter'] == TRUE) {
    $archibald_search_own = '';
    if (!empty($_SESSION['archibald_search_own'])) {
      $archibald_search_own = $_SESSION['archibald_search_own'];
    }

    $form['own'] = array(
      '#type' => 'checkbox',
      '#title' => t('Description owner'),
      '#default_value' => $archibald_search_own,
    );
  }

  if ($form_params['show_status_filter'] == TRUE) {
    $archibald_search_status = '';
    if (!empty($_SESSION['archibald_search_status'])) {
      $archibald_search_status = $_SESSION['archibald_search_status'];
    }

    $form['status'] = array(
      '#type' => 'select',
      '#title' => t('Meta data status'),
      '#default_value' => $archibald_search_status,
      '#options' =>
      array('' => t('All')) + archibald_get_life_cycle_status_options() + array('deleted' => t('Deleted')),
    );
  }

  $form['search'] = array(
    '#type' => 'submit',
    '#value' => t('Search'),
    '#weight' => 9999,
  );
  $form['#theme'] = 'archibald_admin_content_overview_form';

  return $form;
}

/**
 * submit handler for
 * archibald_admin_content_overview_form()
 */
function archibald_admin_content_overview_form_submit() {
  $_SESSION['archibald_search_own'] = '';
  if (!empty($_POST['own'])) {
    $_SESSION['archibald_search_own'] = filter_xss($_POST['own']);
  }

  $_SESSION['archibald_search_status'] = '';
  if (!empty($_POST['status'])) {
    $_SESSION['archibald_search_status'] = filter_xss($_POST['status']);
  }

  $_SESSION['archibald_search_query'] = '';
  if (!empty($_POST['query'])) {
    $_SESSION['archibald_search_query'] = filter_xss($_POST['query']);
  }
}

/**
 * theme archibald content overview
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_admin_content_overview($variables) {
  global $user;
  global $language;

  $content_partners = archibald_load_content_partner();

  $html = '<input type="hidden" id="archibald_admin_content_overview_type" value="' . $variables['type'] . '" />';

  $action_links = "";
  if (archibald_content_add_access()) {
    $action_links .= '<div id="action_link_add">' . l(t('Add'), 'archibald/add') . '</div>';
  }

  if (user_access('edit archibald content')) {
    $row = db_query_range("SELECT 1 FROM {archibald_lom_stats} WHERE new_revision_required = 1", 0, 1)->fetchAssoc();
    if (!empty($row)) {
      $action_links .= '<div><span class="new_revision_required" url="' . url('archibald/finish_modifications') . '">' . t('Some descriptions need a revision.') . '</span></div>';
    }
  }

  if (!empty($action_links)) {
    $html .= '<div id="archibald_admin_content_overview_action_links">' . $action_links . '</div>';
  }

  $html .= drupal_render($variables['search_form']);
  $html .= '<div class="clearfix"></div>';

  $table = array(
    'header' =>
    array(
      t('Link to description'),
      array(
        'data' => t('Status'),
        'colspan' => 2,
      ),
      t('Person responsible'),
    ),
    'rows' => array(),
    'attributes' => array('id' => 'archibald_admin_content_overview'),
  );

  $is_editor = (
    user_access('edit archibald content') ||
    user_access('delete archibald content') ||
    user_access('publish archibald content local') ||
    user_access('publish archibald content central')
  );

  if ($is_editor) {
    if (count($content_partners) > 1) {
      $table['header'][] = t('Content partner');
    }

    $table['header'][] = t('Options');
  }

  // generate table from result set
  foreach ($variables['search_result'] as $res_row) {
    $row = array(0 => '');

    if (empty($res_row->status)) {
      $res_row->status = 'unavailable';
    }

    // Determine title.
    $row[0] = l(
      ArchibaldLom::determinTitle($res_row->title, $res_row->default_language),
      'archibald/' . $res_row->lom_id . '/revisions/' . $res_row->version . '/view'
    );
    $in_translation = "";
    if (!empty($res_row->new_revision_required)) {
      $in_translation = '(' . t("New revision required") . ')';
      if (user_access('edit archibald content')) {
          ctools_include('ajax');
          ctools_include('modal');
          // Add CTools' javascript to the page.
          ctools_modal_add_js();
          drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');
          drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
          $in_translation = '<span class="new_revision_required" url="' . url('archibald/finish_modifications') . '">' . $in_translation . '</span>';
      }

    }


    $lom_status = archibald_get_term_by_key($res_row->status, 'life_cycle_status');
    if ( user_access('change archibald content status') ) {
      $row[] = '<span class="lifecycle_status" lom_id="' . $res_row->lom_id . '" status="' . $res_row->status . '" url="' . url('archibald/' . $res_row->lom_id . '/set_status') . '">' . $lom_status . '</span>' . $in_translation;
    }
    else {
      $row[] = $lom_status;
    }


    $row[] = theme(
      'archibald_admin_content_publish_status',
      array(
        'res_row' => $res_row,
      )
    );

    $row[] = theme(
      'archibald_admin_content_overview_reponsible',
      array(
        'uid' => $res_row->responsible_uid,
        'name' => $res_row->user_name,
        'lom_id' => $res_row->lom_id,
      )
    );

    // if user is allowed to edit show options for this
    if ($is_editor) {
      if (count($content_partners) > 1) {
        if (empty($content_partners[$res_row->content_partner_id])) {
          $cp_name = t('Not set');
        }
        else {
          $cp_name = $content_partners[$res_row->content_partner_id]['name'];
        }

        if (!empty($res_row->publication_version) || !user_access('change archibald content partner')) {
          // if user has permission und ressource is not published
          $cp_class = 'content_partner_fixed';
        }
        else {
          $cp_class = 'content_partner';
        }

        $row[] = array(
          'data' => '<span class="content_partner" url="' . url('archibald/' . $res_row->lom_id . '/set_content_partner') . '">' . $cp_name . '</span>',
          'class' => array($cp_class),
        );
      }

      $options = array();
      if (user_access('edit archibald content')) {
        $edit_label = t('Edit');
        $options[] = l(
          $edit_label,
          'archibald/' . $res_row->lom_id . '/edit',
          array(
            'attributes' => array(
              'type' => 'edit',
            ),
          )
        );

        $options[] = l(
          t('Revisions'),
          'archibald/' . $res_row->lom_id . '/revisions',
          array(
            'attributes' => array(
              'type' => 'revisions',
            ),
          )
        );
      }

      if (user_access('translate archibald content')) {
        $options[] = l(
          t('Translate'),
          'archibald/' . $res_row->lom_id . '/translate',
          array(
            'attributes' => array(
              'type' => 'translate',
            ),
          )
        );
      }
      if (user_access('delete archibald content')) {
        $options[] = l(
          t('Delete'),
          'archibald/' . $res_row->lom_id . '/delete',
          array(
            'attributes' => array(
              'type' => 'delete',
            ),
          )
        );
      }

      if (
        archibald_check_publish_access() &&
        $res_row->status == 'final'
      ) {
        /*
         *  Dont need to check here for can check central this will done later
         *  and we like to show the button all times, because we like to publish local
         *   !empty($content_partners[$res_row->content_partner_id]) &&
         *   !empty($content_partners[$res_row->content_partner_id]['key_fid']
         */

        $options[] = l(
          t('Publish'),
          'archibald/' . $res_row->lom_id . '/publish',
          array(
            'attributes' => array(
              'type' => 'publish',
              'class' => 'publish_link',
            ),
          )
        );
      }

      if (user_access('edit archibald content')) {
        $options[] = l(
          t('Publication log'),
          'archibald/' . $res_row->lom_id . '/log',
          array(
            'attributes' => array(
              'type' => 'log',
            ),
          )
        );
      }
      if (user_access('add archibald content')) {
        $options[] = l(
          t('Use as template'),
          'archibald/add/' . $res_row->default_language . '/' . $res_row->lom_id,
          array(
            'attributes' => array(
              'type' => 'template',
            ),
          )
        );
      }

      $row[] = array(
        'data' => implode(' | ', $options),
        'class' => array('options_links'),
      );
    }

    $table['rows'][] = $row;
  }

  if (!empty($variables['get_only_table'])) {
    return theme('table', $table);
  }
  else {
    $html .= theme('table', $table);
  }

  $html .= theme('pager');

  return $html;
}

/**
 * theme the search form
 *
 * @param array $form
 *
 * @return string
 */
function theme_archibald_admin_content_overview_form($form) {
  $form = reset($form);
  unset($form['query']['#title']);

  $html = '<div id="archibald_admin_content_overview_search_form">';
  $html .= '<div>' . theme('textfield', $form['query']) . '</div>';

  if (!empty($form['own'])) {
    $html .= '<div>' . t('My descriptions') . ' ' . theme('checkbox', $form['own']) . '</div>';
  }

  if (!empty($form['status'])) {
    $html .= '<div>' . theme('select', $form['status']) . '</div>';
  }

  $html .= '<div>' . drupal_render($form['search']) . '</div>';
  $html .= '</div>';
  return $html;
}

/**
 * get array of lifecycle.status options ready for use im form api #type=select
 *
 * @return array
 */
function archibald_get_life_cycle_status_options() {
  return archibald_get_taxonomy_options('life_cycle_status');
}

/**
 * return a list of users with can edit archibald content
 * @global object $user current logged in user
 *
 * @return array
 */
function archibald_get_all_user_editor() {
  $required_user_access = 'edit archibald content';

  $sql = 'SELECT u.uid, u.name FROM {users} u
    LEFT JOIN {users_roles} ur ON (u.uid=ur.uid)
    LEFT JOIN {role_permission} p ON (ur.rid=p.rid)
    WHERE p.permission = :required_user_access';
  // LEFT JOIN {role} r ON (ur.rid=r.rid)

  $users = array();
  $result = db_query(
    $sql, array(
      ':required_user_access' => $required_user_access,
    )
  );

  foreach ($result as $account) {
    $users[$account->uid] = $account->name;
  }
  return $users;
}

/**
 * theme the responsible link item
 *
 * @param array $variables
 *
 * @return string
 */
function theme_archibald_admin_content_overview_reponsible($variables) {
  global $user;

  $span = '<span class="responsible_editor" lom_id="' . $variables['lom_id'] . '" url="' .
    url('archibald/' . $variables['lom_id'] . '/set_new_responsible') .
    '" title=\'' . t('Assign to another editor') . '\'>';

  if (!empty($variables['uid']) && $variables['uid'] == $user->uid) {
    return $span . t('Me') . '</span>';
  }
  else {
    $account = (object)array(
      'uid' => $variables['uid'],
      'name' => $variables['name'],
    );

    $attributes_array = array();
    if (empty($variables['uid'])) {
      return $span . format_username($account) . '</span>';
    }
    else {
      return '<span>' . format_username($account) . '</span>';
    }
  }
}

/**
 * Menu callback; displays a listing of log messages.
 *
 * Messages are truncated at 56 chars. Full-length message could be viewed at
 * the message details page.
 *
 * @param string $lom_id
 * @param string $type
 *   comma seperated list of watchdog types
 *   example:
 *    file_upload
 *    file_replacing
 *    lom_upload
 *    lom_auth
 *    solr_index
 *    comment add
 *    comment delete
 *    .....
 */
function archibald_dblog_overview($lom_id, $type = '') {
  $rows = array();
  $classes = array(
    WATCHDOG_DEBUG => 'dblog-debug',
    WATCHDOG_INFO => 'dblog-info',
    WATCHDOG_NOTICE => 'dblog-notice',
    WATCHDOG_WARNING => 'dblog-warning',
    WATCHDOG_ERROR => 'dblog-error',
    WATCHDOG_CRITICAL => 'dblog-critical',
    WATCHDOG_ALERT => 'dblog-alert',
    WATCHDOG_EMERGENCY => 'dblog-emerg',
  );

  $filter = array(
    'where' => 'link = :link',
    'args' => array(
      ':link' => 'archibald/' . $lom_id,
    ),
  );

  if (!empty($type)) {
    $type = explode(',', $type);
    $filter['where'] .= " AND type IN ('" . implode("', '", $type) . "')";
  }

  $rows[] = array(
    t('Date - Time'),
    t('Message'),
    t('User'),
  );

  $query = db_select('watchdog', 'w')->extend('PagerDefault')->extend('TableSort');

  $query->leftJoin('users', 'u', 'w.uid = u.uid');
  $query->fields(
    'w', array(
      'wid',
      'uid',
      'severity',
      'type',
      'timestamp',
      'message',
      'variables',
      'link',
    )
  )->addField('u', 'name');

  if (!empty($filter['where'])) {
    $query->where($filter['where'], $filter['args']);
  }
  $result = $query->limit(50)->execute();

  foreach ($result as $dblog) {
    $rows[] = array('data' =>
      array(
        // Cells
        format_date($dblog->timestamp, 'short'),
        theme(
          'archibald_dblog_message',
          array(
            'event' => $dblog,
            'link' => TRUE,
          )
        ),
        theme('username', array('account' => $dblog)),
      ),
      // Attributes for tr
      'class' => array(
        drupal_html_class('dblog-' . $dblog->type),
        $classes[$dblog->severity],
      ),
    );
  }

  $build['dblog_table'] = array(
    '#theme' => 'table',
    '#rows' => $rows,
    '#attributes' => array('id' => 'admin-dblog'),
    '#empty' => t('No log messages available.')
  );
  $build['dblog_pager'] = array('#theme' => 'pager');

  return $build;
}

/**
 * Generates a diff between two lom objects.
 *
 * @param string $lom_id
 *   The lom id
 * @param string $revision_a
 *   The old revision
 * @param string $revision_b
 *   The new revision, if an empty string is provided (default) the last
 *   revision will be used
 *   (optiona, default = '')
 */
function archibald_lom_diff($lom_id, $revision_a, $revision_b = "") {
  $json_a = json_decode(ArchibaldLomSaveHandler::jsonExport(ArchibaldLomSaveHandler::load($lom_id, $revision_a)), TRUE);
  $json_b = json_decode(ArchibaldLomSaveHandler::jsonExport(ArchibaldLomSaveHandler::load($lom_id, $revision_b)), TRUE);

  $diff = array();
  archibald_lom_diff_recrusive($json_a, $json_b, $diff);

}

/**
 * Recrusive function which found out the diff's for the 2 lom resource arrays.
 *
 * @param array $lom_a
 *   The first lom object
 * @param array $lom_b
 *   The second lom object
 */
function archibald_lom_diff_recrusive(array $lom_a, array $lom_b, &$diff, $key = "") {
  foreach ($lom_a AS $k => $v) {
    if (!empty($key)) {
      $new_key = $key . "." . $k;
    }
    else {
      $new_key = $k;
    }

    if (!isset($lom_b[$k])) {
      $diff[$new_key] = t("deleted entry: " . $k);
      continue;
    }

    if (is_array($v)) {
      archibald_lom_diff_recrusive($lom_a[$k], $lom_b[$k], $diff, $new_key);
      continue;
    }

    if ($lom_a[$k] != $lom_b[$k]) {
      $diff[$new_key] = t("@key changed from @a to @b", array(
        '@key' => $new_key,
        '@a' => $lom_a[$k],
        '@b' => $lom_b[$k],
      ));
    }

    unset($lom_b[$k]);
  }

  foreach ($lom_b AS $k => $v) {
    if (!empty($key)) {
      $new_key = $key . "." . $k;
    }
    else {
      $new_key = $k;
    }
    if (is_array($v)) {
      archibald_lom_diff_recrusive(array(), $v, $diff, $new_key);
      continue;
    }
    $diff[$new_key] = t("New value for @key added: @val", array(
      '@key' => $new_key,
      '@val' => $v,
    ));
  }
}

/**
 * Returns HTML for a log message.
 *
 * @param $variables
 *   An associative array containing:
 *   - event: An object with at least the message and variables properties.
 *   - link: (optional) Format message as link, event->wid is required.
 *
 * @ingroup themeable
 */
function theme_archibald_dblog_message($variables) {
  $output = '';
  #print_r($event);
  $event = $variables['event'];
  // Check for required properties.
  if (isset($event->message) && isset($event->variables)) {
    // Messages without variables or user specified text.
    if ($event->variables === 'N;') {
      $output = $event->message;
    }
    // Message to translate with injected variables.
    else {
      $output = t($event->message, unserialize($event->variables));
    }
    if ($variables['link'] && isset($event->wid)) {
      // Truncate message to 56 chars.
      $output = truncate_utf8(
        filter_xss(
          $output,
          array()
        ),
        95,
        TRUE,
        TRUE
      );

      $output = l(
        $output,
        'admin/reports/event/' . $event->wid,
        array(
          'html' => TRUE,
        )
      );
    }
  }
  return $output;
}

