<?php

/**
 * @file
 * this file include the translatin interface for lang strings
 */

/**
 * menu callback for translating a ressource
 * @global object $language
 *
 * @param string $lom_id
 *
 * @return string html
 */
function archibald_translate_wizard($lom_id) {
  global $language;
  $_SESSION['langStringOrder'] = array();
  $lom = ArchibaldLomSaveHandler::load($lom_id);
  if (!is_object($lom)) {
    return '';
  }
  $output = "";
  drupal_set_title(t('Translate'));

  $translations = array();


  $translations['general.title'] = $lom->getGeneral()->getTitle();

  $translations['general.description'] = $lom->getGeneral()->getDescription();

  $translations['education.description'] = $lom->getEducation()->getDescription();

  $x = 0;
  foreach ($lom->getGeneral()->getIdentifier() as $i => $identifier) {
    if (empty($identifier)) {
      continue;
    }

    $lang_string_obj = $identifier->getTitle();
    if (empty($lang_string_obj)) {
      continue;
    }
    $translations['general.identifier.title ' . ++$x] = $lang_string_obj;
  }

  foreach ($lom->getGeneral()->getKeyword() as $i => $lang_string_obj) {
    if (empty($lang_string_obj)) {
      continue;
    }
    $i = intval($i);
    $translations['general.keyword ' . ++$i] = $lang_string_obj;
  }

  foreach ($lom->getGeneral()->getCoverage() as $i => $lang_string_obj) {
    if (empty($lang_string_obj)) {
      continue;
    }
    $i = intval($i);
    $translations['general.coverage ' . ++$i] = $lang_string_obj;
  }




  $pref = '';

  $table = array(
    'header' => array(
      t('Fields'),
      t('Translate'),
    ),
    'rows' => array(),
  );

  foreach ($translations as $k => $lom_langstring) {
    $_SESSION['langStringOrder'][$pref] = $lom_langstring->id;
    $pref = $lom_langstring->id;

    if (strpos($k, ' ') !== FALSE) {
      $tmp = explode(' ', $k);
      #$field_name = archibald_get_field_label($tmp[0], TRUE) . ' ' . $tmp[1];
      $field_name = archibald_get_field_label($tmp[0], TRUE);
    }
    else {
      $field_name = archibald_get_field_label($k, TRUE);
    }

    $row = array(
      "<span style='font-weight:bold;'>" . $field_name . "</span><br />" . current($lom_langstring->strings),
      l(t('Translate'), 'admin/archibald/translate_term/' . $lom_id . '/' . $pref),
    );
    $table['rows'][] = $row;
  }

  $in_translation = db_query("SELECT 1 FROM {archibald_lom_stats} WHERE lom_id = :lom_id AND new_revision_required = 1", array(
    ':lom_id' => $lom_id
  ))->fetchAssoc();

  if (!empty($in_translation)) {
    ctools_include('ajax');
    ctools_include('modal');
    // Add CTools' javascript to the page.
    ctools_modal_add_js();
    drupal_add_css(drupal_get_path('module', 'archibald') . '/css/archibald.css');
    drupal_add_js(drupal_get_path('module', 'archibald') . '/js/archibald.js');
    $output .= '<span class="new_revision_required" lom_id="' . $lom_id . '" url="' . url('archibald/finish_modifications') . '">' . t('New revision required') . '</span>';
  }
  $output .= theme('table', $table);

  return $output;
}

/**
 * generate the translation form
 *
 * @param array $form
 * @param array $form_state
 * @param string $langstring_terms_id
 *
 * @return array
 */
function archibald_translate_form($form, $form_state, $lom_id, $langstring_terms_id) {

  $languages = archibald_get_general_language_options();

  $lang_string_obj = new ArchibaldLomDataLangString();
  $lang_string_obj->strings = array();
  $lang_string_obj->setId($langstring_terms_id);
  $res = db_select('archibald_langstring_terms', 'archibaldls')
    ->fields('archibaldls')
    ->condition('langstring_terms_id', $langstring_terms_id)
    ->execute();

  while ($row = $res->fetchAssoc()) {
    $lang_string_obj->setString($row['term'], $row['language']);
  }
  if (is_null($lang_string_obj->strings)) {
    $lang_string_obj->strings = array();
  }

  $strings = $lang_string_obj->getStrings();

  $type = 'textfield';
  foreach ($strings as $ls) {
    if (strpos($ls, "\n") !== FALSE || drupal_strlen($ls) > 170) {
      $type = 'textarea';
    }
  }

  $form['#prefix'] = "";
  $form['langstring_terms_id'] = array(
    '#type' => 'hidden',
    '#default_value' => $langstring_terms_id,
  );
  $form['lom_id'] = array(
    '#type' => 'hidden',
    '#default_value' => $lom_id,
  );
  foreach ($languages as $lang_code => $language) {
    $default_value = '';
    if (!empty($strings[$lang_code])) {
      $default_value = $strings[$lang_code];
    }

    $form['language_' . $lang_code] = array(
      '#type' => $type,
      '#title' => filter_xss($language),
      '#default_value' => $default_value,
      '#size' => 130,
      '#maxlength' => ($type == 'textfield') ? 700 : 3500,
      '#rows' => 8,
    );
  }
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 9999,
  );
  $form['save_and_next'] = array(
    '#type' => 'submit',
    '#value' => t('Save and next field'),
    '#weight' => 9999,
  );
  $form['cancel'] = array(
    '#type' => 'submit',
    '#value' => t('Cancel'),
    '#weight' => 9999,
  );
  return $form;
}

/**
 * submit handler for translating form
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_translate_form_submit($form, &$form_state) {
  $version = ArchibaldLomSaveHandler::getActualVersion($form['lom_id']['#value']);
  $cid = 'lom:' . $form['lom_id']['#value'] . '__version:' . $version;
  cache_set($cid, NULL, 'cache', CACHE_TEMPORARY);

  switch ($form_state['clicked_button']['#id']) {
    case 'edit-cancel':
      drupal_goto('archibald/' . $form['lom_id']['#value'] . '/translate');
      break;

    case 'edit-save-and-next':
      $langstring_id = archibald_translate_form_submit_save($form, $form_state);
      if (!empty($_SESSION['langStringOrder'][$langstring_id])) {
        drupal_goto(
          'admin/archibald/translate_term/' . $form['lom_id']['#value'] . '/' .
          $_SESSION['langStringOrder'][$langstring_id]
        );
      }
      else {
        drupal_goto('archibald/' . $form['lom_id']['#value'] . '/translate');
      }
      break;

    case 'edit-save':
      archibald_translate_form_submit_save($form, $form_state);
      drupal_goto('archibald/' . $form['lom_id']['#value'] . '/translate');
      break;
  }
}

/**
 * save a lang string
 *
 * @param array $form
 *
 * @return string
 *    the langstring id
 */
function archibald_translate_form_submit_save(&$form, &$form_state) {
  $lang_string = new ArchibaldLomDataLangString();
  $lang_string->id = $form['langstring_terms_id']['#value'];
  unset($form['langstring_terms_id']);
  foreach ($form_state['values'] as $k => $tmp_arr) {
    if (!preg_match("/^language_([a-z][a-z])$/", $k, $matches)) {
      continue;
    }
    if (!empty($matches[1]) && trim($matches[1]) != '') {
      if (empty($tmp_arr)) {
        $lang_string->strings[$matches[1]] = '';
      }
      else {
        $lang_string->setString($tmp_arr, $matches[1]);
      }
    }
  }

  $old_langstring_object = ArchibaldLomSaveHandler::requestLangStringObj($lang_string->id);
  archibald_handle_new_revision_is_required($lang_string->id, $old_langstring_object, $lang_string);
  archibald_reset_new_revision_is_required(FALSE, TRUE, $form_state['values']['lom_id']);
  return ArchibaldLomSaveHandler::saveOrInsertLangString($lang_string);
}
